/*
 * $Id: args.c,v 1.5 1996/07/29 20:19:20 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 * Written by
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Hannu Helminen   (dm@stekt.oulu.fi)
 *
 * Patches by
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 *
 * $Log: args.c,v $
 * Revision 1.5  1996/07/29 20:19:20  ettore
 * Miscellaneous fixes.
 *
 * Revision 1.4  1996/04/01  09:01:41  jopi
 * *** empty log message ***
 *
 * Revision 1.3  1995/04/01  07:53:25  jopi
 * Added new options: -notraps -brevision.
 *
 * Revision 1.2  1994/12/12  16:59:44  jopi
 * Compability with Xrm resources.
 *
 * Revision 1.1  1994/02/18  15:47:13  jopi
 * Initial revision
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "macro.h"
#include "vmachine.h"
#include "memory.h"
#include "extern.h"


AppResources app_resources;

void arg_parser(int argc, char **argv)
{
    int     errflg = 0;

    app_resources.directory = NULL;
    app_resources.ramName   = RAMNAME;
    app_resources.basicName = BASICNAME;
    app_resources.kernelName= KERNELNAME;
    app_resources.reuName   = NULL;
    app_resources.reloc     = NULL;
    app_resources.basicRev  = NULL;
    app_resources.kernelRev = NULL;
    app_resources.haltflag  = 0;


    while (!errflg && --argc && '-' == (*++argv)[0]) {

	/*
	 * Flags
	 */

	if (!strcmp (*argv, "-asm")) {
	    ++app_resources.asmflag;
	    continue;
	}
	if (!strcmp (*argv, "-debug")) {
	    ++app_resources.debugflag;
	    continue;
	}
	if (!strcmp (*argv, "-hex")) {
	    app_resources.hexflag = !app_resources.hexflag;
	    continue;
	}
	if (!strcmp (*argv, "-test")) {
	    ++app_resources.testflag;
	    continue;
	}
	if (!strcmp (*argv, "-verbose")) {
	    ++app_resources.verboseflag;
	    continue;
	}
	if (!strcmp (*argv, "-notraps")) {
	    ++app_resources.notraps;
	    continue;
	}

	/*
	 * Resources
	 */

	if (!strncmp (*argv, "-dir", 4)) {
	    if(argc > 1) {
		app_resources.directory = argv[1];
		++argv; --argc;
	    }
	    else
		++errflg;
	    continue;
	}

	else if (!strcmp (*argv, "-ram")) {
	    if(argc > 1) {
		app_resources.ramName = argv[1];
		++argv; --argc;
	    }
	    else
		++errflg;
	    continue;
	}

	else if (!strcmp (*argv, "-basic")) {
	    if(argc > 1) {
		app_resources.basicName = argv[1];
		++argv; --argc;
	    }
	    else
		++errflg;
	    continue;
	}

	else if (!strcmp (*argv, "-kernel")) {
	    if(argc > 1) {
		app_resources.kernelName = argv[1];
		++argv; --argc;
	    }
	    else
		++errflg;
	    continue;
	}

	else if (!strcmp (*argv, "-exrom")) {
	    if(argc > 1) {
		app_resources.exromName = argv[1];
		++argv; --argc;
	    }
v	    else
		++errflg;
	    continue;
	}
	else if (!strcmp (*argv, "-game") || !strcmp (*argv, "-module")) {
	    if(argc > 1) {
		app_resources.module = argv[1];
		++argv; --argc;
	    } else
		++errflg;
	    continue;
	}

	/*
	 * Disk Drives
	 */

	if (!strcmp (*argv, "-dosrom")) {	/* disk drive ROM */
	    if(argc > 1) {
		app_resources.dosRomName = argv[1];
		++argv; --argc;
	    continue;
	    }
	    else
		++errflg;
	}
	if (!strcmp (*argv, "-floppy")) {
	    if(argc > 1) {
		app_resources.floppyName = argv[1];
		++argv; --argc;
	    continue;
	    }
	    else
		++errflg;
	}


	/*
	 * Miscellaneous
	 */

	else if (!strcmp (*argv, "-load")) { /* BASIC load */
	    if(argc > 1) {
		app_resources.programName = argv[1];
		app_resources.reloc = CBM_TXT_DEFAULT;
		++argv; --argc;
	    } else
		++errflg;
	    continue;
	}
	else if (!strcmp (*argv, "-bload")) { /* binary load */
	    if(argc > 1) {
		app_resources.programName = argv[1];

		if((argc > 2) && *argv[2] != '-')
		    app_resources.reloc = argv[2];

		++argv; --argc;
	    } else
		++errflg;
	    continue;
	}

#ifdef PATCH_ROM
	else if (!strncmp (*argv, "-rev", 4)) { /* kernel revision */
	    if(argc > 1 && *argv[1] != '-') {
		app_resources.kernelRev = argv[1];
		++argv; --argc;
	    } else
		++errflg;
	    continue;
	}

	else if (!strncmp (*argv, "-brev", 5)) { /* C128 basic revision */
	    if(argc > 1 && *argv[1] != '-') {
		app_resources.basicRev = argv[1];
		++argv; --argc;
	    } else
		++errflg;
	    continue;
	}
#endif
#ifdef REU
	else if (!strcmp (*argv, "-reu")) {
	    if(argc > 1 && *argv[1] != '-') {
		app_resources.reuName = argv[1];
		++argv; --argc;
	    }
	    else
		app_resources.reuName= REUNAME; /* by default: reu */
	    continue;
	}
#endif
	else if (!strcmp (*argv, "-start")) { /* start address */
	    if(argc > 1) {
		app_resources.startAddr = argv[1];
		++argv; --argc;
	    } else
		++errflg;
	    continue;
	}

	else if (!strcmp (*argv, "-halt")) { /* don't start, go to debug win */
	    if(argc > 1) {
		app_resources.haltflag = argv[1];
		++argv; --argc;
	    } else
		++errflg;
	    continue;
	}


	++errflg;
    } /* while */

    if(errflg) {
	(void) fprintf(stderr,
		"\n%s: Bad argument near '%s'.\n", progname, argv[0]);

	(void) fprintf(stderr, "\
\nUsage: %7s  [-asm] [-debug] [-hex] [-test] [-verbose] [-notraps]\n\
\t\t[-directory path] [-ram name] [-basic name] [-kernel name] [...]\n\
\t\t[-exrom exrom] [-module #] %s%s\n\
\t\t[-floppy disk]\n\
\t\t[-load program] [-bload program [address]] [-start address]\n\n",
	     progname,
#ifdef PATCH_ROM
	     "[-revision #] ",
#else
	     "",
#endif
#ifdef REU
	     "[-reu [reufile]] "
#else
	     ""
#endif
	     );
	
	(void) fprintf(stderr, "\
\t-asm\t\tenter assembler/monitor\n\
\t-exrom\t\texromname for -module option\n\
\t-floppy\t\tattach floppy image\n\
\t-debug\t\tdebug info\n\
\t-hex\t\ttoggle decimal/hexadecimal mode (default hex)\n\
\t-notraps\tdisables serial bus trapping\n\
\t-load\t\tload program from filesystem\n\
\t-bload\t\tload program from filesystem\n\
\t-module\t\tload module into slot 1..3\n\
\t-notraps\tdisable serial emulation and traps\n\
\t-directory\tproject directory\n");

	/*"\t-dosrom\tload disk drive ROM for memory-read commands\n");*/
#ifdef PATCH_ROM
	(void) fprintf(stderr, "\t-revision\tconvert loaded Kernal version to another\n");
	(void) fprintf(stderr, "\t-brevision\tpatch loaded Basic version to the other (x128 only)\n");
#endif
#ifdef REU
	(void) fprintf(stderr, "\t-reu\t\tactivate ram expansion using reufile\n");
#endif
	(void) fprintf(stderr, "\
\t-test\t\ttest (monitor)\n\
\t-verbose\tverbose\n\n");
	exit(1);
    }

    return;
}
