/*
 * $Id: config.h,v 1.13 1996/07/30 13:13:13 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 *
 * Written by
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 * Patches by
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 */


/* ------------------------------------------------------------------------- *
 *
 * VICE Configuration Set Up
 * Recommendations for each define are shown after them, so
 * that you can restore the settings.
 *
 * Use `xmkmf -a' to update the Makefile.
 * Use `make clean' if you want to re-compile from scratch using the new
 * configuration.
 *
 *  Tips on Installation:
 *  For DEC, if necessary, enable XCONTROL, and DECMAP
 *  If you do not have libXpm (this happens with many DEC and HP-UX setups,
 *  for example), disable XPM. 
 *
 * ------------------------------------------------------------------------- */


/* DO NOT change this... */
#if (!defined(C128) && !defined(VIC20) &&!defined(PET))
#define CBM64
#endif 

#if !defined (_CONFIG_H)
#define _CONFIG_H

/*
 * Compatibility
 */

/* Does your compile support the `__inline__' keyword?
 *
 * If you have gcc or another compiler which supports it, you should
 * definitely keep this option commented, since this will allow (much)
 * better performance.
 * If your compiler supports function inlining but needs a different
 * keyword, edit the `INLINE' #define in raster.h.
 */

/* #define NO_INLINE */		/* no */


/*
 * Do you have systeminfo.h and sysinfo(2) ?
 */

#if (defined(SUN))
#define SYSINFO			/* yes */
#endif


/*
 * Do you have the Xpm library ?
 */

#define XPM			/* yes */


/* ------------------------------------------------------------------------- */


/*
 * Window Sizes and refresh rates.
 *
 * Select the desired size and refresh rate for the video emulation windows.
 * You can choose whether you want double/triple size for width and/or height
 * and the maximum refresh rate for each video chip emulated.
 *
 * Setting REFRESH_RATE to <x> means that 1/<x> of the real refreshes are done
 * i.e. the higher REFRESH_RATE is the faster (and less accurate) the
 * emulation is. While the window size must be chosen at compilation time, you
 * will be allowed to change this default value at run-time (from the built-in
 * monitor) or with command-line options.
 * IMPORTANT NOTICE: The default value (2) might be too low for slow machines
 * and remote X-Terminals. A value of 3, 4 or even 5 might be more appropriate
 * in such cases.
 *
 * VINTAGE_DISPLAY emulates the look of old monitors (with <300 lines), with
 * tiny black lines between the actual pixels. This will double the vertical
 * size, but the speed would not be as low as if DOUBLE_Y was selected. Notice
 * that you should not enable DOUBLE_Y or TRIPLE_Y when uncommenting this one.
 */


/*
 * First of all, specify whether you want PAL or NTSC.
 * (this is mostly important for the C64 emulator)
 */

#define PAL			/* yes -- we come from Europe :-) */
/*#define NTSC*/		/* no */


/**** VIC20 (xvic) settings ****/

/* (QUAD_X is available for VIC20 only) */

#if defined( VIC20 )

#define REFRESH_RATE 2

#define DOUBLE_X 		/* yes */

/*#define TRIPLE_X*/		/* no */
/*#define DOUBLE_Y*/		/* no */
/*#define QUAD_X*/		/* no */
/*#define TRIPLE_Y*/		/* no */


#elif defined( C128 )

/**** C128 (x128) settings ****/

/* 80 column (VDC) window */
#define VDC_REFRESH_RATE 4
/* 40 column (VIC) window */
#define VIC_REFRESH_RATE 2

/* (the other VIC and VDC settings are separate) */

#if defined( VIC )

/* x128/VIC settings */

/* #define DOUBLE_X */		/* no */
/* #define TRIPLE_X */    	/* no */
/* #define DOUBLE_Y */		/* no */
/* #define TRIPLE_Y */		/* no */
/* #define VINTAGE_DISPLAY */   /* no */

#else

/* x128/VDC settings */

/* #define DOUBLE_X */       	/* no */
/* #define TRIPLE_X */		/* no */
/* #define DOUBLE_Y */		/* no */
/* #define TRIPLE_Y */          /* no */
/* #define VINTAGE_DISPLAY */   /* no */

#endif

#elif defined( PET )

/**** PET (xpet) settings ****/

#define REFRESH_RATE 2

/* #define DOUBLE_X */		/* no */
/* #define TRIPLE_X */		/* no */
/* #define DOUBLE_Y */		/* no */
/* #define TRIPLE_Y */		/* no */
/* #define VINTAGE_DISPLAY */   /* no */

#else

/**** C64 (x64) settings ****/

#define REFRESH_RATE 2

/* #define DOUBLE_X */		/* no */
/* #define TRIPLE_X */		/* no */
/* #define DOUBLE_Y */		/* no */
/* #define TRIPLE_Y */		/* no */
/* #define VINTAGE_DISPLAY */   /* no */

#endif


/* ------------------------------------------------------------------------- */


/*
 * Speed
 *
 * If you want to slow the emulation down to a fixed speed index, simply fill 
 * in the appropriate values (percentage values referred to the real machine). 
 * If you specify a value of 0, the emulators will run as fast as possible.
 * You will still be alowed to change these default values at run-time or on 
 * the command line.
 * The C128 speed refers to 1 MHz system clock.
 */


#define PET_CUSTOM_SPEED	100	/* 100 */
#define C64_CUSTOM_SPEED	100	/* 100 */
#define VIC20_CUSTOM_SPEED	100	/* 100 */
#define C128_CUSTOM_SPEED	100	/* 100 */



/* ------------------------------------------------------------------------- */

/*
 * Keyboard
 */


/*
 * Set up Keyboard Input Buffer in X-version
 *
 * Some keyboards do not generate KeyRelease Event, some have a
 * different layout. Try which settings are best for your keyboard.
 */

/*#define AUTOREPEAT*/	/* no */	/* operate without keyrelease events */
/*#define US_KBD*/	/* US only */	/* swap asciicircum with asciitilde */


/*
 * Host keyboard repeat treshold (ms). This filters out any excessive
 * key events, so it must be longer than the actual autorepeat interval.
 */

#ifndef C128
#define REPEAT_DELAY	50	/* near 50 */
#else
#define REPEAT_DELAY	100
#endif


/*
 * Some keys only for DEC with additonal Editing Keypad:
 * only enable this if you have the Editing Keymap.
 */

/*#define DECMAP*/		/* no */


/* ------------------------------------------------------------------------- */

/*
 * Install Zilog Z80B CPU on the C128 ?
 */

#if (defined(DO_CONFIG) || defined(C128))

/*#define HAS_ZILOG_Z80*/		/* no */

#endif


/* ------------------------------------------------------------------------- */


/*
 * Video chip emulations
 */


/*
 * Do you want the VIC-II sprite-sprite and sprite-background collision
 * registers to be emulated by default? (this can be overridden at 
 * run-time or with command line options)
 */

/*#define CHECK_SS_COLL*/	/* no */
/*#define CHECK_SB_COLL*/	/* no */


/*
 * Do you want to build the VDC chip (80 column) emulator in x128?
 */

#define USE_VDC		        /* yes */


/*
 * PET Screen width. It can be either 40 or 80 Columns.
 */

#define PET_COLS	80	/* 80 */


/* ------------------------------------------------------------------------- */

/*
 * Define the default system directory (where the ROMs are)
 */

#define ROMPATH		"./"

/*
 * If you want, you can change the names of the subdirectories and of the
 * files, too... (but it should not be necessary)
 */

#ifdef C128
#define PROJECTDIR	"C128"
#endif

#ifdef CBM64
#define PROJECTDIR	"C64"
#endif

#ifdef VIC20
#define PROJECTDIR	"VIC20"
#endif

#ifdef PET
#define PROJECTDIR	"PET"
#endif


#ifndef RAMNAME
#define RAMNAME		"ram"
#endif

#ifndef BASICNAME
#define BASICNAME	"basic"
#endif

#ifndef KERNELNAME
#define KERNELNAME	"kernel"
#endif

#ifndef REUNAME
#define REUNAME		"reu"
#endif

#ifndef EXROMNAME
#define EXROMNAME	"exrom"
#endif

#ifndef CHARGENNAME
#define CHARGENNAME	"chargen"
#endif

#ifndef BIOSROM_NAME
#define BIOSROM_NAME  	"Z80_BIOS"
#endif


/* ------------------------------------------------------------------------- */


/*
 * Miscellaneous
 */

/*
 * T64 Tape images support (for C64 and C128 only)
 */


#if (defined(CBM64) || defined(C128))
#define CBMTAPE                 /* yes */
#endif


/*
 * X11 front-end for the debugger
 *
 * 'XCONTROL' selects the old Control Window from X64-0.2.2 to use in
 * case the new one is indigestible. (untested and not recommended if
 * the new 'xdebug.c' compiles.)
 */

/*#define XCONTROL*/		/* no */

#ifndef XCONTROL
#define XDEBUGGER		/* yes */
#endif


/* 
 * Commodore IEEE488 interface adaptor support for the C64.
 */

#ifdef CBM64
#define	IEEE488			/* yes */
#endif


/*
 *  LED Bar - a simple toy.
 *  This will make the CPU emulation a bit slower than usual.
 *  This DOES NOT WORK when DEBUG is defined.
 */

/*#define LED_ARRAY*/		/* no */


/*
 * Joystick (needs the joystick module for Linux)
 * This won't affect the `virtual' joystick emulation via Numpad.
 */

/*#define JOYSTICK*/		/* Linux PC with joystick only */


/*
 * If you want to use command line editor with monitor, define following
 * It uses termios interface for a tty by default, uncomment next line
 * if you only have termio
 */

#define EDIT			/* yes */


/*
 * Which way to copy files ?
 * Since C1541 disk drive copies files to the first filename, whereas
 * they normally go to the last file, one can change the 'c1541' copy
 * syntax via this option.
 *   UNIX:  copy    oldname [oldname2 ...] newname
 *   1541:  copy    newname oldname [oldname2 ...]
 */

/*#define COPY_TO_LEFT*/	/* no */


/*
 * 40xx or 80xx PET ?
 */

#define BASIC4


/*
 * Shall Virtual RAM Expander be installed ?
 * Sizes available are 128, 256 and 512 KB
 * (C128 internal RAM may be expanded in vmachine.h)
 */

#ifndef VIC20
#ifndef PET

#define REU			/* yes */
#define REUSIZE 256

#endif
#endif


/*
 * Utility to patch between ROM revisions.
 * Supports 0, 3, 67 and 100 for the C64, but user can define more.
 */

#ifndef VIC20
#ifndef PET

#define PATCH_ROM		/* yes */

#endif
#endif


/*
 * Emulator Identification Utility
 */

#define EMULATOR_ID		/* yes */


/*
 * Printer
 */

/* Select Default Character Set */

#define PRN_CS_DEFAULT  0	/* 0 */


#if 1  /* European A4 Papersize */
#define PS_WIDTH	595
#define PS_HEIGHT	842

#else  /* American Letter Papersize */
#define PS_WIDTH	612
#define PS_HEIGHT	792
#endif


/* ------------------------------------------------------------------------- */

/*
 * --EXPERIMENTAL OPTIONS--
 *
 * These ones are still unfinished. Try them at your own risk.
 *
 */


/*
 * Pop-up menus.
 *
 * This option will allow you to recall two control menus by clicking
 * with the Left or Right Button on the emulation window.
 *
 */

/* #define POPUP_MENU */	/* no */


/*
 * SID emulation.
 *
 * There are two versions available: Lionel Ulmer's and Teemu Rantanen's. The
 * first one has been tested on SGI Indy workstations, while the second one
 * works under Linux.
 * Timing still has to be implemented correctly, so do not expect much from
 * these...
 *
 */

/* #define X64SOUND   */      	/* no */

#ifdef X64SOUND

/* #define LIONEL_SID */        /* no */
/* #define TEEMU_SID  */	/* no */

#endif


/*
 * Experimental timer code
 */

/* #define NEW_TIMER_CODE */	/* no */


/*
 * Trap idle
 */

/* #define IDLE_TRAP */		/* no */


/* ------------------------------------------------------------------------- */


/*
 * --DEBUGGING--
 *
 * We all hope you won't need these ones... but you never know :)
 * Take care, since some of these options print a really _huge_ amount of
 * information!
 *
 */


/*
 * In case of problems with the keyboard...
 */

/* #define KBDBUG */	/* no */	/* full */
/* #define KBDBUG2 */	/* no */	/* keypress stack only */


/*
 * In case of problems with the disk drives...
 */

/* #define DEBUG_DRIVE */	/* no */
/* #define DEBUG_FSDRIVE */	/* no */

/* #define DEBUG_SERIAL */	/* no */	/* Serial control */
/* #define DEBUG_SERIAL_1 */	/* no */	/* Serial data flow */


/*
 * In case of problems with the video emulations...
 */

/* #define VIC_II_VMODE_DEBUG */	/* no */
/* #define VIC_II_RASTER_DEBUG */	/* no */
/* #define VIC_II_INTERRUPT_DEBUG */	/* no */
/* #define VIC_II_REGISTERS_DEBUG */    /* no */
/* #define DEBUG_SS_COLLISIONS */       /* no */
/* #define DEBUG_SB_COLLISIONS */       /* no */

/* #define VDC_REG_DEBUG */		/* no */
/* #define VDC_VMODE_DEBUG */		/* no */

/* #define VIC_VMODE_DEBUG */		/* no */
/* #define VIC_RASTER_DEBUG */		/* no */


/*
 * In case of problems with the CIAs...
 */

/* #define CIA1_TIMER_DEBUG */          /* no */
/* #define CIA2_TIMER_DEBUG */          /* no */


/*
 * In case you have problems with the Z80 emulation...
 */

#if defined (HAS_ZILOG_Z80)
/* #define DEBUG_Z80 */		        /* no */
#endif


/*
 * In case some programs or intros crash without any apparent reasons, there
 * might be problems with the undocumented opcodes. Enabling this option will
 * list all the undocumented opcodes being used.
 *
 */

/* #define DEBUG_UNDOC */              /* no */


/*
 * This line enables some additional information about the programming being
 * runned and also allows the `trace' monitor command (which can be enabled by
 * the second button of the mouse too) to work properly. If you don't need it,
 * keeping it disabled will make the CPU emulation slightly faster.
 *
 */

/* #define DEBUG */		        /* no */


/* ------------------------------------------------------------------------- */


/* ------------------------------------------------------------------------- */
/* DO NOT CHANGE ANYTHING below this line!                                   */
/* ------------------------------------------------------------------------- */

#ifdef __hpux
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif
#ifndef _INCLUDE_POSIX_SOURCE
#define _INCLUDE_POSIX_SOURCE
#endif
#endif  /* __hpux */

#ifdef DEBUG
#undef LED_ARRAY
#endif

#define VIC_II	       
#define HACK2

#ifdef SOUND
#undef SOUND
#endif

#endif  /* _CONFIG_H */
