/*
 * $Id: cpu.c,v 1.6 1996/06/18 23:30:44 ettore Exp ettore $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 * Written by
 *   Vesa-Matti Puro  (vmp@lut.fi)
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *
 * Patches by
 *   Ettore Perazzoli (ettore@comm2000.it)           /EP/
 *   Andre Fachat     (fachat@physik.tu-chemnitz.de) /AF/
 *
 *
 * $Log: cpu.c,v $
 * Revision 1.6  1996/06/18 23:30:44  ettore
 * M_AIND mode fixed.
 *
 * Revision 1.5  1996/04/01  09:01:41  jopi
 * Exact cycle count implemented
 * Decimal mode on SBC corrected
 * Bus Activity count added
 *
 * Revision 1.4  1995/11/07  16:51:00  jopi
 * *** empty log message ***
 *
 * Revision 1.3  1995/04/01  07:54:09  jopi
 * X64 0.3 PL 0
 * Prototypes.
 *
 * Revision 1.2  1994/12/12  16:59:44  jopi
 * Several changes in op table
 *
 * Revision 1.1  1994/06/16  17:19:26  jopi
 * Initial revision
 *
 *
 */

#include <stdio.h>

#include "config.h"
#include "cpu.h"
#include "macro.h"
#include "extern.h"


/* C64 Profiler arrays. */

#ifdef DEBUG
#include "memory.h"

int     rd[RAMSIZE];
int     wr[RAMSIZE];
long    instructions[300];
#endif

/* CPU internals */

ADDRESS program_counter;
BYTE    accumulator, status_register;
BYTE    x_register, y_register, stack_pointer;

int     zero_flag;
int     sign_flag;
int     overflow_flag;
int     break_flag;
int     decimal_flag;
int     interrupt_flag;
int     carry_flag;

CLOCK   clk;
extern void test_irq(CLOCK clock);

void    u_stoshr(unsigned int val, ADDRESS address, BYTE index);


/*
 * Code for NORMAL and HACK
 */

void    mainloop(ADDRESS addr)
{
#ifndef HACK
    register unsigned int src = 0, result;
    register BYTE p1 = 0;
    register ADDRESS p2 = 0;
#endif
    register struct lookup_tag *op;

    PC = addr;
    SET_SR(0x20);
    SP = 0;
    
    while (1) {

	test_irq(clk);		/* serve interrupts */

        op = &lookup[LOAD(PC)];

#ifdef DEBUG
 	instructions[LOAD(PC)]++;

	if (traceflg)
	  printf(".%04x\t%s\n", PC, sprint_opcode(PC, 1));
	  /*  show(); */
#endif
   
#ifdef HACK
	op -> funktio();
#else
	clk += op -> cycles;

	switch (clength[op -> addr_mode]) {
	  case 1:
	    PC++;
	    break;
	  case 2:
	    p1 = LOAD(PC + 1);
	    PC += 2;
	    break;
	  case 3:
	    p2 = LOAD(PC + 1) | (LOAD(PC + 2) << 8);
	    PC += 3;
	    break;
	  default:
	    printf("AAARGH\n");
	    end64();
	}

	switch (op -> source) {
	  case M_NONE:
	    break;
	  case M_AC:
	    src = AC;
	    break;
	  case M_XR:
	    src = XR;
	    break;
	  case M_YR:
	    src = YR;
	    break;
	  case M_SP:
	    src = SP;
	    break;
	  case M_SR:
	    src = GET_SR();
	    break;
	  case M_IMM:
	    src = p1;
	    break;
	  case M_ZERO:
	    src = LOAD_ZERO(p1);
	    break;
	  case M_ZERX:
	    src = LOAD_ZERO(p1 + XR);
	    break;
	  case M_ZERY:
	    src = LOAD_ZERO(p1 + YR);
	    break;
	  case M_AIND:
	    src = (LOAD(p2)) + (LOAD((p2&0xff00)+((p2+1)&0xff)) <<8); /* AF */
	    break;
	  case M_INDY:
	    p2 = LOAD_ZERO_ADDR(p1);		/* not passed to convop */
	    src = LOAD(p2 + YR);
	    if (op->pbc_fix && (p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
	    break;
	  case M_INDX:
#ifndef GEMDOS
	    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));
	    break;
#else
	    p2 = LOAD_ZERO_ADDR(p1 + XR);
#endif
	  case M_ABS:
	    src = LOAD(p2);
	    break;
	  case M_ABSX:
	    src = LOAD(p2 + XR);
	    if (op->pbc_fix && (p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
	    break;
	  case M_ABSY:
	    src = LOAD(p2 + YR);
	    if (op->pbc_fix && (p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
	    break;
	  case M_ADDR:
	    src = p2;
	    break;
	  case M_REL:
	    src = p1;
	    break;
	  case M_FC:
	    src = S_CARRY;
	    break;
	  case M_FD:
	    src = S_DECIMAL;
	    break;
	  case M_FI:
	    src = S_INTERRUPT;
	    break;
	  case M_FV:
	    src = S_OVERFLOW;
	    break;

#ifndef NO_UNDOC_CMDS
	    /* These are used by undocumented commands only */
	  case M_ACIM:
	    src = (AC & p1);
	    break;
	  case M_ANXR:
	    src = (AC & XR);
	    break;
	  case M_AXIM:
	    src = ((AC | 0xee) & XR & p1);
	    break;
	  case M_SABY:
	    src = (SP & LOAD(p2 + YR));
	    break;
#endif

	  default:
	    printf("CPU: Internal bus error at %04X\n", PC);
	    end64();
	}

#ifdef DEBUG
	if (traceflg) {
	    printf("src:%d(%d), dst:%d length:%d p1:%d, p2:%d ",
		src, op -> source, op -> destination, clength[op -> addr_mode],
		p1, p2);
	    fflush(stdout);
	    result = (*(op -> funktio))(src);
	    printf("result %x\n", result);
	} else
#endif
	    result = (*(op -> funktio))(src, AC);

	switch (op -> destination) {
	  case M_NONE:
	    break;
	  case M_AC:
	    AC = result;
	    break;
	  case M_XR:
	    XR = result;
	    break;
	  case M_YR:
	    YR = result;
	    break;
	  case M_SP:
	    SP = result;
	    break;
	  case M_SR:
	    SET_SR(result);
	    break;
	  case M_PC:
	    PC = result;
	    break;
	  case M_ZERO:
	    STORE_ZERO(p1, result);
	    break;
	  case M_ZERX:
	    STORE_ZERO(p1 + XR, result);
	    break;
	  case M_ZERY:
	    STORE_ZERO(p1 + YR, result);
	    break;
	  case M_INDY:
	    STORE(LOAD_ZERO_ADDR(p1) + YR, result);
	    break;
	  case M_INDX:
#ifndef GEMDOS
	    STORE(LOAD_ZERO_ADDR(p1 + XR), result);
	    break;
#else
	    p2 = LOAD_ZERO_ADDR(p1 + XR);
#endif
	  case M_ABS:
	    STORE(p2, result);
	    break;
	  case M_ABSX:
	    STORE(p2 + XR, result);
	    break;
	  case M_ABSY:
	    STORE(p2 + YR, result);
	    break;

	  case M_FC:
	    SET_CARRY(result);
	    break;
	  case M_FD:
	    SET_DECIMAL(result);
	    break;
	  case M_FI:
	    SET_INTERRUPT(result);
	    break;
	  case M_FV:
	    SET_OVERFLOW(result);
	    break;

#ifndef NO_UNDOC_CMDS
	    /* These are used by undocumented commands only */
	  case M_ACNC:
	    AC = result;
	    SET_CARRY(IF_SIGN());
	    break;
	  case M_ACXR:
	    AC = XR = result;
	    break;
	  case M_ACXS:
	    AC = XR = SP = result;
	    break;
	  case M_STH0:
	    u_stoshr(src, LOAD_ZERO_ADDR(p1), YR);
	    break;
	  case M_STH2:
	    u_stoshr(src, p2, XR);
	    break;
	  case M_STH3:
	    SP = src;	/* SHS */
	  case M_STH1:
	    u_stoshr(src, p2, YR);
	    break;
#endif   /* UNDOC */

	  default:
	    printf("CPU: Internal bus error at %04X\n", PC);
	    end64();
	}
#endif   /* HACK */


	DoRaster( clk );
#ifdef C128
	DoVDCRaster( clk );
#endif
    }
}


#ifndef NO_UNDOC_CMDS

void    u_stoshr(unsigned int val, ADDRESS address, BYTE index)
{
    val &= (((address >> 8) + 1) & 0xff);
    if (((address & 0xff) + index) > 0xff)
	address = val;
    STORE((address + index), val);
}
#endif   /* UNDOC */

