/*
 * $Id: crtc.c,v 1.4 1996/07/29 20:54:35 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 *
 * See README for copyright notice
 *
 * Written by
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 *
 * $Log: crtc.c,v $
 * Revision 1.4  1996/07/29 20:54:35  ettore
 * Some garbage removed.
 * Uses the new DoRaster() without argument.
 *
 * Revision 1.3  1996/04/01 06:04:49  ettore
 * New video emulation.
 *
 * Revision 1.2  1995/11/07  16:51:00  jopi
 * *** empty log message ***
 *
 * Revision 1.1  1995/06/28  19:34:07  jopi
 * Initial revision
 *
 */

#ifndef PET
#define PET
#endif
#define _CRTC_C


#include "crtc.h"		/* This order is required for HP-UX */
#include "raster.c"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "config.h"
#include "vmachine.h"
#include "extern.h"
#include "rasterutil.h"
#include "interrupt.h"

extern CLOCK clk;

/* Video Chip information */

/* for the defines see vmachine.h */
#define SCREENSIZE  (XSIZE * YSIZE)

VideoStruct *v_crtc = NULL;

/* global */

int	XSIZE = 80;		/* default values */
int	XPIX = 640;

BYTE crtc[19];

BYTE *scrbase = 0;		/* pointer to start of screen */
BYTE *bitmap1_base = 0;		/* pointer to upper half of bitmap */
BYTE *bitmap2_base = 0;		/* pointer to lower half of bitmap */

int yscroll = 0;
int xscroll = 0;


/*
 * Local Variables
 */

static ADDRESS scraddr;		/* screen start address */


/* ------------------------------------------------------------------------- */


void
screen_init (void)
{
  /*
   *  First calculate screen start address in memory
   */
  scraddr = 0x8000;
  scrbase = ram + scraddr;

  if (debugflg)
    printf ("scraddr:%x\n", scraddr);

  yscroll = crtc[0x00] & 0x7f;
  xscroll = crtc[0x01];

  v_crtc->screenmem = scrbase;
}

void crtc_set_char(int crom) {	/* set charom */
  
  v_crtc->chargenmem = char_rom;
  if(crom) {
    v_crtc->chargenmem += 2048;
  }
}

/* ------------------------------------------------------------------------- */

void reset_crtc(void) {
   /* spec says to initialize "all internal scan counter circuits.
    * When /RES is low, all internal counters stop and clear and all
    * scan and video output go low; control registers are unaffected.
    * All scan timing initiates when /RES goes high. "
    * "In this way, /RES can synchronize display frame timing with 
    * line frequency."
    *
    * Well, we just emulate... 
    */

/*   store_crtc (5, 240);*/

   /* This is for PET 3032 - to get it working without any crtc register
    * setting, as the older PETs don't know about them 
    */
   v_crtc->screenmem = ram + 0x8000;

   return;
}

int crtc_offscreen(void) {
	return v_crtc->rasterline >= YPIX;
}

/* ------------------------------------------------------------------------- */

void
store_crtc (ADDRESS addr, BYTE value)
{
  crtc[addr] = value;

  if (debugflg)
    printf ("store crtc[%x] %x\n", addr, (int) value);

  switch (addr)
    {
    case 0:			/* R00  Horizontal total (characters + 1) */
    case 1:			/* R01  Horizontal characters displayed */
      screen_init ();
      break;

    case 2:			/* R02  Columns Displayed  (+1) */

    case 3:			/* R03  Horizontal/Vertical Sync widths */
      screen_init ();
      break;

    case 4:			/* R04  Raster line count */
      break;

    case 5:			/* R05  Vertical Screen position */
      screen_init ();
      break;

    case 6:			/* R06  Number of display lines on screen */
    case 7:			/* R07  Lines displayed */
      break;

    case 8:			/* R08  unused: Interlace and Skew */
      break;

    case 9:			/* R09  Rasters between two display lines */
      break;

    case 10:			/* R10  Cursor (not implemented on the PET) */
    case 11:			/* R11  Cursor (not implemented on the PET) */
      break;

    case 12:			/* R12  Control register */

      /* Bit 0: 1=add 256 to screen start address ( 512 for 80-columns)
       * Bit 1: 1=add 512 to screen start address (1024 for 80-columns)
       * Bit 2: invert flyback
       * Bit 3: invert video signal
       * Bit 4: use top half of 4K character generator
       * Bit 5: (not implemented on the PET)
       * Bit 6: (not implemented on the PET)
       * Bit 7: not used.
       */

      break;

    case 13:			/* R13  Address of first character */
      /* Value + 32786 (8000) */
      screen_init ();
      break;

    case 14:
    case 15:			/* R14-5 Cursor location HI/LO -- unused */
      break;

    case 16:
    case 17:			/* R16-7 Light Pen HI/LO -- read only */
      break;

    case 18:
    case 19:			/* R18-9 Update address HI/LO (only 6545)  */
      break;
    }
}


BYTE
read_crtc (ADDRESS addr)
{
  switch (addr)
    {
    case 14:
    case 15:			/* Cursor location HI/LO */
      return (crtc[addr]);

    case 16:
    case 17:			/* Light Pen X,Y */
      return (0xff);

    default:
      return (0);		/* All the rest are write-only registers */
    }
}

void
store_colorram (ADDRESS addr, BYTE value)
{
}

BYTE
read_colorram (ADDRESS addr)
{
  return 0;
}

/* ------------------------------------------------------------------------- */

void
int_raster (void)
{
  return;
}

void
DoRaster (void)
{
  Raster (v_crtc);

  /* this generates one raster interrupt per frame */
  if (!v_crtc->rasterline)
    if (clk > 1000)
      setalarm(I_RASTER, 0);
}

void
InitScrImg (void)
{
}

Window
Crtc_Init (Display * disp, int scr_num, char *win_name, char *icon_name, char *res_class,
	   char *res_name, int *w, int *h, char *icon_data[], Colormap * cmap_return,
	   int exact_speed, int asleep, int refrate, Atom * wm_delwin)
{
  static BYTE fake_colormem[2000];

  memset (fake_colormem, 1, 2000);	/* this makes it think that there is a color mem */
  v_crtc = AllocVideo (SCR_COLS, SCR_LINES, BORD_WIDTH, BORD_HEIGHT, XPIX, YPIX, XSIZE, YSIZE,
		       CRTC_NUMCOLORS, refrate, CRTC_NUMSPRITES, exact_speed, win_name, asleep);
  if (OpenVideoWindow (disp, scr_num, v_crtc, win_name, win_name, res_class, res_name, icon_data,
		       crtc_cnames, cmap_return, wm_delwin))
    {
      fprintf (stderr, "fatal error: can't open window for CRTC emulation.\n");
      return -1;
    }
  if (SetVideoModes (v_crtc, 1,
  /* CRTC_NORMAL_MODE */
		     NormalGetTextData, NormalGetColorData,
		     NormalGetBgDataFromRegister, HIRES_VMODE))
    {
      fprintf (stderr, "fatal error: can't initialize CRTC emulation.\n");
      return -1;
    }
  v_crtc->vmode = 0;		/* CRTC_NORMAL_MODE; */
  RefreshAll (v_crtc);
  *w = v_crtc->winwidth;
  *h = v_crtc->winheight;
  v_crtc->chargenmem = char_rom;
  v_crtc->screenmem = ram;
  v_crtc->bordcol = 0;
  v_crtc->bgcol = 0;
  v_crtc->colormem = fake_colormem;
  return (v_crtc->win);
}

void
Crtc_ArrangeWindow (int width, int height)
{
  ArrangeVideoImage (v_crtc, width, height);
  RefreshAll (v_crtc);
}

void
Crtc_Free (void)
{
  FreeVideo (v_crtc);
}

void
Crtc_Pause (void)
{
  PauseVideo (v_crtc);
}

void
Crtc_Restart (void)
{
  RestartVideo (v_crtc);
}
