/*
 * $Id: joystick.c,v 1.2 1996/07/30 08:23:06 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 * This file implements joystick for Linux.
 *
 *
 * Written by
 *   Ulmer Lionel     (ulmer@poly.polytechnique.fr)
 *
 * Patches by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: joystick.c,v $
 * Revision 1.2  1996/07/30 08:23:06  ettore
 * Uses hjoyport.
 * Device errors no longer kill the emulator.
 *
 * Revision 1.1  1995/04/01  07:50:52  jopi
 * Initial revision
 *
 *
 */


#include <linux/joystick.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "extern.h"

void joyini(void);
void joyclose(void);
void joystick(void);

int joyfd;
int hjoyport = 1;

void joyini(void)
{
	 
	 /* open device file */
	 joyfd=open("/dev/js0",O_RDONLY);
	 if (joyfd < 0) {
			fprintf(stderr, "Warning: couldn't open the joystick device!\n");
	 } else {
	                if (app_resources.hjoyPort != 1 && app_resources.hjoyPort != 2 ) {
				hjoyport = 3 - app_resources.joyPort;
				fprintf (stderr,
				 "Warning: hjoyPort resource not valid -- assuming port %d\n\n",
		    		 hjoyport);
			} else {
				printf ("Hardware joystick in port #%d.\n",
					(hjoyport = app_resources.hjoyPort));
		        }
	 }
}


void joyclose(void)
{
	 close (joyfd);
}


void joystick(void)
{
	 int status;
	 struct JS_DATA_TYPE js;

	 if (joyfd < 0)
			return;
	 
	 status=read(joyfd,&js,10 JS_RETURN);
	 if (status != JS_RETURN) {
			fprintf(stderr, "Warning: error reading the joystick device!\n");
			return;
	 }

	 if ((js.y<300) && !(joy[hjoyport] & 1))
	 {
			if (debugflg)
			  printf("Up\n");
			
			joy[hjoyport] |= 1;
	 } else if ((js.y>1000) && !(joy[hjoyport] & 2))
	 {
			if (debugflg)
			  printf("Down\n");
			
			joy[hjoyport] |= 2;
	 } else if ((js.y>=300) && (js.y<=1000))
	 {
			if (joy[hjoyport] & 1)
			{
				 if (debugflg)
				   printf("Up released\n");
				 
				 joy[hjoyport] &= ~1;
			} else if (joy[hjoyport] & 2)
			{
				 if (debugflg)
				 	printf("Down released\n");
				 
				 joy[hjoyport] &= ~2;
			}
	 }
	 
	 if ((js.x<300) && !(joy[hjoyport] & 4))
	 {
			if (debugflg)
			  printf("Left\n");
			
			joy[hjoyport] |= 4;
	 } else if ((js.x>1100) & !(joy[hjoyport] & 8))
	 {
			if (debugflg)
			  printf("Rigth\n");
			
			joy[hjoyport] |= 8;
	 } else if ((js.x>=300) && (js.x<=1100))
	 {
			if (joy[hjoyport] & 4)
			{
				 if (debugflg)
				 	 printf("Left released\n");
				 
				 joy[hjoyport] &= ~4;
			} else if (joy[hjoyport] & 8)
			{
				 if (debugflg)
				 	 printf("Right released\n");
				 
				 joy[hjoyport] &= ~8;
			}
	 }
	 
	 if ((js.buttons & 1) && !(joy[hjoyport] & 16))
	 {
			if (debugflg)
			  printf("Button pressed\n");
			
			joy[hjoyport] |= 16;
	 } else if (!(js.buttons & 1) && (joy[hjoyport] & 16))
	 {
			if (debugflg)
			  printf("Button released\n");
			
			joy[hjoyport] &= ~16;
	 }
}                      
