/*
 * $Id: main.c,v 1.13 1996/07/29 21:20:20 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 * Main builds up the emulation and controls its operation.
 *
 * Written by
 *   Vesa-Matti Puro  (vmp@lut.fi)
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 *
 * $Log: main.c,v $
 * Revision 1.13  1996/07/29 21:20:20  ettore
 * Miscellaneous fixes.
 * Handles environment variables.
 * SID-ready.
 *
 * Revision 1.12  1996/04/01  09:01:41  jopi
 * PET and VIC-20 implemented
 * Emulator Information screen added
 *
 * Revision 1.11  1995/11/07  16:51:00  jopi
 * different emulators, emuinfo
 * resource debug
 *
 * Revision 1.10  1995/06/28  19:40:56  jopi
 * PET, emu info
 *
 * Revision 1.9  1995/04/01  07:54:09  jopi
 * X64 0.3 PL 0
 * Options and control for Printer, Serial and ROM patcher updated.
 * Signal calls end64 for decent exit.
 *
 * Revision 1.8  1994/12/12  16:59:44  jopi
 * *** empty log message ***
 *
 * Revision 1.7  1994/06/16  17:19:26  jopi
 * Code reorganized and cleaned up
 *
 * Revision 1.6  1994/02/18  16:10:57  jopi
 * New debugger & arg parser, x20 and x128
 *
 * Revision 1.5  1994/01/26  16:08:37  jopi
 * X64 version 0.2 PL 1
 * load file reorganised
 *
 * Revision 1.4  1993/11/10  01:55:34  jopi
 * reu, asm and disk directory fixed
 * REL_ADDR macro and 1541 made more portable
 *
 * Revision 1.3  93/06/21  13:38:36  jopi
 *  X64 version 0.2 PL 0
 *
 * Revision 1.2  1993/06/13  08:21:50  sonninen
 * *** empty log message ***
 *
 *
 */

#include "macro.h"
#include "vmachine.h"
#include "memory.h"
#include "serial.h"
#include "extern.h"
#include "proto.h"
#include "patchlevel.h"
#include "config.h"
#include "interrupt.h"
#include "sid.h"

#ifdef __hpux
#ifndef _INCLUDE_POSIX_SOURCE
#define _INCLUDE_POSIX_SOURCE
#endif
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif
#endif  /* __hpux */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

/* system info */
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <sys/utsname.h>	/* Needs "config.h" */
#ifdef SYSINFO
#include <sys/systeminfo.h>
#endif

#ifdef PET
#define EI_M  0
#endif
#ifdef VIC20
#define EI_M  1
#endif
#ifdef CBM64
#define EI_M  2
#endif
#ifdef PLUS_SERIES
#define EI_M  3
#endif
#ifdef C128
#define EI_M  4
#endif

#define EMULATOR emutypes[EI_M]

static char *cputypes[] = { "6502", "6502", "6510", "7501", "8502" };
static char *emutypes[] = { "PET", "VIC 20", "C=64", "C=16", "C=128" };


char *progname;

extern CLOCK   clk;
/*
extern CLOCK   next_int_clk;
extern CLOCK   int_clk[NUMOFINT];
extern int     next_int;
*/
extern int     halt;

#ifdef PET
extern int kernel_rev;
extern void set80key(void), set40key(void);
#endif

#if 1
int     module = 0;
int     hexflg = 1;
int     runflg = 0;		/* ==> app_resources */
int     traceflg = 0;		/* CPU */
int     debugflg = 0;		/* CIA */
int     verflg = 0;		/* VIC */
int     joyport = 2;		/* # of joystick port emulated via numpad */
int 	speed = 0;
int     video80 = 0;		/* enable/disable VDC (C128 only) */
int	video40 = 1;		/* enable/disable VIC-II (C128 only) */
int     mitshm = 1;		/* enable/disable MITSHM extensions */
#if !defined (C128)
int	refrate = 1;		/* refresh rate */
#else
int     vdcrefrate = 1;
int     vicrefrate = 1;
#endif
#endif

int     autodump = 0;
int     reuflg = 0;

static int hasX;	/* do we have xstart() run before xexit() ? */

#if 0
#ifdef NEW_TIMER_CODE

extern int new_alarm_number;
extern void new_alarm_trap(int i);
extern void update_intr_alarm(void);

#endif /* NEW_TIMER_CODE */
#endif

#ifdef SUN
extern int getpagesize ( void );
#endif
static void emuinfo ( int emu );
static void resdump ( void );
static void break64 (int dummy );


 /* Code */


int     main(int argc, char **argv)
{
    ADDRESS start_addr;
    int     i;
    int     rl = -1;
    int     len=  0;
    char *p;


    /* Initialisation. */

    /*app_resources.ramName   = RAMNAME;
    app_resources.basicName = BASICNAME;
    app_resources.kernelName= KERNELNAME;*/

    app_resources.notraps = 0;
    app_resources.PrinterLang = app_resources.Locale = NULL;

    progname = *argv;
    while((p = strchr(progname,'/'))) progname = p+1;

    arg_parser(argc, argv);

    signal(SIGINT,  break64);
    signal(SIGPIPE, break64);


#if 1
    if (app_resources.module)
	module = atoi(app_resources.module);

    hexflg = app_resources.hexflag;
    runflg = app_resources.asmflag;
    traceflg=app_resources.traceflag;	/* CPU */
    debugflg=app_resources.debugflag;	/* CIA */
    verflg = app_resources.verboseflag;	/* VIC */
#if defined (C128) || defined (CBM64)
    check_ss_coll = app_resources.checkSsColl;
    check_sb_coll = app_resources.checkSbColl;
#endif
    speed = app_resources.speed; 

#if defined (C128)
    video80 = app_resources.video80;
    video40 = app_resources.video40;
    if (!video80 && !video40)
#if defined (_USE_VDC)
      video80 = 1;
#else
      video40 = 1;
#endif
    vdcrefrate = app_resources.vdcRefreshRate;
    vicrefrate = app_resources.vicRefreshRate;
#else  /* !defined (c128) */
    refrate = app_resources.refreshRate;
#endif
#endif

#if defined (MITSHM)
    mitshm = app_resources.mitshm;
#else
    mitshm = 0;
#endif


    /* Environment Variables */

    /* If directory is not specified, try to concatenate the ROM directory
       for current machine to the value of VICEHOME. */

    if (app_resources.directory == NULL || !*app_resources.directory) {
      /*if (getenv("VICE_ROM")) {
	    app_resources.directory = malloc(64);
	    sprintf(app_resources.directory, "%s/%s",
		    getenv("VICE_ROM"), PROJECTDIR);
	}
	else*/ if (getenv("VICEHOME")) {
	    app_resources.directory = malloc(64);
	    sprintf(app_resources.directory, "%s/%s",
		    getenv("VICEHOME"), PROJECTDIR);
	}
	else
	    app_resources.directory = PROJECTDIR;
    }

    /* If VICE_ROM  or VICEPATH  is undeclared, use VICEHOME ... */


    /*set_locale(app_resources.Locale);*/


    /*
     * Boot
     */

    printf ("\n*** VICE Version %4.2f PL %d  %s ***\n", 
	    (float)VERSION, (int)PATCHLEVEL, PATCHMESG);
    printf ("Welcome to %s, the Commodore %s Emulator for the X-Window System.\n\n",
	    progname, EMULATOR);
    printf ("Copyright (c) 1993-1996\nJ. Sonninen, T. Rantanen, J. Valta, E. Perazzoli and A. Fachat.\n\n");

    /* should be before load_mem - forks another process! */
    /* really experimental and server not (yet) included! */
#ifdef SOUND
    snd_init("/home/fachat/src/audio/sound");
#endif

#if defined (X64SOUND) && defined (CBM64)
    /* Initialize the SID emulation and audio device...
       (will need some parameters soon ....) */
#if defined (LIONEL_SID)
    IniSID();
#endif
#endif /* X64SOUND */

    if (load_mem() < 0 && !app_resources.asmflag)
	exit (1);


#ifdef PET
    printf("PET: loaded kernel, checksum is %d\n",kernel_rev);

    video80 = app_resources.video80;
    video40 = app_resources.video40;
    if(!video80 && !video40) {
      printf("PET: No command line switch for screen width!\n");
      if(kernel_rev == PET8032_CHECKSUM_A 
	  || kernel_rev == PET8032_CHECKSUM_B) {
	printf("PET: Identified PET 8032 kernel by checksum.\n");
	video80 = 1;
      } else
      if(kernel_rev == PET3032_CHECKSUM) {
	printf("PET: Identified PET 3032 kernel by checksum.\n");
	video40 = 1;
      } else 
      if (kernel_rev == PET4032_CHECKSUM) {
	printf("PET: Identified PET 4032 kernel by checksum.\n");
	video40 = 1;
      }
    }
    if(!video80 && !video40) {
      printf("PET: Couldn't identify PET kernel, trying default!\n");
#if PET_COLS == 80
      video80 = 1;
#else
      video40 = 1;
#endif
    }
    if(video80) {
      XSIZE = 80;
      set80key();
    } else {
      XSIZE = 40;
      set40key();
    }
    XPIX = XSIZE*8;
    printf("PET: Setting screen width to %d.\n", XSIZE);
#endif /* PET */


     printf("\nInstalling configuration dependent code...\n\n");

#ifdef PATCH_ROM
#ifdef C128
    if (app_resources.basicRev) {
	(void)patch_rom (app_resources.basicRev);
    }
#endif
    if (app_resources.kernelRev) {
	(void)patch_rom (app_resources.kernelRev);
    }
#endif  /* PATCH_ROM */

#ifdef REU
    if (app_resources.reuName) {
	if (reset_reuc(app_resources.reuName, (REUSIZE << 10)) >= 0)
	    ++reuflg;	/* REU active */
    }
#endif

#ifndef NO_SERIAL
#ifdef IEEE488
    if ((!app_resources.notraps) || ieee488) {
#else
    if (!app_resources.notraps) {
#endif
	printf("Initializing Serial Bus:\n");

	if(!app_resources.notraps) initialize_traps();
	initialize_serial(app_resources.notraps);

	initialize_printer(4, app_resources.PrinterLang,
			   app_resources.Locale );

	initialize_1541(8, DT_DISK | DT_1541);
	initialize_1541(9, DT_DISK | DT_1541);
	initialize_1541(11, DT_FS | DT_1541);


	if (app_resources.floppyName)
	    if ( serial_select_file(DT_DISK | DT_1541, 8, app_resources.floppyName) < 0)
		fprintf (stderr, "\nFloppy attach failed.\n");
	if (app_resources.floppy9Name)
	    if ( serial_select_file(DT_DISK | DT_1541, 9, app_resources.floppy9Name) < 0)
		fprintf (stderr, "\nFloppy attach failed.\n");

#ifdef CBMTAPE
	initialize_tape (1);
	if (app_resources.tapeName && *app_resources.tapeName)
	    if (serial_select_file(DT_TAPE, 1, app_resources.tapeName) < 0) {
		fprintf (stderr, "No Tape.\n");
	    }
#endif

     } /* Traps */
#endif  /* NO_SERIAL */

    putchar ('\n');

#ifndef PET    
    if (app_resources.joyPort != 1 && app_resources.joyPort != 2 )
      fprintf (stderr, "Warning: joyPort resource not valid -- assuming port %d\n\n", joyport);
    else
      printf ("Numpad joystick in port #%d.\n", (joyport = app_resources.joyPort));
#endif    

#if defined(JOYSTICK) && !defined(PET)
    /* initialize real joystick (Linux only) */
    joyini();
#endif

    putchar ('\n');

    /* Load given program */
    if (app_resources.reloc)
	rl = (ADDRESS) strtol(app_resources.reloc, NULL, hexflg ? 16 : 10);

    if (app_resources.programName)
	len = LoadFile (NULL, app_resources.programName, ram, rl, RAMSIZE);

    printf ("Initializing video emulation (MITSHM %s)...\n\n", mitshm ? "enabled" : "disabled");
    x_start();
    ++hasX;

     /* must be done before calling intr() first at RESET */
     for (i = 0; i < NUMOFINT; i++)
        int_clk[i] = 0;

     int_clk[I_SPECX] = 1;
     clk = 0;     

    /* Set the start address. */
#if 0
	if (app_resources.programName && rl < 0) {   /* Default basic load */
	    start_addr = LOAD_ADDR((ADDRESS)65534);  /* Try warm start */
    }
    else	/* Set the default start address. */
#endif
	start_addr = intr(I_RESET, 0);

    if (app_resources.startAddr /*&& *app_resources.startAddr*/)
	start_addr = (ADDRESS) strtol(app_resources.startAddr,
				      NULL, hexflg ? 16 : 10);


    /* Print all values from resources and command line options. */

    if (app_resources.verboseflag || app_resources.debugflag) {
	emuinfo(EI_M);
	resdump();
	printf ("\n");
    }

#ifdef HAS_ZILOG_Z80
    initialize (app_resources.testflag);
#endif

#if 0
    for (i = 0; i < NUMOFINT; i++)
	int_clk[i] = 0;

    int_clk[I_SPECX] = 1;
    clk = 0;

#ifdef NEW_TIMER_CODE
    new_alarm_number = new_alarm();
    update_intr_alarm();
#endif  /* NEW_TIMER_CODE */
#endif

    /* Here we go ... */

    if (app_resources.asmflag)
	mon(start_addr);

    printf("Starting Commodore %s emulator with start address %04X\n\n",
	   EMULATOR, start_addr);

    /* fprintf(stderr, "haltflag=%d\n",app_resources.haltflag); */

    if (app_resources.haltflag) {
	toggle_cntl_win();
	halt=1;
    }

    mainloop(start_addr);
    printf("perkele!\n");
    exit(0);   /* never reached */
}


static void break64(int dummy)
{
    end64();
}


void    end64(void)
{
    printf("\nExiting...\n");		/* add return for C-c */

    if (hasX)
	x_exit();

#ifndef NO_SERIAL
    if (!app_resources.notraps)
	remove_serial(-1);
#endif

#if defined(JOYSTICK) && !defined(PET)
    if (joyfd >= 0)
	joyclose();
#endif

#ifdef REU
    if (reuflg)
	close_reuc(app_resources.reuName);
#endif

    if (autodump)
      dump (create_name(NULL, RAMNAME));

#ifdef DEBUG
    /*
     * This routine stores cumulated count of different instructions
     * if profiler is enabled.
     */

    if (debugflg) {
	int     i;
	long    total = 0;
	char   *bufp;
	FILE   *fp;

	bufp = create_name(NULL, "inst");
	if (NULL == (fp = fopen(bufp, "w")))
	    return;

	fprintf(fp, "\n\tCPU0 Statistics\n\n");

	for (i = 0; i < 256; i++) {
	    total += instructions[i];
	    fprintf(fp, "%02X %-4s %8ld\n", i, lookup[i].mnemonic,
		instructions[i]);
	}
	fprintf(fp, "----------------\n\tTotal %8ld\n", total);
	fprintf(fp, "\tClock: %5ld cycles\n", clk);
	fclose(fp);

	/* Store C64 memory access log file */
	/* if ((bufp = create_name(NULL, "mem")) == NULL ||
	 *   ( fp = fopen( "mem", "w" )) == NULL) return 1;
	 * for ( i=0; i<65536; i++ )
	 *   fprintf( fp, "%04X %10d %10d\n", i, rd[i], wr[i]); fclose( fp );
	 */
    }
#endif

    putchar ('\n');
    exit (0);
}


/* ------------------------------------------------------------------------- */

/*
 * X64 Information screen inspired by Michael A. Cooper's sysinfo(1).
 * uname(2) and sysconf(2) give the needed values.
 */


static void emuinfo(int emu)
{
#ifdef SYSINFO
    char buf[256];
#endif
    long count = 31;
    struct utsname host;

    /*
     * Host Info
     */

    printf("\nHost System:\n\n");

    if (uname (&host) >= 0) {
      printf("    Host Name\t\t\t   : %s\n", host.nodename);
      printf("    OS Version\t\t\t   : %s Release %s\n",
	     host.sysname, host.release /*, host.version */);
      printf("    Architecture\t\t   : %s\n", host.machine);
    }

#ifdef SYSINFO
    sysinfo(SI_ARCHITECTURE, buf, count);
    printf ("    CPU Architecture\t\t   : %s\n", buf);
#endif
#ifdef _SC_NPROCESSORS_ONLN
    printf ("    Number of CPUs in use\t   : %ld\n",
	    sysconf(_SC_NPROCESSORS_ONLN));	/* Number of CPUs alive */
#endif

#ifndef __hpux
#ifdef _SC_PAGESIZE
    count = (long) sysconf(_SC_PAGESIZE);
    count *= (long)sysconf(_SC_PHYS_PAGES);
#else
    count = (long) getpagesize();		/* System memory page size */
    count *= (long)1;
#endif  /* _SC_PAGESIZE */

    if (count >> 20)
      printf ("\n    RAM Installed total\t\t   : %ld MB\n", count >> 20);
#endif /* __hpux */

    printf ("    Bitplanes on Display\t   : %s\n", get_x_depth());


    /*
     * Emulator Info
     */

    printf("\n\nEmulator:\n\n");

    printf ("    Emulator Version\t\t   : %3.1f PL %d  %s\n",
	(float)VERSION, (int)PATCHLEVEL, PATCHMESG);

#ifndef REUSIZE
#define REUSIZE 0
#endif

       printf ("\
    CBM %s Kernel Version\t   :  d  s\n\
    Main Memory                    : %3d KB\n\
    REU Expansion Memory           : %3d KB (%s)\n\n\
    Total number of CPUs emulated  : 1\n\
    CPU Type                       : %s\n",
	       emutypes[emu],
	       (RAMSIZE >> 10), (REUSIZE >>10),
	       app_resources.reuName ? app_resources.reuName : "disabled",
	       cputypes[emu]);


    printf("\n\nPeripheral Information:\n\n");

    printf("    Max Serial Address  : 15\n");

#ifndef NO_SERIAL
    remove_serial(-2);		/* QUERY mode */
    printf("\n\n");
#endif  /* NO_SERIAL */
}


static void resdump (void)
{
#define PRINT_PTR(a,b)  printf("%8s\t%s\n", a, (app_resources.b ? app_resources.b : "(null)"))
#define PRINT_CHR(a,b)  printf("%8s\t%d\n", a, app_resources.b)
#define PRINT_INT(a,b)  printf("%8s\t%d\n", a, app_resources.b)

    printf("\n");
    PRINT_PTR("directory",	directory);
    PRINT_PTR("basicName",	basicName);
    PRINT_PTR("kernelName",	kernelName);
    PRINT_PTR("charName",	charName);
    PRINT_PTR("biosName",	biosName);
 
    PRINT_PTR("basicRev",	basicRev);
    PRINT_PTR("kernelRev",	kernelRev);
 
    PRINT_PTR("exromName",	exromName);
    PRINT_PTR("module",		module);
    PRINT_PTR("reuName",	reuName);
    PRINT_PTR("reusize",	reusize);
 
    PRINT_PTR("ramName",	ramName);
    PRINT_PTR("programName",	programName);
    PRINT_PTR("reloc",		reloc);
    PRINT_PTR("startAddr",	startAddr);   /* PC */
#if defined (C128) 
    PRINT_INT("video40",	video40);
    PRINT_INT("video80",	video80);
#endif
    PRINT_CHR("colors",		colors);
    PRINT_CHR("asmflag",	asmflag);
    PRINT_CHR("hexflag",	hexflag);
    PRINT_CHR("debugflag",	debugflag);
    PRINT_CHR("traceflag",	traceflag);
    PRINT_CHR("verboseflag",	verboseflag);
    PRINT_CHR("testflag",	testflag);
    PRINT_CHR("notraps",	notraps);
 
    PRINT_PTR("dosRomName",	dosRomName);
    PRINT_PTR("floppyName",	floppyName);
 
    PRINT_PTR("printCommand",	printCommand);
    PRINT_PTR("PrinterLang",	PrinterLang);
    PRINT_PTR("Locale",		Locale);

    PRINT_INT("speed",		speed);
#if defined (C128)
    PRINT_INT("vdcRefreshRate",	vdcRefreshRate);
    PRINT_INT("vicRefreshRate",	vicRefreshRate);
#else
    PRINT_INT("refreshRate",	refreshRate);
#endif
    PRINT_INT("mitshm",		mitshm);
}
