
/* ophack2.c
 * this file is generated from op.c -do not edit.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include "cpu.h"
#include "op.h"
#include "macro.h"
#include "vmachine.h"
#include "extern.h"
#include "interrupt.h"


#ifdef DEBUG
#include "memory.h"

int     rd[RAMSIZE];
int     wr[RAMSIZE];
long    instructions[256];
#endif
#ifdef LED_ARRAY
int     ledarr[16];
#endif


/* CPU internals */

ADDRESS program_counter;
BYTE    accumulator, status_register;
BYTE    x_register, y_register, stack_pointer;


int     zero_flag;
int     sign_flag;
int     overflow_flag;
int     break_flag;
int     decimal_flag;
int     interrupt_flag;
int     carry_flag;
int     rmw_flag = 0;	/* emulates the RMW bug when necessary */


CLOCK   clk;
extern void test_irq(CLOCK clock);
extern void DoRaster(void);

void    mainloop(ADDRESS addr)
{
    register BYTE b;
    extern int     next_int;
    register BYTE p1;
    register unsigned int p2;
    register unsigned int src;
    register unsigned int temp;
    CLOCK video_clk = CYCLES_PER_LINE;

#ifdef NEW_TIMER_CODE__
    ;
#endif
    SET_SR(0x20);SP = 0;
    PC = addr;
    while (1) {
	while (clk > next_int_clk)
	    PC = intr(next_int, PC);


	if (cpu_check_irq) PC = cpu_do_int(PC);
#ifdef VIC_II
	if (clk >= video_clk) { DoRaster(); video_clk += CYCLES_PER_LINE; }
#endif

#ifdef NEW_TIMER_CODE__
	alarm_handler();
#endif

#ifndef C128
	if (!memtab_r[PC]) {
		b = (*memrd[PC>>8])(PC);
		p1 = (*memrd[(PC+1)>>8])(PC+1);
		p2 = p1 | (((*memrd[(PC+2)>>8])(PC+2))<<8);
	} else {
		BYTE *pp = memtab_r[PC];
		b = pp[0];
		p1 = pp[1];
		p2 = p1 | (pp[2]<<8);
	}
#else
	b = LOAD(PC);
	p1 = LOAD(PC+1);
	p2 = p1 | (LOAD(PC+2) << 8);
#endif

#ifdef DEBUG
	instructions[b]++;

	if (traceflg)
	  printf(".%04x\t%s\n", PC, sprint_opcode(PC, 1));
	  /*  show(); */
#endif
#ifdef LED_ARRAY
	if (ledflg) {
	    int  eff;
	    ++ledarr[PC >> 12];
		    if ((eff = eff_address(PC, 1)) >= 0)
		++ledarr[eff >> 12];
	};
#endif


switch (b) {
case 0:	/* BRK,	IMPLIED,	M_NONE,	M_PC,	7,	0 */
{
    PC++;
	       /* Force break. */

#ifdef DEBUG
    if (verflg) {
	printf("BRK %ld\n", clk);
	show();
    }
#endif

    if (runflg) {
	--PC;
	printf("BRK\n");
	mon(PC);
	
    }
    if (trap_handler())
	PC = intr(I_BRK, PC);
    
    PC = (PC);;    clk += 7;
}

break;

case 1:	/* ORA,	INDIRECT_X,	M_INDX,	M_AC,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 6;
}

break;

case 2:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 3:	/* SLO,	INDIRECT_X,	M_INDX,	M_INDX,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = asl(src); AC = ora(src); */

#ifdef DEBUG_UNDOC
    puts ("SLOR -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 4:	/* NOOP,	ZERO_PAGE,	M_NONE,	M_NONE,	3,	0 */
{

    PC += 2;

        clk += 3;
}

break;

case 5:	/* ORA,	ZERO_PAGE,	M_ZERO,	M_AC,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 3;
}

break;

case 6:	/* ASL,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    SET_CARRY(src & 0x80);
    src <<= 1;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 7:	/* SLO,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = asl(src); AC = ora(src); */

#ifdef DEBUG_UNDOC
    puts ("SLOR -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 8:	/* PHP,	IMPLIED,	M_SR,	M_NONE,	3,	0 */
{
    src = GET_SR();
    PC++;

    PUSH(src);
    /* First push item onto stack and then decrement SP. */
        clk += 3;
}

break;

case 9:	/* ORA,	IMMEDIATE,	M_IMM,	M_AC,	2,	0 */
{
    src = p1;

    PC += 2;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 10:	/* ASL,	ACCUMULATOR,	M_AC,	M_AC,	2,	0 */
{
    src = AC;

    rmw_flag = 1;	/* Read-Modify-Write instruction */
    PC++;

    SET_CARRY(src & 0x80);
    src <<= 1;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
    rmw_flag = 0;
    clk += 2;
}

break;

case 11:	/* ANC,	IMMEDIATE,	M_ACIM,	M_ACNC,	2,	0 */
{
    src = (AC & p1);

    PC += 2;

#ifdef DEBUG_UNDOC
    puts ("ANC -- PC is now $%04X\n", PC);
#endif

#if 0
    src &= AC;
    SET_CARRY (src & 0x01);
    src >>= 1;
    SET_SIGN(src);
    SET_ZERO(src);
#else
    src &= AC;
    SET_ZERO(src);
    SET_SIGN(IF_CARRY() ? 1 : 0);
#endif
    
    AC = (src);;    clk += 2;
}

break;

case 12:	/* NOOP,	ABSOLUTE,	M_NONE,	M_NONE,	4,	0 */
{

    PC += 3;

        clk += 4;
}

break;

case 13:	/* ORA,	ABSOLUTE,	M_ABS,	M_AC,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 4;
}

break;

case 14:	/* ASL,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    SET_CARRY(src & 0x80);
    src <<= 1;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 15:	/* SLO,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = asl(src); AC = ora(src); */

#ifdef DEBUG_UNDOC
    puts ("SLOR -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 16:	/* BPL,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if sign flag is clear. */
    if (!IF_SIGN()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 17:	/* ORA,	INDIRECT_Y,	M_INDY,	M_AC,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 18:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 19:	/* SLO,	INDIRECT_Y,	M_INDY,	M_INDY,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1) + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = asl(src); AC = ora(src); */

#ifdef DEBUG_UNDOC
    puts ("SLOR -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 20:	/* NOOP,	ZERO_PAGE_X,	M_NONE,	M_NONE,	4,	0 */
{

    PC += 2;

        clk += 4;
}

break;

case 21:	/* ORA,	ZERO_PAGE_X,	M_ZERX,	M_AC,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 4;
}

break;

case 22:	/* ASL,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    SET_CARRY(src & 0x80);
    src <<= 1;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 23:	/* SLO,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = asl(src); AC = ora(src); */

#ifdef DEBUG_UNDOC
    puts ("SLOR -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 24:	/* CLC,	IMPLIED,	M_NONE,	M_FC,	2,	0 */
{
    PC++;

    
    SET_CARRY((0));;    clk += 2;
}

break;

case 25:	/* ORA,	ABSOLUTE_Y,	M_ABSY,	M_AC,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 26:	/* NOOP,	IMPLIED,	M_NONE,	M_NONE,	2,	0 */
{
    PC++;

        clk += 2;
}

break;

case 27:	/* SLO,	ABSOLUTE_Y,	M_ABSY,	M_ABSY,	7,	0 */
{
    src = LOAD(p2 + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = asl(src); AC = ora(src); */

#ifdef DEBUG_UNDOC
    puts ("SLOR -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + YR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 28:	/* NOOP,	ABSOLUTE_X,	M_NONE,	M_NONE,	4,	1 */
{

    PC += 3;

        clk += 4;
}

break;

case 29:	/* ORA,	ABSOLUTE_X,	M_ABSX,	M_AC,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 30:	/* ASL,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    SET_CARRY(src & 0x80);
    src <<= 1;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 31:	/* SLO,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = asl(src); AC = ora(src); */

#ifdef DEBUG_UNDOC
    puts ("SLOR -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 32:	/* JSR,	ABSOLUTE,	M_ADDR,	M_PC,	6,	0 */
{
    src = p2;

    PC += 3;
	       /* Jump to subroutine. */
    PC--;
    PUSH((PC >> 8) & 0xff);	/* Push return address onto the stack. */
    PUSH(PC & 0xff);

#ifdef DEBUG
    if (traceflg)
	print_stack(SP);
#endif

    
    PC = (src);;    clk += 6;
}

break;

case 33:	/* AND,	INDIRECT_X,	M_INDX,	M_AC,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 6;
}

break;

case 34:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 35:	/* RLA,	INDIRECT_X,	M_INDX,	M_INDX,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = rol(src); AC = and(src); */

#ifdef DEBUG_UNDOC
    puts ("RLAN -- PC is now $%04X\n", PC);
#endif
  
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 36:	/* BIT,	ZERO_PAGE,	M_ZERO,	M_NONE,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    SET_SIGN(src);
    SET_OVERFLOW(0x40 & src);	/* Copy bit 6 to OVERFLOW flag. */
    SET_ZERO(src & AC);
        clk += 3;
}

break;

case 37:	/* AND,	ZERO_PAGE,	M_ZERO,	M_AC,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 3;
}

break;

case 38:	/* ROL,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 39:	/* RLA,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = rol(src); AC = and(src); */

#ifdef DEBUG_UNDOC
    puts ("RLAN -- PC is now $%04X\n", PC);
#endif
  
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 40:	/* PLP,	IMPLIED,	M_NONE,	M_SR,	4,	0 */
{
    PC++;

    /* First increment stack pointer and then pull item from stack. */
    src = PULL();
    
    SET_SR((src));;    clk += 4;
}

break;

case 41:	/* AND,	IMMEDIATE,	M_IMM,	M_AC,	2,	0 */
{
    src = p1;

    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 42:	/* ROL,	ACCUMULATOR,	M_AC,	M_AC,	2,	0 */
{
    src = AC;

    rmw_flag = 1;	/* Read-Modify-Write instruction */
    PC++;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
    rmw_flag = 0;
    clk += 2;
}

break;

case 43:	/* ANC,	IMMEDIATE,	M_ACIM,	M_ACNC,	2,	0 */
{
    src = (AC & p1);

    PC += 2;

#ifdef DEBUG_UNDOC
    puts ("ANC -- PC is now $%04X\n", PC);
#endif

#if 0
    src &= AC;
    SET_CARRY (src & 0x01);
    src >>= 1;
    SET_SIGN(src);
    SET_ZERO(src);
#else
    src &= AC;
    SET_ZERO(src);
    SET_SIGN(IF_CARRY() ? 1 : 0);
#endif
    
    AC = (src);;    clk += 2;
}

break;

case 44:	/* BIT,	ABSOLUTE,	M_ABS,	M_NONE,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    SET_SIGN(src);
    SET_OVERFLOW(0x40 & src);	/* Copy bit 6 to OVERFLOW flag. */
    SET_ZERO(src & AC);
        clk += 4;
}

break;

case 45:	/* AND,	ABSOLUTE,	M_ABS,	M_AC,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 4;
}

break;

case 46:	/* ROL,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 47:	/* RLA,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = rol(src); AC = and(src); */

#ifdef DEBUG_UNDOC
    puts ("RLAN -- PC is now $%04X\n", PC);
#endif
  
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 48:	/* BMI,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if sign flag is set. */
    if (IF_SIGN()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 49:	/* AND,	INDIRECT_Y,	M_INDY,	M_AC,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 50:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 51:	/* RLA,	INDIRECT_Y,	M_INDY,	M_INDY,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1) + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = rol(src); AC = and(src); */

#ifdef DEBUG_UNDOC
    puts ("RLAN -- PC is now $%04X\n", PC);
#endif
  
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 52:	/* NOOP,	ZERO_PAGE_X,	M_NONE,	M_NONE,	4,	0 */
{

    PC += 2;

        clk += 4;
}

break;

case 53:	/* AND,	ZERO_PAGE_X,	M_ZERX,	M_AC,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 4;
}

break;

case 54:	/* ROL,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 55:	/* RLA,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = rol(src); AC = and(src); */

#ifdef DEBUG_UNDOC
    puts ("RLAN -- PC is now $%04X\n", PC);
#endif
  
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 56:	/* SEC,	IMPLIED,	M_NONE,	M_FC,	2,	0 */
{
    PC++;

    
    SET_CARRY((1));;    clk += 2;
}

break;

case 57:	/* AND,	ABSOLUTE_Y,	M_ABSY,	M_AC,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 58:	/* NOOP,	IMPLIED,	M_NONE,	M_NONE,	2,	0 */
{
    PC++;

        clk += 2;
}

break;

case 59:	/* RLA,	ABSOLUTE_Y,	M_ABSY,	M_ABSY,	7,	0 */
{
    src = LOAD(p2 + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = rol(src); AC = and(src); */

#ifdef DEBUG_UNDOC
    puts ("RLAN -- PC is now $%04X\n", PC);
#endif
  
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(p2 + YR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 60:	/* NOOP,	ABSOLUTE_X,	M_NONE,	M_NONE,	4,	1 */
{

    PC += 3;

        clk += 4;
}

break;

case 61:	/* AND,	ABSOLUTE_X,	M_ABSX,	M_AC,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 62:	/* ROL,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 63:	/* RLA,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = rol(src); AC = and(src); */

#ifdef DEBUG_UNDOC
    puts ("RLAN -- PC is now $%04X\n", PC);
#endif
  
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 64:	/* RTI,	IMPLIED,	M_NONE,	M_PC,	6,	0 */
{
    PC++;
	       /* Return from interrupt. */
    /* Load program status and program counter from stack. */
    src = PULL();
    SET_SR(src);
    src = PULL();
    src |= (PULL() << 8);	/* Load return address from stack. */

#ifdef DEBUG
    if (traceflg)
	print_stack(SP);
#endif

    
    PC = (src);;    clk += 6;
}

break;

case 65:	/* EOR,	INDIRECT_X,	M_INDX,	M_AC,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;    clk += 6;
}

break;

case 66:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 67:	/* SRE,	INDIRECT_X,	M_INDX,	M_INDX,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = lsr(src); AC = eor(src); */

#ifdef DEBUG_UNDOC
    puts ("SREO -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src &= 0xff;

    AC ^= src;		/* JF */
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 68:	/* NOOP,	ZERO_PAGE,	M_NONE,	M_NONE,	3,	0 */
{

    PC += 2;

        clk += 3;
}

break;

case 69:	/* EOR,	ZERO_PAGE,	M_ZERO,	M_AC,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;    clk += 3;
}

break;

case 70:	/* LSR,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 71:	/* SRE,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = lsr(src); AC = eor(src); */

#ifdef DEBUG_UNDOC
    puts ("SREO -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src &= 0xff;

    AC ^= src;		/* JF */
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 72:	/* PHA,	IMPLIED,	M_AC,	M_NONE,	3,	0 */
{
    src = AC;
    PC++;

    PUSH(src);
    /* First push item onto stack and then decrement SP. */
        clk += 3;
}

break;

case 73:	/* EOR,	IMMEDIATE,	M_IMM,	M_AC,	2,	0 */
{
    src = p1;

    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;    clk += 2;
}

break;

case 74:	/* LSR,	ACCUMULATOR,	M_AC,	M_AC,	2,	0 */
{
    src = AC;

    rmw_flag = 1;	/* Read-Modify-Write instruction */
    PC++;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
    rmw_flag = 0;
    clk += 2;
}

break;

case 75:	/* ASR,	IMMEDIATE,	M_ACIM,	M_AC,	2,	0 */
{
    src = (AC & p1);

    PC += 2;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 76:	/* JMP,	ABSOLUTE,	M_ADDR,	M_PC,	3,	0 */
{
    src = p2;

    PC += 3;

    
    PC = (src);;    clk += 3;
}

break;

case 77:	/* EOR,	ABSOLUTE,	M_ABS,	M_AC,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;    clk += 4;
}

break;

case 78:	/* LSR,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 79:	/* SRE,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = lsr(src); AC = eor(src); */

#ifdef DEBUG_UNDOC
    puts ("SREO -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src &= 0xff;

    AC ^= src;		/* JF */
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 80:	/* BVC,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if overflow flag is clear. */
    if (!IF_OVERFLOW()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 81:	/* EOR,	INDIRECT_Y,	M_INDY,	M_AC,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 82:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 83:	/* SRE,	INDIRECT_Y,	M_INDY,	M_INDY,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1) + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = lsr(src); AC = eor(src); */

#ifdef DEBUG_UNDOC
    puts ("SREO -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src &= 0xff;

    AC ^= src;		/* JF */
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 84:	/* NOOP,	ZERO_PAGE_X,	M_NONE,	M_NONE,	4,	0 */
{

    PC += 2;

        clk += 4;
}

break;

case 85:	/* EOR,	ZERO_PAGE_X,	M_ZERX,	M_AC,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;    clk += 4;
}

break;

case 86:	/* LSR,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 87:	/* SRE,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = lsr(src); AC = eor(src); */

#ifdef DEBUG_UNDOC
    puts ("SREO -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src &= 0xff;

    AC ^= src;		/* JF */
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 88:	/* CLI,	IMPLIED,	M_NONE,	M_FI,	2,	0 */
{
    PC++;

    
    SET_INTERRUPT((0));;    clk += 2;
}

break;

case 89:	/* EOR,	ABSOLUTE_Y,	M_ABSY,	M_AC,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 90:	/* NOOP,	IMPLIED,	M_NONE,	M_NONE,	2,	0 */
{
    PC++;

        clk += 2;
}

break;

case 91:	/* SRE,	ABSOLUTE_Y,	M_ABSY,	M_ABSY,	7,	0 */
{
    src = LOAD(p2 + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = lsr(src); AC = eor(src); */

#ifdef DEBUG_UNDOC
    puts ("SREO -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src &= 0xff;

    AC ^= src;		/* JF */
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(p2 + YR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 92:	/* NOOP,	ABSOLUTE_X,	M_NONE,	M_NONE,	4,	1 */
{

    PC += 3;

        clk += 4;
}

break;

case 93:	/* EOR,	ABSOLUTE_X,	M_ABSX,	M_AC,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);;
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 94:	/* LSR,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 95:	/* SRE,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = lsr(src); AC = eor(src); */

#ifdef DEBUG_UNDOC
    puts ("SREO -- PC is now $%04X\n", PC);
#endif

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src &= 0xff;

    AC ^= src;		/* JF */
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 96:	/* RTS,	IMPLIED,	M_NONE,	M_PC,	6,	0 */
{
    PC++;
	       /* Return from subroutine. */
    src = PULL();
    src += ((PULL()) << 8) + 1;	/* Load return address from stack and add 1. */

#ifdef DEBUG
    if (traceflg)
	print_stack(SP);
#endif

    
    PC = (src);;    clk += 6;
}

break;

case 97:	/* ADC,	INDIRECT_X,	M_INDX,	M_AC,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;    clk += 6;
}

break;

case 98:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 99:	/* RRA,	INDIRECT_X,	M_INDX,	M_INDX,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

#ifdef DEBUG_UNDOC
    puts ("RRAD -- PC is now $%04X\n", PC);
#endif

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(LOAD_ZERO_ADDR(p1 + XR), ((BYTE) temp));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 100:	/* NOOP,	ZERO_PAGE,	M_NONE,	M_NONE,	3,	0 */
{

    PC += 2;

        clk += 3;
}

break;

case 101:	/* ADC,	ZERO_PAGE,	M_ZERO,	M_AC,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;    clk += 3;
}

break;

case 102:	/* ROR,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 103:	/* RRA,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

#ifdef DEBUG_UNDOC
    puts ("RRAD -- PC is now $%04X\n", PC);
#endif

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE_ZERO(p1, ((BYTE) temp));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 104:	/* PLA,	IMPLIED,	M_NONE,	M_AC,	4,	0 */
{
    PC++;

    /* First increment stack pointer and then pull item from stack. */
    src = PULL();
    SET_SIGN(src);	/* Change sign and zero flag accordingly. */
    SET_ZERO(src);
    
    AC = (src);;    clk += 4;
}

break;

case 105:	/* ADC,	IMMEDIATE,	M_IMM,	M_AC,	2,	0 */
{
    src = p1;

    PC += 2;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;    clk += 2;
}

break;

case 106:	/* ROR,	ACCUMULATOR,	M_AC,	M_AC,	2,	0 */
{
    src = AC;

    rmw_flag = 1;	/* Read-Modify-Write instruction */
    PC++;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
    rmw_flag = 0;
    clk += 2;
}

break;

case 107:	/* ARR,	IMMEDIATE,	M_ACIM,	M_AC,	2,	0 */
{
    src = (AC & p1);

    PC += 2;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 108:	/* JMP,	ABS_INDIRECT,	M_AIND,	M_PC,	5,	0 */
{
    src = (LOAD(p2)) + (LOAD((p2&0xff00)+((p2+1)&0xff)) <<8);

    PC += 3;

    
    PC = (src);;    clk += 5;
}

break;

case 109:	/* ADC,	ABSOLUTE,	M_ABS,	M_AC,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;    clk += 4;
}

break;

case 110:	/* ROR,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 111:	/* RRA,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

#ifdef DEBUG_UNDOC
    puts ("RRAD -- PC is now $%04X\n", PC);
#endif

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(p2, ((BYTE) temp));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 112:	/* BVS,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if overflow flag is set. */
    if (IF_OVERFLOW()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 113:	/* ADC,	INDIRECT_Y,	M_INDY,	M_AC,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 114:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 115:	/* RRA,	INDIRECT_Y,	M_INDY,	M_INDY,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1) + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

#ifdef DEBUG_UNDOC
    puts ("RRAD -- PC is now $%04X\n", PC);
#endif

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(LOAD_ZERO_ADDR(p1) + YR, ((BYTE) temp));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 116:	/* NOOP,	ZERO_PAGE_X,	M_NONE,	M_NONE,	4,	0 */
{

    PC += 2;

        clk += 4;
}

break;

case 117:	/* ADC,	ZERO_PAGE_X,	M_ZERX,	M_AC,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;    clk += 4;
}

break;

case 118:	/* ROR,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 119:	/* RRA,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

#ifdef DEBUG_UNDOC
    puts ("RRAD -- PC is now $%04X\n", PC);
#endif

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE_ZERO(p1 + XR, ((BYTE) temp));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 120:	/* SEI,	IMPLIED,	M_NONE,	M_FI,	2,	0 */
{
    PC++;

    
    SET_INTERRUPT((1));;    clk += 2;
}

break;

case 121:	/* ADC,	ABSOLUTE_Y,	M_ABSY,	M_AC,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 122:	/* NOOP,	IMPLIED,	M_NONE,	M_NONE,	2,	0 */
{
    PC++;

        clk += 2;
}

break;

case 123:	/* RRA,	ABSOLUTE_Y,	M_ABSY,	M_ABSY,	7,	0 */
{
    src = LOAD(p2 + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

#ifdef DEBUG_UNDOC
    puts ("RRAD -- PC is now $%04X\n", PC);
#endif

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(p2 + YR, ((BYTE) temp));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 124:	/* NOOP,	ABSOLUTE_X,	M_NONE,	M_NONE,	4,	1 */
{

    PC += 3;

        clk += 4;
}

break;

case 125:	/* ADC,	ABSOLUTE_X,	M_ABSX,	M_AC,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    temp = src + AC + (IF_CARRY() ? 1 : 0);

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);;
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 126:	/* ROR,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 127:	/* RRA,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

#ifdef DEBUG_UNDOC
    puts ("RRAD -- PC is now $%04X\n", PC);
#endif

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(p2 + XR, ((BYTE) temp));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 128:	/* NOOP,	IMMEDIATE,	M_NONE,	M_NONE,	2,	0 */
{

    PC += 2;

        clk += 2;
}

break;

case 129:	/* STA,	INDIRECT_X,	M_AC,	M_INDX,	6,	0 */
{
    src = AC;

    PC += 2;

    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));;    clk += 6;
}

break;

case 130:	/* NOOP,	IMMEDIATE,	M_NONE,	M_NONE,	2,	0 */
{

    PC += 2;

        clk += 2;
}

break;

case 131:	/* SAX,	INDIRECT_X,	M_ANXR,	M_INDX,	6,	0 */
{
    src = (AC & XR);

    PC += 2;

    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));;    clk += 6;
}

break;

case 132:	/* STY,	ZERO_PAGE,	M_YR,	M_ZERO,	3,	0 */
{
    src = YR;

    PC += 2;

    
    STORE_ZERO(p1, (src));;    clk += 3;
}

break;

case 133:	/* STA,	ZERO_PAGE,	M_AC,	M_ZERO,	3,	0 */
{
    src = AC;

    PC += 2;

    
    STORE_ZERO(p1, (src));;    clk += 3;
}

break;

case 134:	/* STX,	ZERO_PAGE,	M_XR,	M_ZERO,	3,	0 */
{
    src = XR;

    PC += 2;

    
    STORE_ZERO(p1, (src));;    clk += 3;
}

break;

case 135:	/* SAX,	ZERO_PAGE,	M_ANXR,	M_ZERO,	3,	0 */
{
    src = (AC & XR);

    PC += 2;

    
    STORE_ZERO(p1, (src));;    clk += 3;
}

break;

case 136:	/* DEY,	IMPLIED,	M_YR,	M_YR,	2,	0 */
{
    src = YR;
    PC++;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;    clk += 2;
}

break;

case 137:	/* NOOP,	IMMEDIATE,	M_NONE,	M_NONE,	2,	0 */
{

    PC += 2;

        clk += 2;
}

break;

case 138:	/* TXA,	IMPLIED,	M_XR,	M_AC,	2,	0 */
{
    src = XR;
    PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 139:	/* ANE,	IMMEDIATE,	M_AXIM,	M_AC,	2,	0 */
{
    src = ((AC | 0xee) & XR & p1);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 140:	/* STY,	ABSOLUTE,	M_YR,	M_ABS,	4,	0 */
{
    src = YR;

    PC += 3;

    
    STORE(p2, (src));;    clk += 4;
}

break;

case 141:	/* STA,	ABSOLUTE,	M_AC,	M_ABS,	4,	0 */
{
    src = AC;

    PC += 3;

    
    STORE(p2, (src));;    clk += 4;
}

break;

case 142:	/* STX,	ABSOLUTE,	M_XR,	M_ABS,	4,	0 */
{
    src = XR;

    PC += 3;

    
    STORE(p2, (src));;    clk += 4;
}

break;

case 143:	/* SAX,	ABSOLUTE,	M_ANXR,	M_ABS,	4,	0 */
{
    src = (AC & XR);

    PC += 3;

    
    STORE(p2, (src));;    clk += 4;
}

break;

case 144:	/* BCC,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if carry flag is clear. */
    if (!IF_CARRY()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 145:	/* STA,	INDIRECT_Y,	M_AC,	M_INDY,	6,	0 */
{
    src = AC;

    PC += 2;

    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));;    clk += 6;
}

break;

case 146:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 147:	/* SHA,	INDIRECT_Y,	M_ANXR,	M_STH0,	6,	0 */
{
    src = (AC & XR);

    PC += 2;

    
    u_stoshr(src, LOAD_ZERO_ADDR(p1), YR);;    clk += 6;
}

break;

case 148:	/* STY,	ZERO_PAGE_X,	M_YR,	M_ZERX,	4,	0 */
{
    src = YR;

    PC += 2;

    
    STORE_ZERO(p1 + XR, (src));;    clk += 4;
}

break;

case 149:	/* STA,	ZERO_PAGE_X,	M_AC,	M_ZERX,	4,	0 */
{
    src = AC;

    PC += 2;

    
    STORE_ZERO(p1 + XR, (src));;    clk += 4;
}

break;

case 150:	/* STX,	ZERO_PAGE_Y,	M_XR,	M_ZERY,	4,	0 */
{
    src = XR;

    PC += 2;

    
    STORE_ZERO(p1 + YR, (src));;    clk += 4;
}

break;

case 151:	/* SAX,	ZERO_PAGE_Y,	M_ANXR,	M_ZERY,	4,	0 */
{
    src = (AC & XR);

    PC += 2;

    
    STORE_ZERO(p1 + YR, (src));;    clk += 4;
}

break;

case 152:	/* TYA,	IMPLIED,	M_YR,	M_AC,	2,	0 */
{
    src = YR;
    PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 153:	/* STA,	ABSOLUTE_Y,	M_AC,	M_ABSY,	5,	0 */
{
    src = AC;

    PC += 3;

    
    STORE(p2 + YR, (src));;    clk += 5;
}

break;

case 154:	/* TXS,	IMPLIED,	M_XR,	M_SP,	2,	0 */
{
    src = XR;
    PC++;

    
    SP = (src);;    clk += 2;
}

break;

case 155:	/* SHS,	ABSOLUTE_Y,	M_ANXR,	M_STH3,	5,	0 */
{
    src = (AC & XR);

    PC += 3;

    
    SP = src;	/* SHS */;    clk += 5;
}

break;

case 156:	/* SHY,	ABSOLUTE_X,	M_YR,	M_STH2,	5,	0 */
{
    src = YR;

    PC += 3;

    
    u_stoshr(src, p2, XR);;    clk += 5;
}

break;

case 157:	/* STA,	ABSOLUTE_X,	M_AC,	M_ABSX,	5,	0 */
{
    src = AC;

    PC += 3;

    
    STORE(p2 + XR, (src));;    clk += 5;
}

break;

case 158:	/* SHX,	ABSOLUTE_Y,	M_XR,	M_STH1,	5,	0 */
{
    src = XR;

    PC += 3;

    
    u_stoshr(src, p2, YR);;    clk += 5;
}

break;

case 159:	/* SHA,	ABSOLUTE_Y,	M_ANXR,	M_STH1,	5,	0 */
{
    src = (AC & XR);

    PC += 3;

    
    u_stoshr(src, p2, YR);;    clk += 5;
}

break;

case 160:	/* LDY,	IMMEDIATE,	M_IMM,	M_YR,	2,	0 */
{
    src = p1;

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;    clk += 2;
}

break;

case 161:	/* LDA,	INDIRECT_X,	M_INDX,	M_AC,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 6;
}

break;

case 162:	/* LDX,	IMMEDIATE,	M_IMM,	M_XR,	2,	0 */
{
    src = p1;

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 2;
}

break;

case 163:	/* LAX,	INDIRECT_X,	M_INDX,	M_ACXR,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);;    clk += 6;
}

break;

case 164:	/* LDY,	ZERO_PAGE,	M_ZERO,	M_YR,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;    clk += 3;
}

break;

case 165:	/* LDA,	ZERO_PAGE,	M_ZERO,	M_AC,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 3;
}

break;

case 166:	/* LDX,	ZERO_PAGE,	M_ZERO,	M_XR,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 3;
}

break;

case 167:	/* LAX,	ZERO_PAGE,	M_ZERO,	M_ACXR,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);;    clk += 3;
}

break;

case 168:	/* TAY,	IMPLIED,	M_AC,	M_YR,	2,	0 */
{
    src = AC;
    PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;    clk += 2;
}

break;

case 169:	/* LDA,	IMMEDIATE,	M_IMM,	M_AC,	2,	0 */
{
    src = p1;

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 2;
}

break;

case 170:	/* TAX,	IMPLIED,	M_AC,	M_XR,	2,	0 */
{
    src = AC;
    PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 2;
}

break;

case 171:	/* LXA,	IMMEDIATE,	M_ACIM,	M_ACXR,	2,	0 */
{
    src = (AC & p1);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);;    clk += 2;
}

break;

case 172:	/* LDY,	ABSOLUTE,	M_ABS,	M_YR,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;    clk += 4;
}

break;

case 173:	/* LDA,	ABSOLUTE,	M_ABS,	M_AC,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 4;
}

break;

case 174:	/* LDX,	ABSOLUTE,	M_ABS,	M_XR,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 4;
}

break;

case 175:	/* LAX,	ABSOLUTE,	M_ABS,	M_ACXR,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);;    clk += 4;
}

break;

case 176:	/* BCS,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if carry flag is set. */
    if (IF_CARRY()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 177:	/* LDA,	INDIRECT_Y,	M_INDY,	M_AC,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 178:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 179:	/* LAX,	INDIRECT_Y,	M_INDY,	M_ACXR,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 180:	/* LDY,	ZERO_PAGE_X,	M_ZERX,	M_YR,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;    clk += 4;
}

break;

case 181:	/* LDA,	ZERO_PAGE_X,	M_ZERX,	M_AC,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;    clk += 4;
}

break;

case 182:	/* LDX,	ZERO_PAGE_Y,	M_ZERY,	M_XR,	4,	0 */
{
    src = LOAD_ZERO(p1 + YR);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 4;
}

break;

case 183:	/* LAX,	ZERO_PAGE_Y,	M_ZERY,	M_ACXR,	4,	0 */
{
    src = LOAD_ZERO(p1 + YR);

    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);;    clk += 4;
}

break;

case 184:	/* CLV,	IMPLIED,	M_NONE,	M_FV,	2,	0 */
{
    PC++;

    
    SET_OVERFLOW((0));;    clk += 2;
}

break;

case 185:	/* LDA,	ABSOLUTE_Y,	M_ABSY,	M_AC,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 186:	/* TSX,	IMPLIED,	M_SP,	M_XR,	2,	0 */
{
    src = SP;
    PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 2;
}

break;

case 187:	/* LAS,	ABSOLUTE_Y,	M_SABY,	M_ACXS,	4,	1 */
{
    src = (SP & LOAD(p2 + YR));

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = SP = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 188:	/* LDY,	ABSOLUTE_X,	M_ABSX,	M_YR,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 189:	/* LDA,	ABSOLUTE_X,	M_ABSX,	M_AC,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 190:	/* LDX,	ABSOLUTE_Y,	M_ABSY,	M_XR,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 191:	/* LAX,	ABSOLUTE_Y,	M_ABSY,	M_ACXR,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 192:	/* CPY,	IMMEDIATE,	M_IMM,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;

    src = YR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 2;
}

break;

case 193:	/* CMP,	INDIRECT_X,	M_INDX,	M_NONE,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 6;
}

break;

case 194:	/* NOOP,	IMMEDIATE,	M_NONE,	M_NONE,	2,	0 */
{

    PC += 2;

        clk += 2;
}

break;

case 195:	/* DCP,	INDIRECT_X,	M_INDX,	M_INDX,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* cmp(--src & 0xff));  */

#ifdef DEBUG_UNDOC
    puts ("DECP -- PC is now $%04X\n", PC);
#endif

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 196:	/* CPY,	ZERO_PAGE,	M_ZERO,	M_NONE,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    src = YR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 3;
}

break;

case 197:	/* CMP,	ZERO_PAGE,	M_ZERO,	M_NONE,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 3;
}

break;

case 198:	/* DEC,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 199:	/* DCP,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* cmp(--src & 0xff));  */

#ifdef DEBUG_UNDOC
    puts ("DECP -- PC is now $%04X\n", PC);
#endif

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 200:	/* INY,	IMPLIED,	M_YR,	M_YR,	2,	0 */
{
    src = YR;
    PC++;

    src = (src + 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);;    clk += 2;
}

break;

case 201:	/* CMP,	IMMEDIATE,	M_IMM,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 2;
}

break;

case 202:	/* DEX,	IMPLIED,	M_XR,	M_XR,	2,	0 */
{
    src = XR;
    PC++;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 2;
}

break;

case 203:	/* SBX,	IMMEDIATE,	M_IMM,	M_XR,	2,	0 */
{
    src = p1;

    PC += 2;

#ifdef DEBUG_UNDOC
    puts ("SBX -- PC is now $%04X\n", PC);
#endif

    src = (AC & XR) - src;	/* Carry is ignored (CMP) */
    /* Overflow flag may be affected */
    SET_CARRY(src < 0x100);

    src &= 0xff;	/* No decimal mode */
    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 2;
}

break;

case 204:	/* CPY,	ABSOLUTE,	M_ABS,	M_NONE,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    src = YR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 4;
}

break;

case 205:	/* CMP,	ABSOLUTE,	M_ABS,	M_NONE,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 4;
}

break;

case 206:	/* DEC,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 207:	/* DCP,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* cmp(--src & 0xff));  */

#ifdef DEBUG_UNDOC
    puts ("DECP -- PC is now $%04X\n", PC);
#endif

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 208:	/* BNE,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if  zero flag is clear. */
    if (!IF_ZERO()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 209:	/* CMP,	INDIRECT_Y,	M_INDY,	M_NONE,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
    
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 210:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 211:	/* DCP,	INDIRECT_Y,	M_INDY,	M_INDY,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1) + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* cmp(--src & 0xff));  */

#ifdef DEBUG_UNDOC
    puts ("DECP -- PC is now $%04X\n", PC);
#endif

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 212:	/* NOOP,	ZERO_PAGE_X,	M_NONE,	M_NONE,	4,	0 */
{

    PC += 2;

        clk += 4;
}

break;

case 213:	/* CMP,	ZERO_PAGE_X,	M_ZERX,	M_NONE,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 4;
}

break;

case 214:	/* DEC,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 215:	/* DCP,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* cmp(--src & 0xff));  */

#ifdef DEBUG_UNDOC
    puts ("DECP -- PC is now $%04X\n", PC);
#endif

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 216:	/* CLD,	IMPLIED,	M_NONE,	M_FD,	2,	0 */
{
    PC++;

    
    SET_DECIMAL((0));;    clk += 2;
}

break;

case 217:	/* CMP,	ABSOLUTE_Y,	M_ABSY,	M_NONE,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
    
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 218:	/* NOOP,	IMPLIED,	M_NONE,	M_NONE,	2,	0 */
{
    PC++;

        clk += 2;
}

break;

case 219:	/* DCP,	ABSOLUTE_Y,	M_ABSY,	M_ABSY,	7,	0 */
{
    src = LOAD(p2 + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* cmp(--src & 0xff));  */

#ifdef DEBUG_UNDOC
    puts ("DECP -- PC is now $%04X\n", PC);
#endif

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(p2 + YR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 220:	/* NOOP,	ABSOLUTE_X,	M_NONE,	M_NONE,	4,	1 */
{

    PC += 3;

        clk += 4;
}

break;

case 221:	/* CMP,	ABSOLUTE_X,	M_ABSX,	M_NONE,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
    
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 222:	/* DEC,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 223:	/* DCP,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* cmp(--src & 0xff));  */

#ifdef DEBUG_UNDOC
    puts ("DECP -- PC is now $%04X\n", PC);
#endif

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 224:	/* CPX,	IMMEDIATE,	M_IMM,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;

    src = XR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 2;
}

break;

case 225:	/* SBC,	INDIRECT_X,	M_INDX,	M_AC,	6,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;    clk += 6;
}

break;

case 226:	/* NOOP,	IMMEDIATE,	M_NONE,	M_NONE,	2,	0 */
{

    PC += 2;

        clk += 2;
}

break;

case 227:	/* ISB,	INDIRECT_X,	M_INDX,	M_INDX,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1 + XR));

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ++src & 0xff; AC = sbc(src); */

#ifdef DEBUG_UNDOC
    puts ("ISBC -- PC is now $%04X\n", PC);
#endif

    src = ((src + 1) & 0xff);	/* INC+SBC */

    temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 228:	/* CPX,	ZERO_PAGE,	M_ZERO,	M_NONE,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    src = XR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 3;
}

break;

case 229:	/* SBC,	ZERO_PAGE,	M_ZERO,	M_AC,	3,	0 */
{
    src = LOAD_ZERO(p1);

    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;    clk += 3;
}

break;

case 230:	/* INC,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    src = (src + 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 231:	/* ISB,	ZERO_PAGE,	M_ZERO,	M_ZERO,	5,	0 */
{
    src = LOAD_ZERO(p1);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ++src & 0xff; AC = sbc(src); */

#ifdef DEBUG_UNDOC
    puts ("ISBC -- PC is now $%04X\n", PC);
#endif

    src = ((src + 1) & 0xff);	/* INC+SBC */

    temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE_ZERO(p1, (src));;
    rmw_flag = 0;
    clk += 5;
}

break;

case 232:	/* INX,	IMPLIED,	M_XR,	M_XR,	2,	0 */
{
    src = XR;
    PC++;

    src = (src + 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);;    clk += 2;
}

break;

case 233:	/* SBC,	IMMEDIATE,	M_IMM,	M_AC,	2,	0 */
{
    src = p1;

    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;    clk += 2;
}

break;

case 234:	/* NOP,	IMPLIED,	M_NONE,	M_NONE,	2,	0 */
{
    PC++;

        clk += 2;
}

break;

case 235:	/* USBC,	IMMEDIATE,	M_IMM,	M_AC,	2,	0 */
{
    src = p1;

    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;    clk += 2;
}

break;

case 236:	/* CPX,	ABSOLUTE,	M_ABS,	M_NONE,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    src = XR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
        clk += 4;
}

break;

case 237:	/* SBC,	ABSOLUTE,	M_ABS,	M_AC,	4,	0 */
{
    src = LOAD(p2);

    PC += 3;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;    clk += 4;
}

break;

case 238:	/* INC,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    src = (src + 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 239:	/* ISB,	ABSOLUTE,	M_ABS,	M_ABS,	6,	0 */
{
    src = LOAD(p2);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = ++src & 0xff; AC = sbc(src); */

#ifdef DEBUG_UNDOC
    puts ("ISBC -- PC is now $%04X\n", PC);
#endif

    src = ((src + 1) & 0xff);	/* INC+SBC */

    temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE(p2, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 240:	/* BEQ,	RELATIVE,	M_REL,	M_NONE,	2,	0 */
{
    src = p1;

    PC += 2;
	       /* Branch if  zero flag is set. */
    if (IF_ZERO()) {
	clk += ((PC & 0xFF00) != (REL_ADDR(PC, src) & 0xFF00) ? 2 : 1);
	PC = REL_ADDR(PC, src);
    }
        clk += 2;
}

break;

case 241:	/* SBC,	INDIRECT_Y,	M_INDY,	M_AC,	5,	1 */
{

    p2 = LOAD_ZERO_ADDR(p1);
    src = LOAD(p2 + YR);

    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 5;
}

break;

case 242:	/* JAM,	IMPLIED,	M_NONE,	M_NONE,	0,	0 */
{
    PC++;
	       /* No such opcode. */
    (void) printf("Illegal instruction.\n");
    verflg = 1;
    --PC;
    show();
    mon(PC);
    }

break;

case 243:	/* ISB,	INDIRECT_Y,	M_INDY,	M_INDY,	8,	0 */
{
    src = LOAD(LOAD_ZERO_ADDR(p1) + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ++src & 0xff; AC = sbc(src); */

#ifdef DEBUG_UNDOC
    puts ("ISBC -- PC is now $%04X\n", PC);
#endif

    src = ((src + 1) & 0xff);	/* INC+SBC */

    temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));;
    rmw_flag = 0;
    clk += 8;
}

break;

case 244:	/* NOOP,	ZERO_PAGE_X,	M_NONE,	M_NONE,	4,	0 */
{

    PC += 2;

        clk += 4;
}

break;

case 245:	/* SBC,	ZERO_PAGE_X,	M_ZERX,	M_AC,	4,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;    clk += 4;
}

break;

case 246:	/* INC,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    src = (src + 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 247:	/* ISB,	ZERO_PAGE_X,	M_ZERX,	M_ZERX,	6,	0 */
{
    src = LOAD_ZERO(p1 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 2;

    /* src = ++src & 0xff; AC = sbc(src); */

#ifdef DEBUG_UNDOC
    puts ("ISBC -- PC is now $%04X\n", PC);
#endif

    src = ((src + 1) & 0xff);	/* INC+SBC */

    temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE_ZERO(p1 + XR, (src));;
    rmw_flag = 0;
    clk += 6;
}

break;

case 248:	/* SED,	IMPLIED,	M_NONE,	M_FD,	2,	0 */
{
    PC++;

    
    SET_DECIMAL((1));;    clk += 2;
}

break;

case 249:	/* SBC,	ABSOLUTE_Y,	M_ABSY,	M_AC,	4,	1 */
{
    src = LOAD(p2 + YR);

    PC += 3;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;
	if ((p2 & 0xFF) + YR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 250:	/* NOOP,	IMPLIED,	M_NONE,	M_NONE,	2,	0 */
{
    PC++;

        clk += 2;
}

break;

case 251:	/* ISB,	ABSOLUTE_Y,	M_ABSY,	M_ABSY,	7,	0 */
{
    src = LOAD(p2 + YR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = ++src & 0xff; AC = sbc(src); */

#ifdef DEBUG_UNDOC
    puts ("ISBC -- PC is now $%04X\n", PC);
#endif

    src = ((src + 1) & 0xff);	/* INC+SBC */

    temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE(p2 + YR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 252:	/* NOOP,	ABSOLUTE_X,	M_NONE,	M_NONE,	4,	1 */
{

    PC += 3;

        clk += 4;
}

break;

case 253:	/* SBC,	ABSOLUTE_X,	M_ABSX,	M_AC,	4,	1 */
{
    src = LOAD(p2 + XR);

    PC += 3;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    src ^= 0xff;
    temp = AC + src + (IF_CARRY() ? 1 : 0);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
#if 0
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
#endif
	if ((temp & 0xf) > 0x9)
	    temp -= 6;
	if ((temp & 0xf0) > 0x90)
	    temp -= 0x60;
    }
    SET_CARRY(temp >= 0x100);
    
    AC = (temp & 0xff);;
	if ((p2 & 0xFF) + XR > 0x100)	/* PBC */
		++clk;
    clk += 4;
}

break;

case 254:	/* INC,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    src = (src + 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;

case 255:	/* ISB,	ABSOLUTE_X,	M_ABSX,	M_ABSX,	7,	0 */
{
    src = LOAD(p2 + XR);

    rmw_flag = 1;	/* Read-Modify-Write instruction */

    PC += 3;

    /* src = ++src & 0xff; AC = sbc(src); */

#ifdef DEBUG_UNDOC
    puts ("ISBC -- PC is now $%04X\n", PC);
#endif

    src = ((src + 1) & 0xff);	/* INC+SBC */

    temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) - (IF_CARRY() ? 0 : 1)) < (src & 0xf))	/* EP */
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE(p2 + XR, (src));;
    rmw_flag = 0;
    clk += 7;
}

break;



}
}
}
#ifndef NO_UNDOC_CMDS

void    u_stoshr(unsigned int val, ADDRESS address, BYTE index)
{
     val &= (((address >> 8) + 1) & 0xff);
    if (((address & 0xff) + index) > 0xff)
	address = val;
    STORE((address + index), val);
}
#endif   /* UNDOC */

