/*
 * $Id: patch128.c,v 1.3 1995/11/07 16:51:00 jopi Exp jopi $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 * Written by
 *   Jouko Valta    (jopi@stekt.oulu.fi)
 *
 * Patches by
 *   Peter Weighill (stuce@csv.warwick.ac.uk)
 *
 *
 * $Log: patch128.c,v $
 * Revision 1.3  1995/11/07  16:51:00  jopi
 * Incorrect bytecount fixed
 *
 * Revision 1.2  1995/04/01  07:54:09  jopi
 * X64 0.3 PL 0
 * Kernel revision patches added: German, Swedish, and Usa.
 *
 * Revision 1.1  1994/12/12  16:59:44  jopi
 * Initial revision
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "macro.h"  /* includes also config.h */
#include "memory.h"

extern BYTE  load_rom ( ADDRESS );
extern void  save_rom ( ADDRESS, BYTE );


/*
 * By an option on the x128 command line you can patch between the
 * different roms.
 *
 *   x128 -R 85		BASIC ROM  1985 revision
 *   x128 -R 86		BASIC ROM  1986 revision
 */

/*
 * Table of differences between the two versions of the C128 Basic V7.0 ROM.
 * To use your own patches, just add the data following the format used
 * below. (Note: Each block must contain data for each patch version.)
 */


#define BASIC_PATCH_VERSIONS  1   /* counting from 0 */

static unsigned short basic_patch_bytes[] = {

  1, 0x41F0,
	0x35,
	0x36,

  3, 0x4D70,
	0x8e, 0x0c, 0x12,
	0x20, 0xd1, 0x7e,

  2, 0x5C87,
	0x8f, 0x5c,
	0x8e, 0x7e,

 14, 0x5ED7,
	0xc8, 0xd0, 0xf8, 0xe6, 0x62, 0xe6, 0x27, 0xad, 0x11, 0x12, 0xc5, 0x27, 0xb0, 0xed,
	0xe6, 0x61, 0xd0, 0x02, 0xe6, 0x62, 0x20, 0x94, 0x7e, 0x90, 0xf0, 0xea, 0xea, 0xea,

  3, 0x5EF8,
	0x4c, 0x37, 0x4d,
	0x60, 0x00, 0x00,

  1, 0x5F02,
	0x0c,
	0x2e,

 16, 0x66A5,
	0x8d, 0x57, 0x11, 0x08, 0xa2, 0x23, 0x20, 0x4a, 0x9d, 0x28, 0xb0, 0x11, 0xad, 0x54, 0x11, 0x8d,
	0xb0, 0x11, 0xad, 0x54, 0x11, 0x8d, 0x56, 0x11, 0xad, 0x55, 0x11, 0x24, 0xd8, 0x10, 0x04, 0x0e,

 14, 0x66B7,
	0xad, 0x55, 0x11, 0x24, 0xd8, 0x10, 0x04, 0x0e, 0x56, 0x11, 0x2a, 0x8d, 0x57, 0x11,
	0x2a, 0x8d, 0x57, 0x11, 0xa2, 0x23, 0x20, 0x4a, 0x9d, 0xea, 0xea, 0xea, 0xea, 0xea,

  2, 0x6839,
	0x8d, 0x92,
	0x82, 0x7e,

  2, 0x6845,
	0x0c, 0xc0,
	0x88, 0x7e,

  2, 0x6860,
	0x0c, 0xc0,
	0x88, 0x7e,

  3, 0x6F45,
	0xbd, 0x05, 0x70,
	0x20, 0xa6, 0x7e,

 24, 0x6FF9,
	0x2f, 0xb6, 0x83, 0x99, 0xfc, 0xb1, 0xbd, 0x25, 0xef, 0x20, 0xbe, 0xd1, 0x4c, 0x50, 0x55, 0x5a,
	0x5f, 0x65, 0x6b, 0x72, 0x78, 0x80, 0x87, 0x8f,

	0x0f, 0x0c, 0x45, 0xbf, 0x7d, 0x83, 0xd6, 0x79, 0x73, 0xc7, 0x7c, 0x97, 0x43, 0x47, 0x4b, 0x4f,
	0x54, 0x59, 0x5e, 0x64, 0x6a, 0x70, 0x77, 0x7e,

  2, 0x7A31,
	0xb7, 0xff,
	0x51, 0x92,

 87, 0x7E82,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,

	0x20, 0x45, 0xa8, 0x4c, 0xf0, 0xff, 0x20, 0x45, 0xa8, 0x4c, 0x0c, 0xc0, 0x20, 0x8f, 0x5c, 0x4c,
	0x86, 0x03, 0xe6, 0x26, 0xd0, 0x02, 0xe6, 0x27, 0x38, 0xa5, 0x26, 0xed, 0x10, 0x12, 0xa5, 0x27,
	0xed, 0x11, 0x12, 0x60, 0x2c, 0x03, 0x0a, 0x30, 0x04, 0xbd, 0x05, 0x70, 0x60, 0xbd, 0xb9, 0x7e,
	0x8d, 0x2d, 0x12, 0xbd, 0xc5, 0x7e, 0x60, 0x9c, 0xbf, 0x22, 0xc8, 0xb4, 0xeb, 0x71, 0x4c, 0x7f,
	0x12, 0x08, 0x68, 0x45, 0x49, 0x4e, 0x52, 0x57, 0x5c, 0x62, 0x68, 0x6e, 0x75, 0x7c, 0x83, 0x8e,
	0x0c, 0x12, 0xa2, 0x1b, 0x86, 0x18, 0x60,

 56, 0x7FC0,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,

	0x28, 0x43, 0x29, 0x31, 0x39, 0x38, 0x36, 0x20, 0x43, 0x4f, 0x4d, 0x4d, 0x4f, 0x44, 0x4f, 0x52,
	0x45, 0x20, 0x45, 0x4c, 0x45, 0x43, 0x54, 0x52, 0x4f, 0x4e, 0x49, 0x43, 0x53, 0x2c, 0x20, 0x4c,
	0x54, 0x44, 0x2e, 0x20, 0x41, 0x4c, 0x4c, 0x20, 0x52, 0x49, 0x47, 0x48, 0x54, 0x53, 0x20, 0x52,
	0x45, 0x53, 0x45, 0x52, 0x56, 0x45, 0x44, 0x2e,

  4, 0x7FFC,
	0xff, 0xff, 0x00, 0x4c,
	0xef, 0x8d, 0x01, 0x61,

  1, 0x8323,
	0x10,
	0x08,

  1, 0x839C,
	0x10,
	0x08,

  1, 0x8A3C,
	0x55,
	0x5b,

  1, 0x8A4B,
	0x55,
	0x5b,

  1, 0x8A57,
	0x38,
	0xea,

  4, 0x98BB,
	0x38, 0x6e, 0x25, 0x01,
	0x20, 0x6e, 0xaa, 0xea,

  2, 0x9F18,
	0x12, 0x88,
	0xc0, 0x6d,

  4, 0xA153,
	0x38, 0x4c, 0x75, 0x92,
	0xea, 0x4c, 0xb2, 0xa3,

 19, 0xAA6E,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff,

	0x20, 0xd3, 0x42, 0xc9, 0x2c, 0xd0, 0x07, 0xad, 0x33, 0x01, 0x20, 0xeb, 0x98, 0xc8, 0x38, 0x6e,
	0x25, 0x01, 0x60,

 56, 0xBFC0,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,

	0x28, 0x43, 0x29, 0x31, 0x39, 0x38, 0x36, 0x20, 0x43, 0x4f, 0x4d, 0x4d, 0x4f, 0x44, 0x4f, 0x52,
	0x45, 0x20, 0x45, 0x4c, 0x45, 0x43, 0x54, 0x52, 0x4f, 0x4e, 0x49, 0x43, 0x53, 0x2c, 0x20, 0x4c,
	0x54, 0x44, 0x2e, 0x20, 0x41, 0x4c, 0x4c, 0x20, 0x52, 0x49, 0x47, 0x48, 0x54, 0x53, 0x20, 0x52,
	0x45, 0x53, 0x45, 0x52, 0x56, 0x45, 0x44, 0x2e,

  4, 0xBFFC,
	0xff, 0xff, 0x00, 0x3a,
	0xc8, 0xcd, 0x01, 0xc5,

  0, 00
};


/*
 * Table of differences between nationalized C128 Kernel ROM versions.
 * To use your national patches, just add the data following
 * the format used below. (Note: Each block must contain data for
 * each patch version.)
 *
 * Format:
 *	 C128  Editor R1/85 Swe
 *	 C128  Editor R1/85 German
 *	 C128  Editor R?/86 US
 */

#define KERNAL_PATCH_VERSIONS  2   /* counting from 0 */

static unsigned short kernal_patch_bytes[] = {

 /* Editor */

  2, 0xC013,
	0x87, 0xFC,
	0x87, 0xFC,
	0x5D, 0xC5,

  3, 0xC090,
	0x20, 0x80, 0xFC,
	0x20, 0x80, 0xFC,
	0x8D, 0x18, 0xD4,

 11, 0xC106,
	0x20, 0x81, 0xFE, 0xA9, 0x00, 0x8D, 0x3F, 0x03, 0x20, 0x87, 0xFC,
	0x20, 0x81, 0xFE, 0xA9, 0x00, 0x8D, 0x3F, 0x03, 0x20, 0x87, 0xFC,
	0xA2, 0x0B, 0xBD, 0x6F, 0xC0, 0x9D, 0x3E, 0x03, 0xCA, 0x10, 0xF7,

  2, 0xC22D,
	0x87, 0xFC,
	0x87, 0xFC,
	0x5D, 0xC5,

  3, 0xC6AA,
	0x4C, 0xC6, 0xFC,
	0x4C, 0xC6, 0xFC,
	0x6C, 0x3C, 0x03,

  3, 0xC6CA,
	0xBD, 0x00, 0x10,
	0xBD, 0x00, 0x10,
	0x4C, 0x00, 0xCF,

 25, 0xCF00,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

	0xA5, 0xD1, 0xD0, 0x12, 0xBD, 0x00, 0x10, 0x85, 0xD1, 0xA9, 0x00, 0x18, 0xCA, 0x30, 0x05, 0x7D,
	0x00, 0x10, 0x90, 0xF8, 0x85, 0xD2, 0x4C, 0xC4, 0xC6,

 30, 0xCF80,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

	0xAD, 0x06, 0xD5, 0x48, 0x5D, 0xF0, 0xF7, 0x29, 0x3F, 0x5D, 0xF0, 0xF7, 0x8D, 0x06, 0xD5, 0xBD,
	0xF0, 0xF7, 0xAA, 0x08, 0x78, 0x20, 0xF0, 0x03, 0x28, 0x68, 0x8D, 0x06, 0xD5, 0x60,

 64, 0xCFC0,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x08, 0x41, 0x87,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0xCF,

	0x28, 0x43, 0x29, 0x31, 0x39, 0x38, 0x36, 0x20, 0x43, 0x4F, 0x4D, 0x4D, 0x4F, 0x44, 0x4F, 0x52,
	0x45, 0x20, 0x45, 0x4C, 0x45, 0x43, 0x54, 0x52, 0x4F, 0x4E, 0x49, 0x43, 0x53, 0x2C, 0x20, 0x4C,
	0x54, 0x44, 0x2E, 0x20, 0x41, 0x4C, 0x4C, 0x20, 0x52, 0x49, 0x47, 0x48, 0x54, 0x53, 0x20, 0x52,
	0x45, 0x53, 0x45, 0x52, 0x56, 0x45, 0x44, 0x2E, 0xFF, 0xFF, 0xFF, 0xFF, 0x76, 0x8F, 0x01, 0x3C,


	/* Kernel */

  1, 0xE15A,
	0x50,
	0x50,
	0x55,

  2, 0xE192,
	0xDC, 0xE1,
	0xDC, 0xE1,
	0x67, 0xFC,

  1, 0xE337,
	0x27,
	0x27,
	0x26,

  1, 0xF099,
	0x08,
	0x08,
	0x09,

  1, 0xF09E,
	0x03,
	0x03,
	0x04,

 13, 0xF0A2,
	0xAD, 0x18, 0x0A, 0x8D, 0x19, 0x0A, 0xAD, 0x1B, 0x0A, 0x8D, 0x1A, 0x0A, 0x18,
	0xAD, 0x18, 0x0A, 0x8D, 0x19, 0x0A, 0xAD, 0x1B, 0x0A, 0x8D, 0x1A, 0x0A, 0x18,
	0x18, 0xAD, 0x18, 0x0A, 0x8D, 0x19, 0x0A, 0xAD, 0x1B, 0x0A, 0x8D, 0x1A, 0x0A,

  1, 0xF2CD,
	0x45,
	0x45,
	0x4A,

  1, 0xF7A9,
	0xFF,
	0xFF,
	0xFE,

  1, 0xFBB3,
	0x82,
	0xFF,
	0xFF,

  1, 0xFBB6,
	0x8F,
	0xFF,
	0xFF,

 48, 0xFC44,
	0x60, 0xA5, 0xAE, 0xA4, 0xAF, 0x85, 0xAC, 0x84, 0xAD, 0x4C, 0x33, 0xF5, 0x8D, 0x37, 0x0A, 0x8D,
	0x11, 0x0A, 0x8D, 0x12, 0x0A, 0x8D, 0x13, 0x0A, 0xA2, 0x06, 0x8E, 0x10, 0x0A, 0x60, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

	0x60, 0xA5, 0xAE, 0xA4, 0xAF, 0x85, 0xAC, 0x84, 0xAD, 0x4C, 0x33, 0xF5, 0x8D, 0x37, 0x0A, 0x8D,
	0x11, 0x0A, 0x8D, 0x12, 0x0A, 0x8D, 0x13, 0x0A, 0xA2, 0x06, 0x8E, 0x10, 0x0A, 0x60, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

	0xA5, 0xEB, 0x8D, 0x30, 0x0A, 0x60, 0xA5, 0xAE, 0xA4, 0xAF, 0x85, 0xAC, 0x84, 0xAD, 0x4C, 0x33,
	0xF5, 0x8D, 0x37, 0x0A, 0x8D, 0x11, 0x0A, 0x8D, 0x12, 0x0A, 0x8D, 0x13, 0x0A, 0xA2, 0x06, 0x8E,
	0x10, 0x0A, 0x60, 0xA0, 0x00, 0xA9, 0x7F, 0x8C, 0x00, 0xD6, 0x8D, 0x01, 0xD6, 0x4C, 0xDC, 0xE1,


 256, 0xFC80,
	0x8D, 0xC5, 0x0A, 0x8D, 0x18, 0xD4, 0x60, 0x2C, 0xC5, 0x0A, 0x30, 0x37, 0xA5, 0xD3, 0x29, 0x10,
	0xF0, 0x0D, 0xAD, 0x3F, 0x03, 0xC9, 0xFD, 0xF0, 0x2A, 0xA9, 0x34, 0xA0, 0xFE, 0xD0, 0x0B, 0xAD,
	0x3F, 0x03, 0xC9, 0xFA, 0xF0, 0x1D, 0xA9, 0x6F, 0xA0, 0xC0, 0x85, 0xCC, 0x84, 0xCD, 0xA0, 0x0B,
	0xB1, 0xCC, 0x99, 0x3E, 0x03, 0x88, 0x10, 0xF8, 0xC8, 0x8C, 0xC5, 0x0A, 0x08, 0x78, 0x20, 0x0C,
	0xCE, 0x28, 0x60, 0x4C, 0x5D, 0xC5, 0xAE, 0x3F, 0x03, 0xE0, 0xFD, 0xD0, 0x55, 0xAE, 0xC5, 0x0A,
	0x30, 0x50, 0xF0, 0x1D, 0xBC, 0x45, 0xFE, 0xCA, 0x88, 0x48, 0x98, 0xDD, 0x45, 0xFE, 0x68, 0x90,
	0x08, 0xD9, 0x4A, 0xFE, 0xD0, 0xF2, 0xB9, 0x65, 0xFE, 0x48, 0x29, 0x7F, 0xC9, 0x20, 0x68, 0x90,
	0x23, 0xA2, 0x05, 0xDD, 0x3F, 0xFE, 0xF0, 0x03, 0xCA, 0xD0, 0xF8, 0x8E, 0xC5, 0x0A, 0xE0, 0x00,
	0xF0, 0x20, 0xA8, 0x24, 0xF6, 0x30, 0x0D, 0x24, 0xD7, 0x10, 0x0A, 0xA2, 0x0A, 0x20, 0xDA, 0xCD,
	0x29, 0x40, 0xD0, 0x06, 0x60, 0xAD, 0x27, 0x0A, 0xD0, 0xFA, 0x98, 0x09, 0x40, 0x29, 0x7F, 0x4C,
	0x2F, 0xCC, 0xA6, 0xD3, 0xA4, 0xD5, 0x6C, 0x3C, 0x03, 0x14, 0x0D, 0x1D, 0x88, 0x85, 0x86, 0x87,
	0x11, 0x33, 0x57, 0x41, 0x34, 0x5A, 0x53, 0x45, 0x01, 0x35, 0x52, 0x44, 0x36, 0x43, 0x46, 0x54,
	0x58, 0x37, 0x59, 0x47, 0x38, 0x42, 0x48, 0x55, 0x56, 0x39, 0x49, 0x4A, 0x30, 0x4D, 0x4B, 0x4F,
	0x4E, 0x2D, 0x50, 0x4C, 0x3D, 0x2E, 0x5C, 0x5D, 0x2C, 0x3A, 0x40, 0x5B, 0x13, 0x01, 0x3B, 0x5E,
	0x2F, 0x31, 0x5F, 0x04, 0x32, 0x20, 0x02, 0x51, 0x03, 0x84, 0x38, 0x35, 0x09, 0x32, 0x34, 0x37,
	0x31, 0x1B, 0x2B, 0x2D, 0x0A, 0x0D, 0x36, 0x39, 0x33, 0x08, 0x30, 0x2E, 0x91, 0x11, 0x9D, 0x1D,

	0x8D, 0xC5, 0x0A, 0x8D, 0x18, 0xD4, 0x60, 0x2C, 0xC5, 0x0A, 0x30, 0x37, 0xA5, 0xD3, 0x29, 0x10,
	0xF0, 0x0D, 0xAD, 0x3F, 0x03, 0xC9, 0xFD, 0xF0, 0x2A, 0xA9, 0x34, 0xA0, 0xFE, 0xD0, 0x0B, 0xAD,
	0x3F, 0x03, 0xC9, 0xFA, 0xF0, 0x1D, 0xA9, 0x6F, 0xA0, 0xC0, 0x85, 0xCC, 0x84, 0xCD, 0xA0, 0x0B,
	0xB1, 0xCC, 0x99, 0x3E, 0x03, 0x88, 0x10, 0xF8, 0xC8, 0x8C, 0xC5, 0x0A, 0x08, 0x78, 0x20, 0x0C,
	0xCE, 0x28, 0x60, 0x4C, 0x5D, 0xC5, 0xAE, 0x3F, 0x03, 0xE0, 0xFD, 0xD0, 0x55, 0xAE, 0xC5, 0x0A,
	0x30, 0x50, 0xF0, 0x1D, 0xBC, 0x45, 0xFE, 0xCA, 0x88, 0x48, 0x98, 0xDD, 0x45, 0xFE, 0x68, 0x90,
	0x08, 0xD9, 0x4A, 0xFE, 0xD0, 0xF2, 0xB9, 0x65, 0xFE, 0x48, 0x29, 0x7F, 0xC9, 0x20, 0x68, 0x90,
	0x23, 0xA2, 0x05, 0xDD, 0x3F, 0xFE, 0xF0, 0x03, 0xCA, 0xD0, 0xF8, 0x8E, 0xC5, 0x0A, 0xE0, 0x00,
	0xF0, 0x20, 0xA8, 0x24, 0xF6, 0x30, 0x0D, 0x24, 0xD7, 0x10, 0x0A, 0xA2, 0x0A, 0x20, 0xDA, 0xCD,
	0x29, 0x40, 0xD0, 0x06, 0x60, 0xAD, 0x27, 0x0A, 0xD0, 0xFA, 0x98, 0x09, 0x40, 0x29, 0x7F, 0x4C,
	0x2F, 0xCC, 0xA6, 0xD3, 0xA4, 0xD5, 0x6C, 0x3C, 0x03, 0x14, 0x0D, 0x1D, 0x88, 0x85, 0x86, 0x87,
	0x11, 0x33, 0x57, 0x41, 0x34, 0x59, 0x53, 0x45, 0x01, 0x35, 0x52, 0x44, 0x36, 0x43, 0x46, 0x54,
	0x58, 0x37, 0x5A, 0x47, 0x38, 0x42, 0x48, 0x55, 0x56, 0x39, 0x49, 0x4A, 0x30, 0x4D, 0x4B, 0x4F,
	0x4E, 0xBE, 0x50, 0x4C, 0xAF, 0x2E, 0xBC, 0xBD, 0x2C, 0x5B, 0x2B, 0xBB, 0x13, 0x01, 0x23, 0x5D,
	0x2D, 0x31, 0x3C, 0x04, 0x32, 0x20, 0x02, 0x51, 0x03, 0x84, 0x38, 0x35, 0x09, 0x32, 0x34, 0x37,
	0x31, 0x1B, 0x2B, 0x2D, 0x0A, 0x0D, 0x36, 0x39, 0x33, 0x08, 0x30, 0x2E, 0x91, 0x11, 0x9D, 0x1D,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,


 87, 0xFD82,
	0x94, 0x8D, 0x9D, 0x8C, 0x89, 0x8A, 0x8B, 0x91, 0x23, 0xD7, 0xC1, 0x24, 0xDA, 0xD3, 0xC5, 0x01,
	0x25, 0xD2, 0xC4, 0x26, 0xC3, 0xC6, 0xD4, 0xD8, 0x27, 0xD9, 0xC7, 0x28, 0xC2, 0xC8, 0xD5, 0xD6,
	0x29, 0xC9, 0xCA, 0x30, 0xCD, 0xCB, 0xCF, 0xCE, 0xDB, 0xD0, 0xCC, 0xDD, 0x3E, 0xDC, 0xDD, 0x3C,
	0x2A, 0xC0, 0xDB, 0x93, 0x01, 0x2B, 0xDE, 0x3F, 0x21, 0x5F, 0x04, 0x22, 0xA0, 0x02, 0xD1, 0x83,
	0x84, 0x38, 0x35, 0x18, 0x32, 0x34, 0x37, 0x31, 0x1B, 0x2B, 0x2D, 0x0A, 0x8D, 0x36, 0x39, 0x33,
	0x08, 0x30, 0x2E, 0x91, 0x11, 0x9D, 0x1D,

	0x94, 0x8D, 0x9D, 0x8C, 0x89, 0x8A, 0x8B, 0x91, 0x40, 0xD7, 0xC1, 0x24, 0xD9, 0xD3, 0xC5, 0x01,
	0x25, 0xD2, 0xC4, 0x26, 0xC3, 0xC6, 0xD4, 0xD8, 0x2F, 0xDA, 0xC7, 0x28, 0xC2, 0xC8, 0xD5, 0xD6,
	0x29, 0xC9, 0xCA, 0x3D, 0xCD, 0xCB, 0xCF, 0xCE, 0x3F, 0xD0, 0xCC, 0xC0, 0x3A, 0xDC, 0xDD, 0x3B,
	0x5E, 0x2A, 0xDB, 0x93, 0x01, 0x27, 0x5C, 0x5F, 0x21, 0x3E, 0x04, 0x22, 0xA0, 0x02, 0xD1, 0x83,
	0x84, 0x38, 0x35, 0x18, 0x32, 0x34, 0x37, 0x31, 0x1B, 0x2B, 0x2D, 0x0A, 0x8D, 0x36, 0x39, 0x33,
	0x08, 0x30, 0x2E, 0x91, 0x11, 0x9D, 0x1D,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,


 87, 0xFDDB,
	0x94, 0x8D, 0x9D, 0x8C, 0x89, 0x8A, 0x8B, 0x91, 0x96, 0xB3, 0xB0, 0x97, 0xAD, 0xAE, 0xB1, 0x01,
	0x98, 0xB2, 0xAC, 0x99, 0xBC, 0xBB, 0xA3, 0xBD, 0x9A, 0xB7, 0xA5, 0x9B, 0xBF, 0xB4, 0xB8, 0xBE,
	0x29, 0xA2, 0xB5, 0x30, 0xA7, 0xA1, 0xB9, 0xAA, 0xA6, 0xAF, 0xB6, 0xDC, 0x3E, 0xDC, 0xA4, 0x3C,
	0xA8, 0xDF, 0xDB, 0x93, 0x01, 0x2B, 0xDE, 0x3F, 0x81, 0x5F, 0x04, 0x95, 0xA0, 0x02, 0xAB, 0x03,
	0x84, 0x38, 0x35, 0x18, 0x32, 0x34, 0x37, 0x31, 0x1B, 0x2B, 0x2D, 0x0A, 0x8D, 0x36, 0x39, 0x33,
	0x08, 0x30, 0x2E, 0x91, 0x11, 0x9D, 0x1D,

	0x94, 0x8D, 0x9D, 0x8C, 0x89, 0x8A, 0x8B, 0x91, 0x96, 0xA7, 0xA8, 0x97, 0xA2, 0xAA, 0xA3, 0x01,
	0x98, 0xA9, 0xC4, 0x99, 0xC5, 0xD3, 0xCE, 0xA4, 0x9A, 0xC2, 0xDF, 0x9B, 0xA1, 0xC9, 0xD6, 0xD7,
	0xD1, 0xC3, 0xD5, 0xC1, 0xCB, 0xDA, 0xD8, 0xCD, 0xAB, 0xD9, 0xC8, 0xBF, 0xBA, 0xCA, 0xB0, 0xAC,
	0xAD, 0xDF, 0xDB, 0x93, 0x01, 0xDD, 0xDE, 0xB9, 0x81, 0xB1, 0x04, 0x95, 0xA0, 0x02, 0xA5, 0x03,
	0x84, 0x38, 0x35, 0x18, 0x32, 0x34, 0x37, 0x31, 0x1B, 0x2B, 0x2D, 0x0A, 0x8D, 0x36, 0x39, 0x33,
	0x08, 0x30, 0x2E, 0x91, 0x11, 0x9D, 0x1D,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,


 34, 0xFE34,
	0x29, 0xFD, 0x82, 0xFD, 0xDB, 0xFD, 0x8B, 0xFB, 0x29, 0xFD, 0x29, 0xFD, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF,

	0x29, 0xFD, 0x82, 0xFD, 0xDB, 0xFD, 0x8B, 0xFB, 0x29, 0xFD, 0x29, 0xFD, 0xAF, 0xC0, 0xBF, 0x00,
	0x00, 0x01, 0x03, 0x07, 0x0C, 0x0C, 0x0C, 0x45, 0xC0, 0x41, 0x45, 0x55, 0xAF, 0x41, 0x45, 0x49,
	0x4F, 0x55,

	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF,


 11, 0xFE66,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xAC, 0xBF, 0xB2, 0xAE, 0xB3, 0xBF, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

 11, 0xFE81,
	0xA5, 0x01, 0x29, 0x40, 0x49, 0x40, 0x4A, 0x4A, 0x85, 0xD3, 0x60,
	0xA5, 0x01, 0x29, 0x40, 0x49, 0x40, 0x4A, 0x4A, 0x85, 0xD3, 0x60,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,

  1, 0xFEFF,
	0x67,
	0xD4,
	0xFF,

  2, 0xFF51,
	0xA5, 0xF7,
	0xA5, 0xF7,
	0x80, 0xCF,

  0, 00
};


int  patch_rom(char *str)
{
    int     rev, num, Versions;
    short   bytes, n, i = 0;
    unsigned short *patch_bytes;
    ADDRESS a;

    rev = atoi (str);
    num = rev;

    if (rev > 80) {
	Versions = BASIC_PATCH_VERSIONS;
	patch_bytes = basic_patch_bytes;

	switch (rev) {
	  case 85:
	    rev = 0; /* index for data */
	    break;
	  case 86:
	    rev = 1;
	    break;
	  default:
	    printf("Cannot patch ROM to revision %d\n", rev);
	    return (-1);
	}
    }
    else {
	Versions = KERNAL_PATCH_VERSIONS;
	patch_bytes = kernal_patch_bytes;

/*	curr = load_rom(0xff80);
	if (rev == curr) {
	    printf("ROM not patched: Already revision %d\n", curr);
	    return (0);
	}
*/
	/* create index */

	switch (rev) {
	  case 1:
	  case 2:
	  case 3:
	    --rev; /* index for kernel data */
	    break;

	  default:
	    printf("Cannot patch ROM to revision %d\n", rev);
	    return (-1);
	}
    }


    printf("Installing ROM patch for revision %d\n", num);

    while ((bytes = patch_bytes[i++]) > 0) {
	a = (ADDRESS)patch_bytes[i++];

	printf("%.4X (%d byte%s)\n",
	       a & 0xFFFF, bytes, ((bytes > 1) ? "s":""));

	i += (bytes * rev);	/* select patch */
	for(n = bytes; n--;)
	    save_rom(a++, (BYTE)patch_bytes[i++]);

	i += (bytes * (Versions - rev));	/* skip patch */
    }
    return (0);
}
