/*
 * $Id: raster.h,v 1.2 1996/07/29 22:34:48 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *
 * Header file for the raster emulation routines.
 * 
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: raster.h,v $
 * Revision 1.2  1996/07/29 22:34:48  ettore
 * Slightly faster: PUTMONO* and PUTMULTI use parameters instead of
 * global variables.
 * Changes in VideoStruct and SpriteStruct for correct memory fetching
 * in the VIC-II fashion.
 * sprmask added to SpriteLineStruct for the new sprite collision check.
 * VINTAGE_DISPLAY option.
 * Some garbage removed.
 *
 * Revision 1.1  1996/04/01 05:50:11  ettore
 * Initial revision
 *
 */


#if !defined( _RASTER_H )

#define _RASTER_H

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "types.h"
#include "vmachine.h"


#if defined( QUAD_X )
#define PIXEL_XSIZE 4
#define CHAR_XSIZE  32
#elif defined( DOUBLE_X )
#define PIXEL_XSIZE 2
#define CHAR_XSIZE  16
#elif defined( TRIPLE_X )
#define PIXEL_XSIZE 3
#define CHAR_XSIZE  24
#else
#define PIXEL_XSIZE 1
#define CHAR_XSIZE 8
#endif

#if defined (DOUBLE_Y)
#define PIXEL_YSIZE 2
#elif defined (TRIPLE_Y)
#define PIXEL_YSIZE 3
#elif defined (VINTAGE_DISPLAY)
#define PIXEL_YSIZE 2
#else
#define PIXEL_YSIZE 1
#endif

/* macros to draw byte images onto the screen... */

#define _PUT1( n, c )  p[ (n) ] = (c)
#define _PUT2( n, c )  p[ (n) * 2 ] = p[ (n) * 2 + 1 ] = (c)
#define _PUT3( n, c )  p[ (n) * 3 ] = p[ (n) * 3 + 1 ] = p[ (n) * 3 + 2 ] = (c)
#define _PUT4( n, c )  p[ (n) * 4 ] = p[ (n) * 4 + 1 ] = p[ (n) * 4 + 2 ] = \
                       p[ (n) * 4 + 3 ] = (c)
#define _PUT6( n, c )  p[ (n) * 6 ] = p[ (n) * 6 + 1 ] = p[ (n) * 6 + 2 ] = \
                       p[ (n) * 6 + 3 ] = p[ (n) * 6 + 4 ] = p[ (n) * 6 + 5 ] = (c)
#define _PUT8( n, c )  p[ (n) * 8 ] = p[ (n)  * 8 + 1 ] = p[ (n) * 8 + 2 ] = \
                       p[ (n) * 8 + 3 ] = p[ (n) * 8 + 4 ] = p[ (n) * 8 + 5 ] = \
                       p[ (n) * 8 + 6 ] = p[ (n) * 8 + 7 ] = (c)
#define _PUT12( n, c ) p[ (n) * 12 ] = p[ (n)  * 12 + 1 ] = p[ (n) * 12 + 2 ] = \
                       p[ (n) * 12 + 3 ] = p[ (n) * 12 + 4 ] = p[ (n) * 12 + 5 ] = \
                       p[ (n) * 12 + 6 ] = p[ (n) * 12 + 7 ] = p[ (n) * 12 + 8 ] = \
                       p[ (n) * 12 + 9 ] = p[ (n) * 12 + 10 ] = p[ (n) * 12 + 11 ] = (c)

#define _PUTMONO( f, byte, fg ) { \
    if ( (byte) & 0x80 ) f( 0, (fg) ); \
    if ( (byte) & 0x40 ) f( 1, (fg) ); \
    if ( (byte) & 0x20 ) f( 2, (fg) ); \
    if ( (byte) & 0x10 ) f( 3, (fg) ); \
    if ( (byte) & 0x08 ) f( 4, (fg) ); \
    if ( (byte) & 0x04 ) f( 5, (fg) ); \
    if ( (byte) & 0x02 ) f( 6, (fg) ); \
    if ( (byte) & 0x01 ) f( 7, (fg) ); }

#define _PUTMULTI( f, byte, c1, c2, c3 ) { \
    if ( (byte) & 0x80 ) \
      if ( (byte) & 0x40 ) f( 0, (c3) ); else f( 0, (c2) ); \
    else \
      if ( (byte) & 0x40 ) f( 0, (c1) ); \
    if ( (byte) & 0x20 ) \
      if ( (byte) & 0x10 ) f( 1, (c3) ); else f( 1, (c2) ); \
    else \
      if ( (byte) & 0x10 ) f( 1, (c1) ); \
    if ( (byte) & 0x08 ) \
      if ( (byte) & 0x04 ) f( 2, (c3) ); else f( 2, (c2) ); \
    else \
      if ( (byte) & 0x04 ) f( 2, (c1) ); \
    if ( (byte) & 0x02 ) \
      if ( (byte) & 0x01 ) f( 3, (c3) ); else f( 3, (c2) ); \
    else \
      if ( (byte) & 0x01 ) f( 3, (c1) ); }

#define PUTMONO8(byte, fg)    		_PUTMONO( _PUT1, (byte), (fg) )
#define PUTMONO16(byte, fg)   		_PUTMONO( _PUT2, (byte), (fg) )
#define PUTMONO24(byte, fg)   		_PUTMONO( _PUT3, (byte), (fg) )
#define PUTMONO32(byte, fg)   		_PUTMONO( _PUT4, (byte), (fg) )
#define PUTMONO48(byte, fg)   		_PUTMONO( _PUT6, (byte), (fg) )
#define PUTMULTI8(byte, c1, c2, c3)	_PUTMULTI( _PUT2, (byte), (c1), (c2), (c3))
#define PUTMULTI16(byte, c1, c2, c3)    _PUTMULTI( _PUT4, (byte), (c1), (c2), (c3))
#define PUTMULTI24(byte, c1, c2, c3)    _PUTMULTI( _PUT6, (byte), (c1), (c2), (c3))
#define PUTMULTI32(byte, c1, c2, c3)    _PUTMULTI( _PUT8, (byte), (c1), (c2), (c3))
#define PUTMULTI48(byte, c1, c2, c3)    _PUTMULTI( _PUT12 (byte), (c1), (c2), (c3))

#if defined( QUAD_X )		/* Note: QUAD_X does not provide PUTMULTI2x and PUTMONO2x */
#define PUTMULTI    PUTMULTI32
#define PUTMONO     PUTMONO32
#define PUTMULTI2x  PUTMULTI32
#define PUTMONO2x   PUTMONO32
#elif defined( DOUBLE_X )
#define PUTMULTI    PUTMULTI16
#define PUTMONO     PUTMONO16
#define PUTMULTI2x  PUTMULTI32
#define PUTMONO2x   PUTMONO32
#elif defined( TRIPLE_X )
#define PUTMULTI    PUTMULTI24
#define PUTMONO     PUTMONO24
#define PUTMULTI2x  PUTMULTI48
#define PUTMONO2x   PUTMONO48
#else
#define PUTMULTI    PUTMULTI8
#define PUTMONO     PUTMONO8
#define PUTMULTI2x  PUTMULTI16
#define PUTMONO2x   PUTMONO16
#endif

#if !defined( BYTES_PER_CHAR )
#define BYTES_PER_CHAR 8
#endif

#define GET_CHAR_DATA( m, c, l )  (m)[ ( (c) * BYTES_PER_CHAR ) + (l) ]

#define LO_NIBBLE( c )   ( (c) & 0xf )
#define HI_NIBBLE( c )   ( ( (c) & 0xf0 ) >> 4 )


/*
   
   RefreshStruct defines the area to be refreshed.
   
 */

typedef struct
  {
    int num_elem;
    int ys, ye, xs, xe;
  }
RefreshStruct;


/*
   
   SpriteLineStruct defines the values of the sprites of each line
   
 */

typedef struct
  {
    BYTE c1, c2, c3;		/* colors of sprite */
    int multicolor;		/* multicolor flag */
    BYTE data[3];		/* sprite data for this line */
    int xexp;			/* x double-size flag */
    int x;			/* x position */
    int visible;		/* sprite activation flag */
    int priority;		/* priority flag */
  }
SpriteLineStruct;


/*

   VideoModeStruct defines the characteristics of each possible gfx/text mode.
   
 */

#define HIRES_VMODE      0
#define MULTICOLOR_VMODE 1
#define SPECIAL_VMODE    2

typedef int datafunct ();
typedef void drawfunct ();

typedef struct
  {
    datafunct *getfgdata;	/* function which reads foreground data */
    datafunct *getbgdata;	/* function which reads background data */
    datafunct *getcolordata;	/* function which reads color data */
    drawfunct *drawline;	/* function which writes data on the window */
  }
VideoModeStruct;


/*

   LineStruct defines the values for each line, allowing refresh of changed areas only.

 */

typedef struct
  {
    int n;			/* number of line shown (referred to drawable area) */
    PIXEL bordcol;		/* value for border color */
    BYTE *fgdata;		/* values for foreground */
    BYTE *bgdata;		/* values for background color information */
    BYTE *colordata1, *colordata2, *colordata3;	 /* values for foreground color information */
    int numsprites;		/* number of sprites on the line */
    SpriteLineStruct *sprites;	/* sprite information */
    int xsmooth;		/* x smooth scroll offset */
    int vmode;			/* video mode */
    int blank;			/* blank mode flag */
    int brdgrow_xl, brdgrow_xr;	/* (see VideoStruct for description) */
    int sprmask;		/* each bit is 1 if the corresponding sprite is visible */
  }
LineStruct;


/*

   SpriteStruct defines the sprites

 */

typedef struct
  {
    int visible;		/* sprite activation flag */
    int x, y;			/* x, y coordinates */
    int xexp, yexp;		/* x and y double-size flags */
    int multicolor;		/* multicolor flag */
    int priority;		/* priority flag */
    int color;			/* primary sprite color */
    int memptr;	        	/* sprite memory counter */
    int memptr_inc;		/* value to add to memptr for next fetch */
    int dma_flag;	        /* 1 when DMA access is activated for this sprite */
  }
SpriteStruct;


/*

   VideoStruct defines the characteristics of the video chip emulation.

 */

typedef struct
  {
    int scrlines, scrcols;	/* total overscan size */
    int xpix, ypix;		/* size of drawable screen */
    int tcols, tlines;		/* number of columns/lines in text mode */
    int bordwidth, bordheight;	/* size of left and upper borders */
    int winwidth, winheight;	/* size of window */
    int fscanline, lscanline;	/* first and last shown lines */
    int xpos;			/* x position of drawable screen */
    int xoffs;			/* x offset of shown image */
    int xsmooth, ysmooth;	/* x, y smooth scroll offsets */
    XImage *scrimg;		/* screen image */
    PIXEL *scrp;		/* pointer to the present position on scrimg */
    Window win;			/* output window */
    Display *display;		/* display */
    char *wintitle;		/* title of output window */
    int refrate;		/* refresh rate (the higher, the faster) */
    int refcount;		/* counter for skipped refreshes */
    int rasterline;		/* raster position */
    CLOCK oldclk;	        /* value of clock at latest refresh */
    int blank;			/* blank mode flag */
    int scount;			/* counter for speed evaluation */
    double time;		/* time value for speed evaluation */
    RefreshStruct refstruct;	/* struct to handle partial refreshes */
    LineStruct *lineinfo;	/* line information */
    SpriteStruct *sprites;	/* sprite information */
    int sprmc1, sprmc2;		/* sprite multicolor registers */
    int maxsprites;		/* maximum number of activated sprites */
    BYTE *screenmem;		/* base of screen memory */
    BYTE *colormem;		/* base of color memory */
    BYTE *bitmapmem;		/* base of bitmap memory */
    BYTE *chargenmem;		/* base of chargen memory */
    BYTE *vbankbase;		/* start of video bank seen by chip */
    BYTE *spriteptrbase;	/* start of area where sprite addresses are written */
    int memptr;			/* memory pointer */
    int ycounter;		/* counts character lines */
    BYTE bordcol, bgcol;	/* border and background color */
    int numvmodes;		/* number of available video modes */
    VideoModeStruct *vmodedesc;	/* video mode definitions */
    int vmode;			/* current video mode */
    int brdgrow_xl, brdgrow_xr;	/* these values are used to make the borders grow */
    int brdgrow_yu, brdgrow_yl;	/* (eg as in the VIC-II 24-line and 38-column modes) */
    int numcolors;		/* number of available colors */
    PIXEL *colors;		/* pixel color codes */
    int asleep;                 /* 1 if the emulation is paused */
    int openvborders;		/* 1 if the `open vertical borders' effect is in use */
    int speed_index;            /* if != 0, it tries to keep the specified speed index */
    void (*s_a_handler)();	/* alarm signal handler (for exact speed emulation) */
    int beenpaused;             /* set to 1 when the emulator is paused */
#if defined (_VICII_C)
    BYTE vbuf[40];		/* video memory buffer for bad lines (VIC-II only) */
#endif
  }
VideoStruct;


/* The following macros set the drawing functions for the video emulation. */

#define	setfgfunct( v, f )	(v)->fgfunct = (f)
#define setbgfunct( v, f )	(v)->bgfunct = (f)
#define setsprfunct( v, f )	(v)->sprfunct = (f)
#define setudbfunct( v, f )	(v)->udbfunct = (f)
#define setlrbfunct( v, f )	(v)->lrbfunct = (f)


/* Miscellaneous general-pourpose macros... */

#define firstdrawpos( v ) 	(v)->scrp + (v)->bordwidth
#define currdrawline( v )       (v)->rasterline + (v)->bordheight
#define scpos( v, l ) 		( (l) >> 3 ) * 40

#if !defined (INLINE)
#if defined( NO_INLINE )
#define INLINE static
#else
#define INLINE static __inline__
#endif /* defined (NO_INLINE) */
#endif /* !defined (INLINE) */

#define DrawBlankLine( v, p, xs, xe, colors ) \
  { memset( &(p)[ (xs) * PIXEL_XSIZE ], (colors)[ (int)(v)->bordcol ], \
    ( (xe) - (xs) + 1 ) * PIXEL_XSIZE ); }

#endif /* defined( _RASTER_H ) */
