/*
 * $Id: rasterutil.c,v 1.2 1996/07/29 22:44:44 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README files for copyright notice.
 *
 *
 * Miscellaneous functions for control of the raster emulation.
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: rasterutil.c,v $
 * Revision 1.2  1996/07/29 22:44:44  ettore
 * Better exact-speed control; no longer causes keyboard losses.
 * Fixed the problems that happened when two raster emulators were
 * activated together.
 *
 * Revision 1.1  1996/04/01 05:52:28  ettore
 * Initial revision
 *
 */


#include "config.h"
#include "rasterutil.h"

#include <string.h>

#if defined( __hpux)
#define _INCLUDE_POSIX_SOURCE 
#endif
#define __EXTENSIONS__
#include <signal.h>
#if defined( __hpux)
#undef _INCLUDE_POSIX_SOURCE
#define _INCLUDE_HPUX_SOURCE
#endif
#include <sys/time.h>

#if defined(C128)
extern VideoStruct *v_vdc;
extern VideoStruct *v_vic_ii;
#elif defined( VIC20)
extern VideoStruct *v_vic;
#define USED_VSTRUCT v_vic
#elif defined( PET)
extern VideoStruct *v_crtc;
#define USED_VSTRUCT v_crtc
#else /* if defined( CBM64) */
extern VideoStruct *v_vic_ii;
#define USED_VSTRUCT v_vic_ii
#endif


void
DisableAlarm (void)
{
  struct sigaction s;

  s.sa_handler = SIG_IGN;
  sigaction (SIGALRM, &s, (struct sigaction *) NULL);
}

void
PauseVideo (VideoStruct * v)
{
  char pausetitle[80];

  strcpy (pausetitle, v->wintitle);
  strcat (pausetitle, " (paused)");
  XSetStandardProperties (v->display, v->win, pausetitle, NULL, None, NULL, 0, NULL);
  XSync (v->display, 1);
  DisableAlarm ();
  v->beenpaused = 1;
}

void
RestartVideo (VideoStruct * v)
{
  XSetStandardProperties (v->display, v->win, v->wintitle, NULL, None, NULL, 0, NULL);
  XSync (v->display, 1);
  v->scount = -1;
  v->refcount = v->refrate;
  SetVideoSpeed (v, v->speed_index);
}

int
SetVideoSpeed (VideoStruct * v, int speed_index)
{
  struct sigaction s;
  struct itimerval i;

  if ((v->speed_index = speed_index))
    {
      double sec;
      s.sa_handler = v->s_a_handler;
      sigemptyset (&s.sa_mask);
#if !defined( SA_RESTART)
      s.sa_flags = 0;
#else
      s.sa_flags = SA_RESTART;
#endif
      sec = (100.0 / (double) RFSH_PER_SEC) / (double) speed_index;
      i.it_interval.tv_sec = i.it_value.tv_sec = 0;
      i.it_interval.tv_usec = i.it_value.tv_usec = sec * 1000000.0;
      if (setitimer (ITIMER_REAL, &i, NULL))
	return -1;
      if (sigaction (SIGALRM, &s, (struct sigaction *) NULL))
	return -1;
    }
  return 0;
}

void
SetSpeed (int speed_index)
{
#if !defined( C128)
  SetVideoSpeed (USED_VSTRUCT, speed_index);
  if (!speed_index)
    DisableAlarm ();
#else
#if !defined( USE_VDC)
  SetVideoSpeed (v_vic_ii, speed_index);
  if (!speed_index)
    DisableAlarm ();
#else
  if (!v_vic_ii->asleep)
    {
      SetVideoSpeed (v_vic_ii, speed_index);
      SetVideoSpeed (v_vdc, 0);
    }
  else
    {
      SetVideoSpeed (v_vdc, speed_index);
      SetVideoSpeed (v_vic_ii, 0);
    }
  if (!v_vic_ii->speed_index && !v_vdc->speed_index)
    DisableAlarm ();
#endif /* !defined (USE_VDC) */
#endif /* defined (C128) */
}

void
SetVideoRefreshRate (VideoStruct * v, int refresh_rate)
{
  v->refrate = refresh_rate;
}

#if defined( C128)
void
Vdc_SetRefreshRate (int refresh_rate)
{
  SetVideoRefreshRate (v_vdc, refresh_rate);
}

void
Vic_SetRefreshRate (int refresh_rate)
{
  SetVideoRefreshRate (v_vic_ii, refresh_rate);
}

#else

void
SetRefreshRate (int refresh_rate)
{
  SetVideoRefreshRate (USED_VSTRUCT, refresh_rate);
}

#endif /* C128 */
