/* $Id: sid.c,v 1.1 1996/07/29 22:48:45 ettore Exp ettore $ */

#define _SID_C

#include "config.h"

#if defined (LIONEL_SID)

/* $Id: sid.c,v 1.1 1996/07/29 22:48:45 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 * SID 6581 interfacing.
 *
 *
 * Written by
 *   Lionel Ulmer        (ulmer@cs.brandeis.edu)
 *
 *
 * $Log: sid.c,v $
 * Revision 1.1  1996/07/29 22:48:45  ettore
 * Initial revision
 *
 */

/* WARNING : this is only an experimental code to test if sound
   emulation is possible without slowing Vice too much. I'm planning
   to rewrite it soon (hadn't the time yet).
 
   If you want to port this code to another machine, there is all the
   lines between the #ifdef SGISOUND to change (ie sound init, sound
   buffer writing and also synchronisation).
   */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "sid.h"
/* I need to know RFSH_PER_SEC to allocate the buffer */
#include "vmachine.h"
#include "extern.h"

#ifdef SGISOUND
#include <dmedia/audio.h>
#include <unistd.h>
#include <sys/types.h>
#include <bstring.h>
#include <sys/time.h>
#elif defined (LINUXSOUND)
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#endif


#define ATTACK  0
#define DECAY   1
#define SUSTAIN 2
#define RELEASE 3
#define NOTHING 4

#ifndef PI
#define PI 3.14159265358979323846
#endif /* PI */
#ifndef RAND_MAX
#define RAND_MAX 2147483647
#endif /* RAND_MAX */

#undef DEBUG

/* General config variables */
int sid_byte,sid_freq,sid_stereo;
#ifdef SGISOUND
static ALconfig    sid_audioconfig;
static ALport      sid_audioPort;
static fd_set sgi_fdset;
#elif defined (LINUXSOUND)
static int	   sid_audiofd;
#endif


/* Sound buffer */
static BYTE *sid_buf;

/* Attack / Decay rates */
static unsigned int sid_attint[16],sid_decint[16];
static float sid_decrel[16]={
  0.002, 0.008, 0.016, 0.024, 0.038, 0.056, 0.068, 0.080,
  0.100, 0.240, 0.500, 0.800, 1.000, 3.000, 5.000, 8.000
} ;
  
static float sid_attack[16]={
  0.006, 0.024, 0.048, 0.072, 0.114, 0.168, 0.204, 0.240,
  0.300, 0.750, 1.500, 2.400, 3.000, 9.000, 15.00, 24.00
} ;

/* Voice variables */
static unsigned int sid_vfreq[3],sid_vpuls[3],sid_adsr[3][4],sid_vstat[3];
static unsigned int sid_vwave[3],sid_envcnt[3],sid_wavecnt[3],sid_vsawt[3];
static long sid_noisedelay[3];
static float sid_venv[3],sid_vform[3],sid_envup[3],sid_forup[3];
static int sid_blimit,sid_noisecst;
static unsigned int sid_sndgen,sid_buflen;


void store_sid (ADDRESS address, BYTE byte)
{
  BYTE voice,val;
  float f;
  int tmp;

  address &= 0x1f;
  sid[address] = byte;

#ifdef X64SOUND
  /* Generates the sound output */
  SIDemu(0);

  /* Changes the SID emulation parameters */
  if (address>0x14) {
    /* Filters : nothing done yet  */

    return;
  } 
  
  /* Find the voice modified */
  voice=address/7;

  /* Do the changes */
  switch (address) {
    /* Frequency Lo */
  case 0x00:
  case 0x07:
  case 0x0E:
    f=(sid[address]+(sid[address+1] << 8))*SIDCLK/0xFFFFFF;
    tmp=sid[address+2]+((sid[address+3] & PWHI) << 8);
    goto FreqS;
    /* Frequency Hi */
  case 0x01:
  case 0x08:
  case 0x0F:
    f=(sid[address-1]+(sid[address] << 8))*SIDCLK/0xFFFFFF;
    tmp=sid[address+1]+((sid[address+2] & PWHI) << 8);
  FreqS:
    if (f!=0)
      sid_vfreq[voice]=sid_freq/f;
    else 
      sid_vfreq[voice]=0;

    sid_vpuls[voice]=(tmp*sid_vfreq[voice])/0xFFF;

#ifdef DEBUG
    printf("FREQ : %d %d\n",voice,sid_vfreq[voice]);
#endif /* DEBUG */
    break;

    /* Pulse Width Lo */
  case 0x02:
  case 0x09:
  case 0x10: 
    sid_vpuls[voice]=((sid[address]+((sid[address+1] & PWHI) << 8))*
		      sid_vfreq[voice])/0xFFF;

#ifdef DEBUG
    printf("PULS : %d %d\n",voice,sid_vpuls[voice]);    
#endif /* DEBUG */
    break;
    
    /* Pulse Width Hi (00001111) */
  case 0x03:
  case 0x0A:
  case 0x11: 
    sid_vpuls[voice]=((sid[address-1]+((sid[address] & PWHI) << 8))*
		      sid_vfreq[voice])/0xFFF;

#ifdef DEBUG
    printf("PULS : %d %d\n",voice,sid_vpuls[voice]);
#endif /* DEBUG */
    break;

    /* Control (Noise Pulse ///   /\/\  TEST  RING  SYNC  GATE) */
  case 0x04:
  case 0x0B:
  case 0x12: 
    val=sid[address];
    /* GATE signal */
    if (val & GATE) {
      if ((sid_vstat[voice]==RELEASE) || 
	  (sid_vstat[voice]==NOTHING)) {
	sid_adsr[voice][ATTACK] =sid_attint[sid[address+1] >> 4];
	sid_adsr[voice][DECAY]  =sid_decint[sid[address+1] & DCY];
	sid_adsr[voice][SUSTAIN]=sid[address+2] >> 4;
	sid_adsr[voice][RELEASE]=sid_decint[sid[address+2] & RLS];
	
	sid_vstat[voice]=ATTACK;

	sid_envup[voice]=(1.0-sid_venv[voice])/sid_adsr[voice][ATTACK];

	sid_envcnt[voice]=sid_adsr[voice][ATTACK];

#ifdef DEBUG
	printf("ADSR : %d %d %d %d %d\n",voice,sid_adsr[voice][ATTACK],
	       sid_adsr[voice][DECAY],sid_adsr[voice][SUSTAIN]
	       ,sid_adsr[voice][RELEASE]);
#endif /* DEBUG */
      } else {
	/* Not sure about this.... Must test on a real C-64 */
	sid_adsr[voice][RELEASE]=sid_decint[sid[address+2] & RLS];

	sid_envup[voice]=(0.0-sid_venv[voice])/sid_adsr[voice][RELEASE];
	
	sid_vstat[voice]=RELEASE;
	
	sid_envcnt[voice]=sid_adsr[voice][RELEASE];

#ifdef DEBUG
	printf("ADSR-2 : %d %d %d %d %d\n",voice,sid_adsr[voice][ATTACK],
	       sid_adsr[voice][DECAY],sid_adsr[voice][SUSTAIN]
	       ,sid_adsr[voice][RELEASE]);
#endif /* DEBUG */
      }
    }

    /* Waveform */
    val >>=4;
    if (val!=sid_vwave[voice]) {
      sid_vwave[voice]=val;
      
      switch (val) {
      case 0x08:
	/* Noise */
	break;
	
      case 0x04:
	/* Pulse */
	break;
	
      case 0x02:
	/* Triangle */
	if (sid_vfreq[voice])
	  sid_forup[voice]=2.0/sid_vfreq[voice];
	else
	  sid_forup[voice]=0.0;	/* ??? */
	sid_wavecnt[voice]=(1+sid_vform[voice])/2*sid_vfreq[voice];
	break;
	
      case 0x01:
	/* Sawtooth */
	sid_vsawt[voice]=sid_vfreq[voice]/2;
	/* FOR LIONEL: there can be SIGFPEs here... please add the correct behavior */
	if (sid_vfreq[voice])
	  sid_forup[voice]=4.0/sid_vfreq[voice];
	else
	  sid_forup[voice]=0.0;	/* ??? */
	sid_wavecnt[voice]=(1+sid_vform[voice])/4*sid_vfreq[voice];;
	break;
      }
    }
	
#ifdef DEBUG
    printf("Voice : %d %d\n",voice,val >> 4);
#endif /* DEBUG */
    
    break;

    /* Attack 11110000 / Decay 00001111 */
  case 0x05:
  case 0x0C:
  case 0x13: 
    /* For the moment, ADSR values are frozen at the beginning of the cycle.
       (not sure about that, must check on a real C-64) */
    break;
    
    /* Sustain 11110000 / Release 00001111 */
  case 0x06:
  case 0x0D:
  case 0x14: 
    /* For the moment, ADSR values are frozen at the beginning of the cycle.
       (not sure about that, must check on a real C-64) */
    break;
  }
#endif /* X64SOUND */
}


BYTE read_sid (ADDRESS address)
{
  address &= 0x1f;
  if (address == 0x1b)
    return rand();
  else
    return sid[address];
}


void IniSID()
{
  int i,size;

  /* It's temporary, should be selected via the command-line */
  sid_byte=8;
  sid_freq=22050;
  sid_stereo=0;

  /* SID emulation init... */
  sid_clk=0;
  sid_sndgen=0;
  sid_blimit=(CYCLES_PER_SEC/sid_freq)+1;
  sid_noisecst=0x100000/(CYCLES_PER_SEC/sid_freq);

  /* Convert ms to "clock cycles" */
  for (i=0;i<16;i++) {
    sid_attint[i]=sid_attack[i]*sid_freq;
    sid_decint[i]=sid_decrel[i]*sid_freq;
  }

  /* Voices variables init */
  for (i=0;i<3;i++) {
    sid_vstat[i]=NOTHING;
    sid_vpuls[i]=0;
    sid_vfreq[i]=0;
    sid_vwave[i]=0;

    sid_venv[i]=0;
    sid_vform[i]=0;

    sid_envcnt[i]=0;
    sid_wavecnt[i]=0;

    sid_noisedelay[i]=0x180000/(CYCLES_PER_SEC/sid_freq);
  }

/*  fout=fopen("test.au","wb"); */

  /* Sound init */
  sid_buflen=((int) sid_freq/RFSH_PER_SEC);
  sid_buf=(BYTE *) malloc(sid_buflen*(sid_byte/8)*(1+sid_stereo));
  if (sid_buf==NULL) {
    fprintf(stderr,"Not enough memory to allocate audio buffer.\n");
    end64();
  }

#ifdef SGISOUND  
  {
    long chpars[]={AL_OUTPUT_RATE,0};
    /* SGI Initialization */
    ALseterrorhandler(0);
    
    printf("SGI sound device initialization.\n");
    
    chpars[1]=sid_freq;
    ALsetparams(AL_DEFAULT_DEVICE,chpars,2);
    ALgetparams(AL_DEFAULT_DEVICE,chpars,2);
    
    sid_audioconfig=ALnewconfig();
    ALsetchannels(sid_audioconfig,AL_MONO);
    if (sid_byte==16)
      ALsetwidth(sid_audioconfig,AL_SAMPLE_16);
    else
      ALsetwidth(sid_audioconfig,AL_SAMPLE_8);
    ALsetsampfmt(sid_audioconfig,AL_SAMPFMT_TWOSCOMP);
    
    ALsetqueuesize(sid_audioconfig,sid_freq);
    
    sid_audioPort=ALopenport("outport","w",sid_audioconfig);
    if (!sid_audioPort) {
      fprintf(stderr,"Couldn't open audio-port\n");
      ALfreeconfig(sid_audioconfig);
      end64();
    }
    /* FDset init... */
    FD_ZERO(&sgi_fdset);
    FD_SET(ALgetfd(sid_audioPort),&sgi_fdset);
  }
#elif defined (LINUXSOUND)
  printf ("Linux sound device initialization...\n");
  if ((sid_audiofd = open ("/dev/dsp", O_WRONLY)) < 0) {
    perror ("Couldn't open audio device\n");
    end64 ();
  }
  if (ioctl (sid_audiofd, SNDCTL_DSP_SPEED, &sid_freq)) {
    perror ("Can't set DSP frequency\n");
    end64 ();
  }
  if (ioctl (sid_audiofd, SNDCTL_DSP_SETFMT, &sid_byte)) {
    perror ("Can't set DSP number of bits\n");
    end64 ();
  }
  
  /* a lot of stuff should go here... for now I am just testing :P */
#endif
  printf("SID initialization done.\n");
}

void SIDemu(int flag)
{
  unsigned int togen,i,j;
  float res;
  int resint;
  BYTE *tmp_buf,min,max;
  CLOCK tmp;

  /* To be optimized.... 
     (perhaps using long long....) */
  if (flag)
    togen=sid_buflen;
  else {
    tmp=clk-sid_clk;
    if (tmp<sid_blimit)
      return;
    togen=(tmp*sid_freq)/CYCLES_PER_SEC;
  }

  sid_clk=clk;
  if (togen<sid_buflen)
    sid_sndgen=togen;
  else
    sid_sndgen=0;

#ifdef DEBUG
  printf("%d %d\n",togen,sid_buflen);
#endif /* DEBUG */

  /* Slight optimisation (useful when there are no sound played) */
  if (sid_vstat[0]!=NOTHING)
    min=0;
  else if (sid_vstat[1]!=NOTHING)
    min=1;
  else if (sid_vstat[2]!=NOTHING)
    min=2;
  else
    min=3;
    
  if (sid_vstat[2]!=NOTHING)
    max=3;
  else if (sid_vstat[1]!=NOTHING)
    max=2;
  else if (sid_vstat[0]!=NOTHING)
    max=1;
  else
    max=0;

  tmp_buf=sid_buf;
  for (i=sid_sndgen;i<togen;i++) {
    res=0;
    for (j=min;j<max;j++) {      
      /* If ADSR = 0, don't generate the waveform */
      if (sid_vstat[j]==NOTHING) {
	/*	sid_wavecnt[j]++; */
	continue;
      }	
      
      /* Envelope Generation */
      switch (sid_vstat[j]) {
      case ATTACK:
	if (sid_envcnt[j]==0) {
	  /* ATTACK -> DECAY */
	  sid_vstat[j]=DECAY;
	  sid_venv[j]=1.0;
	  sid_envup[j]=(sid_adsr[j][SUSTAIN]/15.0-1.0)/
	    sid_adsr[j][DECAY];
	  
	  sid_envcnt[j]=sid_adsr[j][DECAY];
	}
	sid_envcnt[j]--;
	sid_venv[j]+=sid_envup[j];
	break;
      
      case DECAY:
	if (sid_envcnt[j]==0) {
	  /* DECAY -> SUSTAIN */
	  sid_vstat[j]=SUSTAIN;
	  sid_venv[j]=sid_adsr[j][SUSTAIN]/15.0;
	  sid_envup[j]=0;
	  
	  sid_envcnt[j]=0;
	}
	sid_envcnt[j]--;
	sid_venv[j]+=sid_envup[j];
	break;
	
      case RELEASE: 
	if(sid_envcnt[j]==0) {
	  /* RELEASE -> NOTHING */
	  sid_vstat[j]=NOTHING;
	  sid_venv[j]=0;
	  sid_envup[j]=0;
	  
	  sid_envcnt[j]=0;
	}
	sid_envcnt[j]--;
	sid_venv[j]+=sid_envup[j];
	break;
      }
      
      /* Waveform Generation */
      switch (sid_vwave[j]) {
      case 0x08:
	/* Noise ... */
#ifdef TOTO
	void Frequency-generator(long freq) {
	  long delay=0x180000;        /* C-notation for $180000 */
	  long cycle=0;
	  for (;;;) {                 /* Repeat forever */
	    delay= delay-freq;
	    if (delay<0) {
	      delay= delay+0x100000;  /* C-notation for $100000 */
	      waveform_output= calculate new value for waveform;
	    };
	    waveform[cycle]= waveform_output;
	    cycle= cycle+1;
	  }
	}
#endif
	sid_noisedelay[j]-=sid_vfreq[j];
	if (sid_noisedelay[j]<0) {
	  sid_noisedelay[j]+=sid_noisecst;
	  sid_vform[j]=2*((float) rand())/RAND_MAX-1;
	}
	break;
	
      case 0x04:
	/* Pulse ... */
	if (sid_wavecnt[j]<sid_vpuls[j])
	  sid_vform[j]=-1;
	else if (sid_wavecnt[j]<sid_vfreq[j])
	  sid_vform[j]=1;
	else {
	  sid_wavecnt[j]=-1;
	  sid_vform[j]=1;
	}
	break;
	
      case 0x02:
	/* Triangle ... */
	if (sid_wavecnt[j]<sid_vfreq[j])
	  sid_vform[j]+=sid_forup[j];
	else {
	  sid_wavecnt[j]=-1;
	  sid_vform[j]=-1;
	}
	break;
	
      case 0x01:
	/* Sawtooth .. */
	if (sid_wavecnt[j]<sid_vsawt[j])
	  sid_vform[j]+=sid_forup[j];
	else if (sid_wavecnt[j]<sid_vfreq[j])
	  sid_vform[j]-=sid_forup[j];
	else {
	  sid_wavecnt[j]=-1;
	  sid_vform[j]=-1;
	}
	break;
	
      default:
	sid_vform[j]=0;
	break;
      }

      res+=sid_venv[j]*sid_vform[j];
      
      /* Updates counters */
      sid_wavecnt[j]++;
    }
  
    /* Puts in the buffer */
#ifdef DEBUG
      printf("%d\n",res);
#endif /* DEBUG */
    switch (sid_byte) {
    case 16:
      resint=((res+3.0)/6.0)*65535.0;
      resint^=0x8000;
      *(tmp_buf++)=resint >> 8;
      *(tmp_buf++)=resint & 0xFF;
      break;

    case 8:
      resint=((res+3.0)/6.0)*200.0;
      resint^=0x80;
      *(tmp_buf++)=resint;
      break;
    }
  }
  
  /* If we are called from screen-refresh => writes in the audio device */
  if (flag) {    
#ifdef SGISOUND
    /* Waits that the buffer is written (to do the synchronisation)
       (sid_buflen should be large enough to prevent "clicks" and small
       enough to be "undetectable") */
    if (ALgetfilled(sid_audioPort)>sid_buflen) {
      ALsetfillpoint(sid_audioPort,sid_freq-sid_buflen);
      select(FD_SETSIZE,NULL,&sgi_fdset,NULL,NULL);
    }
    ALwritesamps(sid_audioPort,sid_buf,sid_buflen);
#elif defined (LINUXSOUND)
    write (sid_audiofd, sid_buf, sid_buflen);
#endif
  }
}

#elif defined (TEEMU_SID)

/*
 * $Id: sid.c,v 1.1 1996/07/29 22:48:45 ettore Exp ettore $
 * 
 * Author: Teemu Rantanen <tvr@cs.hut.fi>
 * 
 * Copyright (c) 1996 Teemu Rantanen
 *                    All rights reserved
 * 
 * Created: Sun May  5 12:16:36 1996 tvr
 * Last modified: Thu Jun 20 14:08:41 1996 tvr
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#include "sid.h"
#include "macro.h"
#include "vmachine.h"
#include "extern.h"
#include "config.h"
#include "memory.h"
#include "interrupt.h"

/*
 * This is a quick'n'dirty implementation (less than 10 hours of work) of
 * sound chip emulation. Most of the features are implemented but not correct.
 * I hope someone has time to test this and rewrite a better and faster
 * version, I don't.
 *
 *                                               // tvr (96/14/05)
 */

/*
 * Some bugs fixed, noise generation changed, ring modulation implemented,
 * etc, etc. Now even Impossible Mission speech synthesis works. Filters are
 * still missing.
 *
 * Emulation isn't correct, but it's getting good enough to be useful.
 *
 *                                               // tvr (96/20/06)
 */

#include <sys/fcntl.h>
#ifdef linux
#include <linux/soundcard.h>
#endif
#ifdef __FreeBSD__
#include <machine/soundcard.h>
#endif

#define SCLOCK (16*1024*1024.0)
#define XCLOCK (1024*1024.0)

#define FREQ(v) ((v)->data[0] + (v)->data[1]*0x100)
#define PW(v) ((v)->data[2] + ((v)->data[3]&0x0f)*0x100)

#define GATE(v) ((v)->data[4] & 0x01)
#define SYNC(v) ((v)->data[4] & 0x02)
#define RING(v) ((v)->data[4] & 0x04)
#define TEST(v) ((v)->data[4] & 0x08)
#define NOISE(v) ((v)->data[4] & 0x80)
#define PULSE(v) ((v)->data[4] & 0x40)
#define SAWTOOTH(v) ((v)->data[4] & 0x20)
#define TRIANGLE(v) ((v)->data[4] & 0x10)

#define VATTACK(v) ((v)->data[5] / 0x10)
#define VDECAY(v) ((v)->data[5] & 0x0f)
#define VSUSTAIN(v) ((v)->data[6] / 0x10)
#define VRELEASE(v) ((v)->data[6] & 0x0f)

#define VOL(s) ((s)->data[3] & 0x0f)
#define OFF3(s) ((s)->data[3] & 0x80)

#define ATTACK 0
#define DECAY 1
#define SUSTAIN 2
#define RELEASE 3


#define RNDS			(32768)


typedef struct
{
    double			 fptr;
    int				 adsrmode;
    double			 adsrptr;
    double			 speed;
    BYTE			 data[7];
    double			*rnd;
    int				 rndptr;
    double			 freqconst;
    double			 oneperspeed;
    int				 fptrchanged;
    double			 adsrvalue;
} voice_t;

#define SIDBUFSIZE	        16384

typedef struct
{
    voice_t			voice[3];
    BYTE			data[8];
    int				speed;
    double			clkstep;
    double			clk;
    double			rnd[RNDS];
    int				fd;
    int				bufptr;
    char			buf[SIDBUFSIZE];
    int				kludgefactor2;
} sid_t;


static void addfptr(voice_t *voice)
{
    voice->fptr += FREQ(voice)*voice->freqconst;
    /* shouldn't loop more than once... */
    while (voice->fptr > 1.0)
    {
	voice->fptrchanged = 1;
	voice->fptr -= 1.0;
    }
}

/* XXX: correct? */
static void syncfptr(voice_t *voice, voice_t *syncvoice)
{
    if (SYNC(voice))
	voice->fptr = syncvoice->fptr;
}

static double attack[16] =
{
    0.002, 0.008, 0.016, 0.024, 0.038, 0.056, 0.068, 0.080,
    0.100, 0.250, 0.500, 0.800, 1.000, 3.000, 5.000, 8.000
};

static double decayrelease[16] =
{
    0.006, 0.024, 0.048, 0.072, 0.114, 0.168, 0.204, 0.240,
    0.300, 0.750, 1.500, 2.400, 3.000, 9.000, 15.000, 24.000
};

/*
 * XXX: - adsrptr can be other than zero when gate is changed
 *        real value should be calculated
 *      - release is not linear?
 */
static void doadsrptr(voice_t *voice)
{
    switch (voice->adsrmode)
    {
    case ATTACK:
	if (!GATE(voice))
	{
	    voice->adsrmode = RELEASE;
	    voice->adsrptr = 0.0;
	    return;
	}
	voice->adsrptr += voice->oneperspeed / attack[VATTACK(voice)];
	if (voice->adsrptr >= 1.0)
	{
	    voice->adsrmode = DECAY;
	    voice->adsrptr = 0.0;
	}
	break;
    case DECAY:
	if (!GATE(voice))
	{
	    voice->adsrmode = RELEASE;
	    voice->adsrptr = 0.0;
	    return;
	}
	voice->adsrptr += voice->oneperspeed / decayrelease[VDECAY(voice)];
	if (voice->adsrptr >= 1.0)
	{
	    voice->adsrmode = SUSTAIN;
	    voice->adsrptr = 0.0;
	}
	break;
    case SUSTAIN:
	if (!GATE(voice))
	{
	    voice->adsrmode = RELEASE;
	    voice->adsrptr = 0.0;
	    return;
	}
	break;
    case RELEASE:
	if (GATE(voice))
	{
	    voice->adsrmode = ATTACK;
	    voice->adsrptr = 0.0;
	    return;
	}
	voice->adsrptr += voice->oneperspeed / decayrelease[VRELEASE(voice)];
	if (voice->adsrptr > 1.0)
	    voice->adsrptr = 1.0;
	break;
    }
}


static double doadsr(voice_t *voice)
{
    switch (voice->adsrmode)
    {
    case ATTACK:
	return voice->adsrptr;
    case DECAY:
	return 1.0 - voice->adsrptr*(1.0 - VSUSTAIN(voice) / 15.0);
    case SUSTAIN:
	return VSUSTAIN(voice) / 15.0;
    case RELEASE:
	return (VSUSTAIN(voice) / 15.0) * (1.0 - voice->adsrptr);
    }
    return 0.0;
}

/*
 * XXX: - how often should value change?
 *      - shiftregister should be implemented
 */
#define RNDKLUDGEVALUE (SCLOCK/XCLOCK)
static double voicerandom(voice_t *voice)
{
    return voice->rnd[(voice->rndptr + (int)(voice->fptr*RNDKLUDGEVALUE)) % RNDS];
}


/*
 * XXX: if more than one waveform is set, a result is a logical and of the
 *      waveforms
 */
static double dooscillator(voice_t *voice)
{
    if (TEST(voice))
    {
	voice->fptr = 0.0;
	voice->rndptr = 0;
	voice->fptrchanged = 0;
	return 0.0;
    }
    if (voice->fptrchanged)
    {
	voice->fptrchanged = 0;
	voice->rndptr += RNDKLUDGEVALUE;
    }
    if (PULSE(voice))
    {
	if (voice->fptr < PW(voice) / 4095.0)
	    return 1.0;
	return 0.0;
    }
    if (SAWTOOTH(voice))
	return voice->fptr;
    if (TRIANGLE(voice))
    {
	if (voice->fptr < 0.5)
	    return 2.0*voice->fptr;
	return 2.0 - 2.0*voice->fptr;
    }
    if (NOISE(voice))
	return voicerandom(voice);
    return 0.0;
}

/* XXX: correct? */
static double doring(voice_t *voice, double osc, double ringosc)
{
    if (RING(voice) && TRIANGLE(voice))
	return osc*ringosc;
    return osc;
}

static int init_sid = 0;
static sid_t siddata;

/* #define DUMPSID */
/* #define DUMPREADWRITE */
/* #define TESTSID */

#ifdef DUMPSID
static void dumpsid(void)
{
    int				i, j;
    double			f1, f2;
    printf("\n\nsiddata:");
    for (i = 0; i < 8; i++)
	printf(" %02x", siddata.data[i]);
    printf("\n");
    for (j = 0; j < 3; j++)
    {
	printf("voice %d\n", j);
	printf("  data:");
	for (i = 0; i < 7; i++)
	    printf(" %02x", siddata.voice[j].data[i]);
	printf("\n");
	f1 = doadsr(&siddata.voice[j]);
	f2 = dooscillator(&siddata.voice[j]);
	printf("  fptr=%f adsrmode=%d adsrptr=%f v1=%f v2=%f\n",
	       siddata.voice[j].fptr,
	       siddata.voice[j].adsrmode,
	       siddata.voice[j].adsrptr,
	       f1, f2);
    }
}
#endif

#ifdef TESTSID
static int sidtested = 0;

static void testsid(void)
{
    sidtested = 1;
    printf("testing sid\n");
    /* test noise */
    store_sid(0xd418, 0x0f);
    store_sid(0xd40f, 2);
    store_sid(0xd414, 0xf0);
    store_sid(0xd412, 0x81);
}
#endif


void playsid(void)
{
    double			 v1, v2, v3, v4, v5, v6, v7, v8, v9;
    short			 vs;
    int				 i;

    for (i = 0; i < siddata.kludgefactor2; i++)
    {
	addfptr(&siddata.voice[0]);
	addfptr(&siddata.voice[1]);
	addfptr(&siddata.voice[2]);
	syncfptr(&siddata.voice[0], &siddata.voice[2]);
	syncfptr(&siddata.voice[1], &siddata.voice[0]);
	syncfptr(&siddata.voice[2], &siddata.voice[1]);
	doadsrptr(&siddata.voice[0]);
	doadsrptr(&siddata.voice[1]);
	doadsrptr(&siddata.voice[2]);
	v1 = doadsr(&siddata.voice[0]);
	v2 = doadsr(&siddata.voice[1]);
	v3 = doadsr(&siddata.voice[2]);
	siddata.voice[2].adsrvalue = v3;
	if (OFF3(&siddata))
	    v3 = 0.0;
	v4 = dooscillator(&siddata.voice[0]);
	v5 = dooscillator(&siddata.voice[1]);
	v6 = dooscillator(&siddata.voice[2]);
	v7 = doring(&siddata.voice[0], v4, v6);
	v8 = doring(&siddata.voice[1], v5, v4);
	v9 = doring(&siddata.voice[2], v6, v5);
	vs = (((v1*v7+v2*v8+v3*v9)/3.0*64000.0 - 32000.0)/15.0)*VOL(&siddata);
	if (siddata.bufptr == SIDBUFSIZE)
	{
	    int				i, j;
	    for (i = 0; i < siddata.bufptr; i += j)
	    {
		j = write(siddata.fd, siddata.buf + i, siddata.bufptr - i);
		if (j <= 0)
		{
		    printf("argh... soundwrite failed, playback disabled\n");
		    return;
		}
	    }
	    siddata.bufptr = 0;
	}
	memcpy(siddata.buf + siddata.bufptr, &vs, sizeof(vs));
	siddata.bufptr += sizeof(vs);
    }

    siddata.clk += siddata.clkstep;
    int_clk[I_SID] = siddata.clk; 
    find_next_int();
#ifdef DUMPSID
    {
	static int foo = 0;
	if (siddata.clk > foo + 1000000/siddata.kludgefactor2)
	{
	    foo = siddata.clk;
	    dumpsid();
	}
    }
#endif
#ifdef TESTSID
    if (siddata.clk > 2000000 && !sidtested)
	testsid();
#endif
}


static int setupdevice(int fd)
{
#if defined(linux) || defined(__FreeBSD__)
    int		st, tmp;

    tmp = 16;
    st = ioctl(fd, SNDCTL_DSP_SAMPLESIZE, &tmp);
    if (st < 0 || tmp != 16)
      {
	perror ("DSP_SAMPLESIZE");
	return 1;
      }
    tmp = 0;
    st = ioctl(fd, SNDCTL_DSP_STEREO, &tmp);
    if (st < 0 || tmp != 0)
      {
	perror ("SNDCTL_DSP_STEREO");
	return 1;
      }
    tmp = siddata.speed;
    st = ioctl(fd, SNDCTL_DSP_SPEED, &tmp);
    if (st < 0 || tmp <= 0)
      {
	perror ("SNDCTL_DSP_SPEED");
	return 1;
      }
    printf ("speed = %d\n", tmp);
    siddata.speed = tmp;
    return 0;
#else
    return 0;
#endif
}

static void initsid(void)
{
    int				 i;
    char			*tmp;
    int				 kludgefactor = 1;
    int				 kludgefactor2 = 1;
    double			 tmpf;

    init_sid = 1;

    memset(&siddata, 0, sizeof(siddata));
    siddata.voice[0].rnd = siddata.rnd;
    siddata.voice[1].rnd = siddata.rnd;
    siddata.voice[2].rnd = siddata.rnd;
    for (i = 0; i < RNDS; i++)
	siddata.rnd[i] = ((double)(rand() % RAND_MAX)) / RAND_MAX;
    tmp = getenv("SIDDEV");
    if (!tmp)
	tmp = "/dev/dsp";
    siddata.fd = open(tmp, O_WRONLY|O_CREAT, 0777);
    tmp = getenv("SIDSPEED");
    if (tmp)
	siddata.speed = atoi(tmp);
    if (siddata.speed < 8000 || siddata.speed > 50000)
	siddata.speed = 22050;
    if (setupdevice(siddata.fd))
	printf("warning: could not set soundcard parameters\n");
    siddata.clk = clk;
    tmp = getenv("KLUDGEFACTOR");
    if (tmp)
	kludgefactor = atoi(tmp);
    if (kludgefactor < 1)
	kludgefactor = 1;
    tmp = getenv("KLUDGEFACTOR2");
    if (tmp)
	kludgefactor2 = atoi(tmp);
    if (kludgefactor2 < 1)
	kludgefactor2 = 1;
    siddata.clkstep = (XCLOCK / siddata.speed)*kludgefactor2;
    siddata.kludgefactor2 = kludgefactor2*kludgefactor;
    siddata.voice[0].speed = siddata.speed;
    siddata.voice[1].speed = siddata.speed;
    siddata.voice[2].speed = siddata.speed;
    tmpf = (XCLOCK/SCLOCK)/siddata.speed;
    siddata.voice[0].freqconst = tmpf;
    siddata.voice[1].freqconst = tmpf;
    siddata.voice[2].freqconst = tmpf;
    tmpf = 1.0 / siddata.speed;
    siddata.voice[0].oneperspeed = tmpf;
    siddata.voice[1].oneperspeed = tmpf;
    siddata.voice[2].oneperspeed = tmpf;
    playsid();
}

BYTE read_sid (ADDRESS address)
{
    /* XXX: 0x00? */
    BYTE			ret = 0xff;
    address &= 0x1f;
    switch (address)
    {
    case 0x00:
    case 0x01:
    case 0x02:
    case 0x03:
    case 0x04:
    case 0x05:
    case 0x06:
    case 0x07:
    case 0x08:
    case 0x09:
    case 0x0a:
    case 0x0b:
    case 0x0c:
    case 0x0d:
    case 0x0e:
    case 0x0f:
    case 0x10:
    case 0x11:
    case 0x12:
    case 0x13:
    case 0x14:
    case 0x15:
    case 0x16:
    case 0x17:
    case 0x18:
	/* write-only registers */
	break;
    case 0x19:
	/* pot/x */
	break;
    case 0x1a:
	/* pot/y */
	break;
    case 0x1b:
	/* osc3/random */
	ret = voicerandom(&siddata.voice[2]);
	break;
    case 0x1c:
	/* env3 */
	ret = siddata.voice[2].adsrvalue*255.999;
	break;
    }
#ifdef DUMPREADWRITE
    printf("read_sid(0x%04x)=0x%02x\n", address, ret);
#endif
    return ret;
}

void store_sid(ADDRESS addr, BYTE byte)
{
#ifdef DUMPREADWRITE
    printf("store_sid(0x%04x, 0x%02x)\n", addr, byte);
#endif
    if (!init_sid)
	initsid();
    addr &= 0x1f;
    switch (addr)
    {
    case 0x00:
    case 0x01:
    case 0x02:
    case 0x03:
    case 0x04:
    case 0x05:
    case 0x06:
	siddata.voice[0].data[addr - 0x00] = byte;
	break;
    case 0x07:
    case 0x08:
    case 0x09:
    case 0x0a:
    case 0x0b:
    case 0x0c:
    case 0x0d:
	siddata.voice[1].data[addr - 0x07] = byte;
	break;
    case 0x0e:
    case 0x0f:
    case 0x10:
    case 0x11:
    case 0x12:
    case 0x13:
    case 0x14:
	siddata.voice[2].data[addr - 0x0e] = byte;
	break;
    case 0x15:
    case 0x16:
    case 0x17:
    case 0x18:
    case 0x19:
    case 0x1a:
    case 0x1b:
    case 0x1c:
	siddata.data[addr - 0x15] = byte;
	break;
    }
}

#else /* !defined (TEEMU_SID) && !defined (LIONEL_SID) */

#include <stdlib.h>
#include "sid.h"

void
store_sid (ADDRESS address, BYTE byte)
{
  address &= 0x1f;
  sid[address] = byte;
  return;
}

BYTE
read_sid (ADDRESS address)
{
  address &= 0x1f;
  if (address == 0x1b)
    return rand();
  else
    return sid[address];
}

#endif
