
/* 
 * $Log: sndif.c,v $
 * Revision 1.1  1996/07/29 22:49:28  fachat
 * Initial revision
 *
 */

/* 
 * 25/04/1996 a.fachat
 *
 * Simple sound interface routines. The sound-server is running as a
 * separate program and is reading its stdin for sound changes
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>

#include "config.h"
#include "sndif.h"
#include "sound.h"

static int snd_ok = 0;
static int wrfd = 0;
static pid_t srvpid;

static void snd_exit(void) {
	kill(srvpid, SIGKILL);
}

/* 
 * snd_init forks to the sound server, setting its stdin stream to
 * the parent's (vice's) wrfd file descriptor.
 */
void snd_init(char *server) {
#ifndef SOUND
	return;
#else
        int fds[2];
	int er;

	if(pipe(fds)) {
	  perror("snd - create pipe");
	  return;
	}

	if(!(er=fork())) {
	  /* child code */
	  close(fds[1]);	/* close write end */
	  dup2(fds[0], STDIN_FILENO);
	  close(fds[0]);
	  execl(server, server, NULL);
	  perror("sndsrv - exec!");
	  while(1) sleep(1000);
	  exit(1);
	}
	if(er>0) { 		
	  srvpid = 0;
	  atexit(snd_exit);
	  /* parent code */
	  close(fds[0]);	/* close read end */
	  wrfd = fds[1];
	  snd_ok = 1;
	} else {
	  perror("snd - fork");
	  return;
	}
#endif
}


/****************** PET functions ********************************/

void snd_pet(int fl) {
	char b;
	if(!snd_ok) return;
	b = fl ? PETSND_ON:PETSND_OFF;
	write(wrfd, &b, 1);
}

void snd_pet_T(int t) {
	char b[3];
	if(!snd_ok) return;
	b[0] = PETSND_FREQ;
	b[1] = t & 0xff;
	b[2] = (t>>8) & 0xff;
	write(wrfd, b, 3);
}

void snd_pet_sample(char t) {
	char b[2];
	if(!snd_ok) return;
	b[0] = PETSND_SAMPLE;
	b[1] = t;
	write(wrfd, b, 2);
}


