/*
 * $Id: sprcoll.h,v 1.2 1996/07/29 22:51:34 ettore Exp ettore $
 *
 * This file contains the code required for sprite-sprite and sprite-background
 * collision register emulation.
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: sprcoll.h,v $
 * Revision 1.2  1996/07/29 22:51:34  ettore
 * Much faster implementation.
 *
 * Revision 1.1  1996/04/01 08:25:07  jopi
 * Initial revision
 *
 *
 */

#if !defined (_SPRCOLL_H)

#include "raster.h"

#define _SPRCOLL_H
#define BYTE unsigned char

#define SPR_SIZE	3
#define MASK_SIZE	(SPR_SIZE + 1) * 2

#if !defined (INLINE)
#if defined( NO_INLINE )
#define INLINE static
#else
#define INLINE extern __inline__
#endif /* defined (NO_INLINE) */
#endif /* !defined (INLINE) */
typedef BYTE sprmask_t;		/* mask of the sprite image */
typedef BYTE sprcollmask_t;	/* mask for colliding sprites (1 = sprite is colliding) */

typedef struct
  {
    sprmask_t mask[MASK_SIZE];	/* sprite mask */
    int size;			/* size (number of masks) */
    int relx;			/* x coordinate, relative to border */
    int startcol;		/* first column for collision check (can be <0 or >ncols) */
    int endcol;			/* last column for collision check (can be <0 or >ncols) */
    int visible;		/* != 0 if on screen */
    int ready;			/* != 0 if the mask has already been built */
  }
sprmask_struct;

void init_sprcoll (void);
void init_sprite_masks (VideoStruct * v, LineStruct * l, sprmask_struct * sprmask_return);
void check_ss_collisions (LineStruct * l, sprmask_struct * s, sprmask_t * ss_collmask_return);
void check_sb_collisions (VideoStruct * v, LineStruct * l, sprmask_struct * s,
			  sprmask_t * sb_collmask_return);
void print_binary (BYTE value);
void print_huge_mask (sprmask_t * mask, int size);

#endif /* !defined (SPRCOLL_H) */
