/*
 * $Id: types.h,v 1.8 1996/07/30 08:19:53 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 * This file declares types used all over.
 *
 *
 * Written by
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 *
 * $Log: types.h,v $
 * Revision 1.8  1996/07/30 08:19:53  ettore
 * hjoyPort.
 *
 * Revision 1.7  1996/07/29 23:06:33  ettore
 * PIXEL turned into unsigned char.
 * start_addr turned into startAddr.
 * directory, gamepath, tapeName and floppy9Name added.
 *
 * Revision 1.6  1996/04/01  09:01:41  jopi
 * New variables
 *
 * Revision 1.5  1995/11/07  16:51:00  jopi
 * *** empty log message ***
 *
 * Revision 1.4  1995/04/01  07:54:09  jopi
 * X64 0.3 PL 0
 * Typedef for signed char.
 * Room for new options for Printer charsets, Basic patch, REU size.
 *
 * Revision 1.3  1994/12/12  16:59:44  jopi
 * 16-bit portability
 *
 * Revision 1.2  1994/08/10  18:34:13  jopi
 * More changeability
 *
 * Revision 1.1  1994/06/16  17:19:26  jopi
 * Initial revision
 *
 */


#ifndef X64_TYPES_H
#define X64_TYPES_H


typedef signed char	SIGNED_CHAR;
/*typedef char		SIGNED_CHAR;*/

typedef unsigned char 	BYTE;
typedef unsigned short 	ADDRESS;
typedef unsigned short 	U_WORD;
typedef unsigned long	CLOCK;

typedef unsigned char   PIXEL;
#define PIXEL_MAX       UCHAR_MAX

typedef int		ADDR_T;
/*typedef long		ADDR_T;*/


/*
 * The resources specification contains:
 *
 * The name and class of the resource.
 * The representation type of the resource.
 * The location in the structure to keep the value of this resource.
 * The default value for this resource.
 *
 * The resources are initialized by either xdebug.c or args.c.
 */

typedef struct _AppResources {
    char* directory;	/* VICEHOME -- the work directory */
    char* gamepath;	/* ROM,game,etc library, defaults to directory */

    char* basicName;
    char* kernelName;
    char* charName;

    char* basicRev;
    char* kernelRev;

    char* exromName;
    char* module;
    char* reuName;
    char* reusize;

    char* biosName;	/* CP/M BIOS ROM */

    char* ramName;
    char* programName;
    char* reloc;
    char* startAddr;	/* PC */

    /* This one has been replaced by video80 and video40 */
    /*    char  video; */ /* true == C128 80 columns */

    char  colors;
    char  asmflag;
    char  hexflag;
    char  debugflag;
    char  traceflag;
    char  verboseflag;
    char  testflag;
    char  haltflag;
    char  notraps;

    char* tapeName;

    char* dosRomName;
    char* floppyName;
    char* floppy9Name;

    char* printCommand;
    char* PrinterLang;
    char* Locale;

    /* added 03-15-96 -- EP */

    int   joyPort;
    int   hjoyPort;
    int   checkSsColl;
    int   checkSbColl;
    int   speed;
#if defined (C128)
    int	  video80;
    int   video40;
    int   vdcRefreshRate;
    int   vicRefreshRate;
#else
#ifdef PET
    int	  video80;
    int   video40;
#endif
    int   refreshRate;
#endif

    int   mitshm;
} AppResources;


typedef struct trap_s
{
    char        *name;
    ADDRESS      address;
    BYTE         check[3];
#ifdef __STDC__
    void       (*func)(void);
#else
    void       (*func)();
#endif
} trap_t;
 

#endif  /* X64_TYPES_H */
