/*
 * $Id: vdc.c,v 1.2 1996/07/29 23:10:13 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README files for copyright notice.
 *
 *
 * VDC video chip emulation.
 *
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: vdc.c,v $
 * Revision 1.2  1996/07/29 23:10:13  ettore
 * VdcStruct introduced.
 * Variable size characters variable screen size are now supported.
 * A first rough implementation of the Hires mode done.
 * Miscellaneous bugs fixed.
 * Some garbage removed.
 *
 * Revision 1.1  1996/04/01 05:48:13  ettore
 * Initial revision
 *
 */


#define _VDC_C

#include "extern.h"
#include "config.h"
#define __VDC__
#include "raster.c"
#include "macro.h"
#include "vmachine.h"
#include "vdc.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>


/* #define VDC_REG_DEBUG */
/* #define VDC_VMODE_DEBUG */


/* global */

static BYTE vdc[37];
static BYTE vdcRAM[65535];	/* possible values: 16K, 2*32K, or 64K */
extern Window VDCborderwin;

VideoStruct *v_vdc = NULL;

struct {
  int char_size;		/* character size: 0 -> 16 bytes, 1 -> 32 bytes */
  int char_height;		/* character height (in scan lines) */
} VdcStruct;


/*

   store_vdc(): stores a value in a VDC register.

 */

void
store_vdc (BYTE reg, BYTE value)
{
#if defined (VDC_REG_DEBUG)
  printf ("VDC: store reg[ %d ] = %d\n", reg, value);
#endif

  if (reg >= 37)
    return;

  vdc[reg] = value;

  switch (reg)
    {
    case 1:			/* Columns displayed */
#if defined (VDC_REG_DEBUG)
      printf ("VDC: %d columns requested\n", value);
#endif
      vdc_resize (value, v_vdc->tlines, VdcStruct.char_height);
      return;
      
    case 6:			/* Rows displayed */
#if defined (VDC_REG_DEBUG)
      printf ("VDC: %d rows requested\n", value);
#endif
      vdc_resize (v_vdc->tcols, value, VdcStruct.char_height);
      return;
      
    case 9:			/* Scan Lines per Character */
      VdcStruct.char_size = (value & 0x1f) >= 16 ? 1 : 0;
      vdc_resize (v_vdc->tcols, v_vdc->tlines, (value & 0x1f) + 1);
      vdc[9] |= 0xe0;		/* bits 5-7 are set to 1 */
#if defined (VDC_REG_DEBUG)
      printf ("VDC: char_height set to %d, char_size set to %d.\n", VdcStruct.char_height,
	      VdcStruct.char_size);
#endif
      return;

    case 10:			/* Cursor Mode */
    case 11:
      return;

    case 12:			/* Display Address */
    case 13:
      v_vdc->screenmem = v_vdc->bitmapmem = vdcRAM + (vdc[12] << 8) + vdc[13];
#if defined (VDC_REG_DEBUG)
      printf ("VDC: screenmem = %d\n", v_vdc->screenmem - vdcRAM);
#endif
      return;

    case 14:			/* Cursor Address */
    case 15:
      return;

    case 18:			/* Video RAM Address -- RAM accessed via 31 */
    case 19:
      return;

    case 20:			/* Colour Address */
    case 21:
      v_vdc->colormem = vdcRAM + (vdc[20] << 8) + vdc[21];
#if defined (VDC_REG_DEBUG)
      printf ("VDC: colormem = %d\n", v_vdc->colormem - vdcRAM);
#endif
      return;

    case 24:			/* Vertical Scroll */
      return;

    case 25:			/* Display Modes, Horisontal Scroll */
#if defined (VDC_VMODE_DEBUG)
      if (value & 0x80)
	printf ("VDC: HIRES MODE ON\n");
#endif
      v_vdc->vmode = value & 0x80 ? VDC_BITMAP_MODE : VDC_NORMAL_MODE;
      return;

    case 26:			/* Display Colour */
      v_vdc->bgcol = value &0xf;
      return;

    case 28:			/* Char Set Address */
      v_vdc->chargenmem = vdcRAM + ((vdc[28] & 0xe0) << 8);

#if defined( VDC_REG_DEBUG )
      printf ("VDC: chargen = %d\n", v_vdc->chargenmem - vdcRAM);
#endif
      return;

    case 29:			/* Underline Scan Count */
      return;

    case 30:			/* Character Count (Copy/fill repetition) */
      vdc_perform_fillcopy ();
      return;

    case 31:			/* Video RAM Data */
      vdc_perform_fillcopy ();
      return;

    case 32:			/* Block Copy Start Address */
    case 33:
      return;
    }
  return;
}


/*

   read_vdc(): reads from one VDC register.

 */

BYTE
read_vdc (BYTE reg)
{
#if defined( VDC_REG_DEBUG )
  printf ("VDC: read [%02X] %02x\n", reg, ((reg < 37) ? vdc[reg] : 0xff));
#endif

#if !defined( USE_VDC )
  return 0xff;
#endif

  if (reg == 31)
    {
      BYTE c = vdcRAM[(vdc[18] << 8) + vdc[19]];
      /* vdc_perform_fillcopy(); */
      return c;
    }
  return ((reg < 37) ? vdc[reg] : 0xff);
}


/* 

   vdc_perform_fillcopy() performs the copy or fill operation and updates the VDC regs
   consequently.
   
 */

void
vdc_perform_fillcopy (void)
{
  int ptr, ptr2;
  int blklen = vdc[30] ? vdc[30] : 1; /* I hope this one is correct... :P */
  
  ptr2 = vdc[32] * 256 + vdc[33];
  ptr = vdc[18] * 256 + vdc[19];

  if (vdc[24] & 0x80)
    {
      int i;

#if defined( VDC_REG_DEBUG )
      printf ("VDC: copying %d bytes from %d to %d\n", blklen, ptr2, ptr); 
#endif
      for (i = 0; i < blklen; i++)
	vdcRAM[ptr + i] = vdcRAM[ptr2 + i];
      ptr2 += blklen;
      vdc[32] = (ptr2 & 0xff00) >> 8;
      vdc[33] = ptr2 & 0xff;
    }
  else
    {
      memset (vdcRAM + ptr, vdc[31], blklen);
#if defined( VDC_REG_DEBUG)
      printf ("VDC: filling %d bytes with value %d from %d\n", blklen, vdc[31], ptr);
#endif
    }
  ptr += blklen;
  vdc[19] = ptr & 0xff;
  vdc[18] = (ptr & 0xff00) >> 8;
  vdc[30] = 0;

#if defined( VDC_REG_DEBUG )
  printf ("VDC: (now)vdc18 = %d vdc19 = %d\n", vdc[18], vdc[19]);
#endif
}


/*

   vdc_resize(): resizes the VDC window so that it can contain tcols columns and tlines lines of
   characters 8 pixel wide and char_height pixel tall. 

 */

void
vdc_resize (int tcols, int tlines, int char_height)
{
  int diffpix;

  RefreshAll (v_vdc);		/* the refresh must be done _before_ we change the values... */

  /* we arrange the columns first... */
  diffpix = (tcols - v_vdc->tcols) * 8;
  v_vdc->tcols = tcols;
  if ((v_vdc->xpix += diffpix) > SCR_COLS)
    {
      diffpix = (SCR_COLS / 8) - v_vdc->tcols;
      v_vdc->xpix = SCR_COLS / 8;
      v_vdc->tcols = SCR_COLS / 8;
      fprintf (stderr, "VDC: too many columns requested -- forcing %d.\n", v_vdc->tcols);
    }
  v_vdc->winwidth += diffpix * PIXEL_YSIZE;

  /* ...then we do the same with the rows. */
  diffpix = tlines * char_height - v_vdc->tlines * VdcStruct.char_height;
  v_vdc->tlines = tlines;
  if ((v_vdc->scrlines += diffpix) > SCR_LINES)
    {
      diffpix = (SCR_LINES / VdcStruct.char_height) - v_vdc->tlines;
      v_vdc->ypix = (SCR_LINES - v_vdc->bordheight * 2) / VdcStruct.char_height;
      v_vdc->tlines = (SCR_LINES - v_vdc->bordheight * 2) / VdcStruct.char_height;
      fprintf (stderr, "VDC: too many rows requested -- forcing %d.\n", v_vdc->tlines);
    }
  v_vdc->winheight += diffpix * PIXEL_YSIZE;
  v_vdc->ypix += diffpix * PIXEL_YSIZE;
  VdcStruct.char_height = char_height;

  /* NOTE: I know this is a really nasty trick -- but it is necessary (at least for now)... */
  v_vdc->rasterline = 0;
  v_vdc->scrp = (PIXEL *)v_vdc->scrimg->data;
  v_vdc->refstruct.num_elem = 0;
  resize_window (v_vdc->display, v_vdc->win, v_vdc->winwidth, v_vdc->winheight);
  ArrangeVideoImage (v_vdc, v_vdc->winwidth, v_vdc->winheight);
  return;
}


/*

   VicII_Init(): Initializes the VIC-II emulation.

 */

Window
Vdc_Init (Display * disp, int scr_num, char *win_name, char *icon_name, char *res_class,
	  char *res_name, int *w, int *h, char *icon_data[], Colormap * cmap_return,
	  int exact_speed, int asleep, int refrate, Atom * wm_delwin)
{
  v_vdc = AllocVideo (SCR_COLS, SCR_LINES, BORD_WIDTH, BORD_HEIGHT, XPIX, YPIX, XSIZE, YSIZE,
		      VDC_NUMCOLORS, refrate, VDC_NUMSPRITES, exact_speed, win_name, asleep);
  if (!v_vdc)
    {
      fprintf (stderr, "fatal error: can't initialize Vdc emulation.\n");
      return -1;
    }
  if (OpenVideoWindow (disp, scr_num, v_vdc, win_name, win_name, res_class, res_name, icon_data,
		       vdc_cnames, cmap_return, wm_delwin))
    {
      fprintf (stderr, "fatal error: can't open window for Vdc emulation.\n");
      return -1;
    }
  if (SetVideoModes (v_vdc, 2,
  /* VDC_NORMAL_MODE */
		     Vdc_GetTextData, Vdc_GetColorData,
		     NormalGetBgDataFromRegister, SPECIAL_VMODE,
		     Vdc_DrawLine,
  /* VDC_BITMAP_MODE */
		     Vdc_GetBitmapData, NullGetFunct,
		     NormalGetBgDataFromRegister, SPECIAL_VMODE,
		     Vdc_DrawMonoLine))
    {
      fprintf (stderr, "fatal error: can't initialize Vdc emulation.\n");
      return -1;
    }
  v_vdc->vmode = VDC_NORMAL_MODE;
  RefreshAll (v_vdc);
  *w = v_vdc->winwidth;
  *h = v_vdc->winheight;
  v_vdc->chargenmem = char_rom;
  v_vdc->screenmem = v_vdc->bitmapmem = vdcRAM;
  v_vdc->colormem = vdcRAM + 2000;
  v_vdc->scrlines = v_vdc->ypix + v_vdc->bordheight * 2;
  memset (v_vdc->screenmem, 0, 2000);
  memset (v_vdc->colormem, 1, 2000);
  VdcStruct.char_size = 0;
  VdcStruct.char_height = 8;
  return (v_vdc->win);
}


/*

   The following functions are used to read the screen data (see raster.c for more info)

 */

INLINE BYTE
Vdc_GetCharData (BYTE * coloraddr, BYTE * charaddr, int ll, BYTE * charmemlo, BYTE * charmemup,
		 BYTE flashpattern, BYTE curspattern, BYTE underlinepattern, BYTE * curspos)
{
  BYTE b;

  b = ((*coloraddr) & VDC_ALTCHARSET_ATTR ? charmemlo : charmemup)
    [((int)(*charaddr) << (4 + VdcStruct.char_size)) + ll];
  
  if (*coloraddr & VDC_UNDERLINE_ATTR)
    b |= underlinepattern;
  if (*coloraddr & VDC_REVERSE_ATTR)
    b = ~b;
  if (*coloraddr & VDC_FLASH_ATTR)
    b &= flashpattern;
  if (charaddr == curspos && curspattern)
    b = ~b;
  return b;
}


/*

   This custom GetTextData() function supports the VDC-specific features: it would have been a
   waste of time and space to have them implemented in the general-pourpose NormalGetTextData
   which is used for the other video chip emulations.

*/

int 
Vdc_GetTextData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  int i = 0, ll = l->n % VdcStruct.char_height, line = l->n / VdcStruct.char_height;
  BYTE *curspos = vdcRAM + (vdc[14] << 8) + vdc[15];	/* cursor position */
  BYTE *coloraddr = v->colormem + line * v->tcols, *src = v->screenmem + line * v->tcols;
  BYTE *dest = l->fgdata, *charmemup = v->chargenmem, *charmemlo = v->chargenmem + 0x1000;
  BYTE curspattern, underlinepattern, flashpattern;

  curspattern = (vdc[10] & 0x60) >> 5;	/* bits 5-6 of register #10 specify cursor blink mode: */
  if (curspattern == 0x1)	        /* 01: no cursor at all */
    curspattern = 0;
  else
    {
      if (ll >= (vdc[10] & 0x1f) && ll <= vdc[11])
	{
	  if (!curspattern)	        /* 00: fixed cursor */
	    curspattern = 0xff;
	  else if (curspattern & 0x1)   /* 11: 1/32 blink */
	    curspattern = ((clk / CYCLES_PER_RFSH) & 0x20) ? 0xff : 0;
	  else                          /* 10: 1/16 blink */
	    curspattern = ((clk / CYCLES_PER_RFSH) & 0x10) ? 0xff : 0;
	}
      else
	curspattern = 0;
    }
  
  /* register #29 specifies the scan line for underlining */
  underlinepattern = (ll == vdc[29] & 0x1f) ? 0xff : 0;
  flashpattern = ((clk / CYCLES_PER_RFSH) & 0x20) ? 0 : 0xff;

  if (no_check)
    {
      *xs = 0;
      *xe = v->tcols - 1;
      for (; i < v->tcols; i++, src++)
	dest[i] = Vdc_GetCharData (&coloraddr[i], src, ll, charmemlo, charmemup, flashpattern,
				   curspattern, underlinepattern, curspos);
      return 1;
    }
  else
    {
      BYTE b;
      for (; i < v->tcols; i++, src++)
	{
	  b = Vdc_GetCharData (&coloraddr[i], src, ll, charmemlo, charmemup, flashpattern,
			       curspattern, underlinepattern, curspos);
	  if (b != dest[i])
	    break;
	}
      if (i < v->tcols)
	{
	  *xs = *xe = i;
	  for (; i < v->tcols; i++, src++)
	    {
	      b = Vdc_GetCharData (&coloraddr[i], src, ll, charmemlo, charmemup, flashpattern,
				   curspattern, underlinepattern, curspos);
	      if (dest[i] != b)
		{
		  dest[i] = b;
		  *xe = i;
		}
	    }
	  return 1;
	}
      else
	return 0;
    }
}

/*

   Unluckily, the pixel order in the VDC is different from the one of the VIC-II, so we have to
   define a different GetBitmapData function here...

 */

int
Vdc_GetBitmapData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  return GetData (l->fgdata, v->bitmapmem + l->n * v->tcols, v->tcols, 1, xs, xe, no_check);
}
		   
int
Vdc_GetColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check)
{
  BYTE *addr = v->colormem + (l->n / VdcStruct.char_height) * v->tcols;

  return (GetDataHiLo (l->colordata1, l->colordata2, addr, v->tcols, 1, xs, xe, no_check));
}

void
Vdc_DrawLine (PIXEL * p, LineStruct * l, int xs, int xe, PIXEL * colors)
{
  BYTE *colorp1, *colorp2, *pixelp;
  int i;

  colorp1 = &(l->colordata1[0]);
  colorp2 = &(l->colordata2[0]);
  pixelp = &(l->fgdata[0]);
  p += xs * CHAR_XSIZE;
  for (i = xs; i <= xe; i++, p += CHAR_XSIZE)
    {
      register BYTE byte = pixelp[i];
      register PIXEL fg = colors[colorp2[i]];
      PUTMONO(byte, fg);
    }
}

/* this saves some time when in monochrome mode ... */

void
Vdc_DrawMonoLine (PIXEL * p, LineStruct * l, int xs, int xe, PIXEL * colors)
{
  int i;
  BYTE * pixelp = &(l->fgdata[0]);
  register BYTE byte;
  register PIXEL fg;
  
  p += xs * CHAR_XSIZE;
  fg = colors[vdc[26] >> 4];
  for (i = xs; i <= xe; i++, p += CHAR_XSIZE)
    {
      byte = pixelp[i];
      PUTMONO(byte, fg);
    }
}


/*

   Vdc_ArrangeWindow(): centers the image at the center of the window.

 */

void
Vdc_ArrangeWindow (int w, int h)
{
  ArrangeVideoImage (v_vdc, w, h);
  RefreshAll (v_vdc);
}

void
Vdc_Free (void)
{
  FreeVideo (v_vdc);
}

void
Vdc_Pause (void)
{
  PauseVideo (v_vdc);
}

void
Vdc_Restart (void)
{
  RestartVideo (v_vdc);
}

void
Vdc_DoRaster (void)
{
  Raster (v_vdc);
}


/*

   The following is needed to turn Vdc emulation on and off in x128.

 */

void
Vdc_SetStatus (int awake)
{
  set_window_mapping (v_vdc->display, v_vdc->win, !(v_vdc->asleep = !awake));
  SetSpeed (speed);
}
