/*
 * $Id: vdc.h,v 1.2 1996/07/29 23:11:14 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README files for copyright notice.
 *
 *
 * Header file for VDC chip emulation.
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: vdc.h,v $
 * Revision 1.2  1996/07/29 23:11:14  ettore
 * Some little changes to match rev. 1.2 of vdc.c.
 *
 * Revision 1.1  1996/04/01 05:48:36  ettore
 * Initial revision
 *
 */

#if !defined( _VDC_H )
#define _VDC_H

#define NEEDS_NormalGetBgDataFromRegister
#define NEEDS_GetDataHiLo
#define NEEDS_NullGetFunct

#include "raster.h"

#if defined( _VDC_C )

#ifdef BYTES_PER_CHAR
#undef BYTES_PER_CHAR
#endif
#ifdef XPIX
#undef XPIX
#endif
#ifdef XSIZE
#undef XSIZE
#endif

/* just preliminary values -- the complete implementation should behave differently... */

#undef BORD_WIDTH
#undef BORD_HEIGHT
#undef SCR_COLS

#define BYTES_PER_CHAR 16
#define BORD_HEIGHT   5
#define BORD_WIDTH    5
#define SCR_COLS    720		/* full overscan x size */
#undef SCR_LINES
#define SCR_LINES   500		/* I hope this is enough... */
#define XPIX	    640
#define YPIX        200
#define XSIZE        80
#define YSIZE        25

#endif /* defined (_VDC_C) */

#define VDC_NUMSPRITES 0


/* color definitions */

#define VDC_NUMCOLORS 	16

#if defined(_VDC_C)
char *vdc_cnames[] =
{
  "#000000",			/* Black */
  "#404040",			/* Dark Gray */
  "#000080",			/* Blue */
  "#0000ff",			/* Light Blue */
  "#008000",			/* Green */
  "#00ff00",			/* Light Green */
  "#008080",			/* Cyan */
  "#00ffff",			/* Light Cyan */
  "#800000",			/* Red */
  "#ff0000",			/* Light Red */
  "#800080",			/* Purple */
  "#ff00ff",			/* Light Purple */
  "#808000",			/* Yellow */
  "#ffff00",			/* Light Yellow */
  "#808080",			/* Light Gray */
  "#ffffff"			/* White */
};
#endif /* _VDC_C */


/* VDC attribute masks */

#define VDC_ALTCHARSET_ATTR	0x80
#define VDC_REVERSE_ATTR	0x40
#define VDC_UNDERLINE_ATTR	0x20
#define VDC_FLASH_ATTR		0x10


/* Video mode definitions */

#define VDC_NORMAL_MODE             0
#define VDC_BITMAP_MODE             1


/* Prototypes... */

void store_vdc (BYTE reg, BYTE value);
BYTE read_vdc (BYTE reg);
void vdc_perform_fillcopy (void);
void vdc_resize (int tcols, int tlines, int char_height);
void Vdc_SetVModes (void);
Window Vdc_Init (Display * disp, int scr_num, char *win_name, char *icon_name, char *resclass,
		 char *res_name, int *w, int *h, char *icon_data[], Colormap * cmap_return,
		 int exact_speed, int asleep, int refrate, Atom * wm_delwin);
void Vdc_DrawLine (PIXEL * p, LineStruct * l, int xs, int xe, PIXEL * colors);
void Vdc_DrawMonoLine (PIXEL * p, LineStruct * l, int xs, int xe, PIXEL * colors);
int Vdc_GetColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int Vdc_GetTextData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int Vdc_GetBitmapData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
void Vdc_ArrangeWindow (int width, int height);
void Vdc_Free (void);
void Vdc_Restart (void);
void Vdc_Pause (void);
void Vdc_DoRaster (void);
void Vdc_SetStatus (int awake);

#endif /* _VDC_H */
