/*
 * $Id: via1.c,v 1.1 1996/04/18 17:15:13 fachat Exp ettore $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 *
 * Written by
 *   Vesa-Matti Puro  (vmp@lut.fi)
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 * Timer fixes by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 * $Log: via1.c,v $
 * Revision 1.1  1996/04/18 17:15:13  fachat
 * Initial revision
 *
 */

#include <stdio.h>
#include <time.h>

#include "config.h"
#include "vmachine.h"
#include "extern.h"
#include "via.h"

#include "interrupt.h"

/*#define VIA1_TIMER_DEBUG */

/* global */

BYTE    via1[16];
BYTE    keyarr[KBD_ROWS];
int     via1ta_stop = 0; /* maybe 1? */
int     via1tb_stop = 0; /* maybe 1? */
int     via1ta_interrupt = 0;
int     via1tb_interrupt = 0;

/* local functions */

static void update_via1ta ( int );
static void update_via1tb ( int );

/*
 * Local variables
 */

static int   via1ifr;	/* Interrupt Flag register for via 1 */
static int   via1ier;	/* Interrupt Enable register for via 1 */

static int via1ta;	/* value of via 1 timer A at last update */
static int via1tb;	/* value of via 1 timer B at last update */

static CLOCK via1tau;	/* time when via 1 timer A is updated */
static CLOCK via1tbu;	/* time when via 1 timer B is updated */


/* ------------------------------------------------------------------------- */
/* VIA 1 */


void    reset_via1(void)
{
    int i;
    if(debugflg) printf("VIA1: reset\n");
    /* clear registers */
    for(i=0;i<4;i++) via1[i]=0;
    for(i=11;i<16;i++) via1[i]=0;

/*
 * The following line doesn't appear in the specs...
 */
    /*via1[VIA_ACR] = 0x40;*/  /* timer T1 run mode */

    /* disable vice interrupts */
    setirq(I_VIA1T1, 0); unsetalarm(I_VIA1T1);
    setirq(I_VIA1T2, 0); unsetalarm(I_VIA1T2);
    setirq(I_VIA1SR, 0); unsetalarm(I_VIA1SR);
    setirq(I_VIA1FL, 0); unsetalarm(I_VIA1FL);
    via1ier = 0;
    via1ifr = 0;
}


void    store_via1(ADDRESS addr, BYTE byte)
{
    if (debugflg)
	printf("store via1[%x] %x\n", (int) addr, (int) byte);


    switch (addr) {

	/* these are done with saving teh value */
      case VIA_PRA: /* port A */
      case VIA_PRA_NHS: /* port A, no handshake */
      case VIA_PRB: /* port B */
      case VIA_DDRA: 
      case VIA_DDRB:
      case VIA_SR: /* Serial Port output buffer */
	via1[addr] = byte;
	break;

	/* Timers */

      case VIA_T1CL:
      case VIA_T1LL:
	via1[VIA_T1LL] = byte;
	break;

      case VIA_T1CH /*TIMER_AH*/: /* Write timer A high */
	if(debugflg) printf("Write timer A high: %02x\n",byte);
	via1[VIA_T1LH] = byte;
        /* load counter with latch value */
        via1[VIA_T1CL] = via1[VIA_T1LL];
        via1[VIA_T1CH] = via1[VIA_T1LH];
        /* Clear T1 interrupt */
        via1ifr &= ~VIA_IM_T1;
        setirq(I_VIA1T1, 0);
        update_via1ta(1);
        break;

      case VIA_T1LH: /* Write timer A high order latch */
        via1[addr] = byte;
        /* Clear T1 interrupt */
        via1ifr &= ~VIA_IM_T1;
        setirq(I_VIA1T1, 0);
        break;

      case VIA_T2CH: /* Write timer 2 high */
        via1[VIA_T2CH] = byte;
        via1[VIA_T2CL] = via1[VIA_T2LL]; /* bogus, both are identical */
        update_via1tb(1);
        /* Clear T2 interrupt */
        via1ifr &= ~VIA_IM_T2;
        setirq(I_VIA1T2, 0);
        break;

	/* Interrupts */

      case VIA_IFR: /* 6522 Interrupt Flag Register */
        via1ifr &= ~byte;
        if(!via1ifr & VIA_IM_T1) setirq(I_VIA1T1, 0);
        if(!via1ifr & VIA_IM_T2) setirq(I_VIA1T2, 0);
        break;

      case VIA_IER: /* Interrupt Enable Register */
#if defined (VIA1_TIMER_DEBUG)
        printf ("Via#1 set VIA_IER: 0x%x\n", byte);
#endif
        if (byte & VIA_IM_IRQ) {
            /* set interrupts */
            if ((byte & VIA_IM_T1) && (via1ifr & VIA_IM_T1)) {
                setirq(I_VIA1T1, 1);
            }
            if ((byte & VIA_IM_T2) && (via1ifr & VIA_IM_T2)) {
                setirq(I_VIA1T2, 1);
            }
            via1ier |= byte & 0x7f;
        }
        else {
            /* clear interrupts */
            if( byte & VIA_IM_T1 ) setirq(I_VIA1T1, 0);
            if( byte & VIA_IM_T2 ) setirq(I_VIA1T2, 0);
            via1ier &= ~byte;
        }
        break;

	/* Control */

      case VIA_ACR:
	via1[addr] = byte;

	/* bit 7 timer 1 output to PB7 */
	/* bit 6 timer 1 run mode -- default seems to be continuous */

	/* bit 5 timer 2 count mode */
	if (byte & 32) {
/* TODO */
/*	    update_via1tb(0);*/	/* stop timer if mode == 1 */
	}

	/* bit 4, 3, 2 shift register control */

	/* bit 1, 0  latch enable port B and A */
	break;

      case VIA_PCR:
	via1[addr] = byte;

	/* bit 7, 6, 5  CB2 handshake/interrupt control */
	/* bit 4  CB1 interrupt control */

	/* bit 3, 2, 1  CA2 handshake/interrupt control */
	/* bit 0  CA1 interrupt control */

	break;

      default:
	via1[addr] = byte;

    }  /* switch */
}


/* ------------------------------------------------------------------------- */

BYTE    read_via1(ADDRESS addr)
{
    if (debugflg)
	printf("read via1[%d]\n", addr);

    switch (addr) {

      case VIA_PRA: /* port A */
      case VIA_PRA_NHS: /* port A, no handshake */
	{
	    int     t, i;
	    BYTE    j, v = 1;
	    t = ~(via1[VIA_PRB] & via1[VIA_DDRB]);
	    for (j = i = 0; i < KBD_ROWS; v <<= 1, i++)
		if (t & keyarr[i])
		    j |= v;
	    j = ~j;

	    if (debugflg) {
		printf("read port A %d\n", j);
		printf("a: %x b:%x  ca: %x cb: %x joy: %x\n",
		       (int) j, (int) via1[VIA_PRB],
		       (int) via1[VIA_DDRA], (int) via1[VIA_DDRB], joy[2]);
	    }
	    return (((j & ~via1[VIA_DDRA]) | (via1[VIA_PRA] & via1[VIA_DDRA])) & ~joy[2]);
	}

      case VIA_PRB: /* port B */
	{
	    int     t, i;
	    BYTE    j;
	    t = via1[VIA_PRA] & via1[VIA_DDRA];  /* intr() updates keyboard */

	    for (j = i = 0; i < KBD_ROWS; t >>= 1, i++)
		if (!(t & 1))
		    j |= keyarr[i];
	    j = ~j;

	    if (debugflg) {
		printf("read port B %d\n", j);
		printf("a: %x b:%x  ca: %x cb: %x joy: %x\n",
		       (int) via1[VIA_PRA], (int) j,
		       (int) via1[VIA_DDRA], (int) via1[VIA_DDRB], joy[1]);
	    }
	    return (((j & ~via1[VIA_DDRB]) | (via1[VIA_PRB] & via1[VIA_DDRB])) & ~joy[1]);
	}


	/* Timers */

      case VIA_T1CL /*TIMER_AL*/: /* timer A low */
        via1ifr &= ~VIA_IM_T1;
        setirq(I_VIA1T1, 0);
        return ((via1ta - clk + via1tau) & 0xff);


      case VIA_T1CH /*TIMER_AH*/: /* timer A high */
        return (((via1ta - clk + via1tau) >> 8) & 0xff);

      case VIA_T2CL /*TIMER_BL*/: /* timer B low */
        via1ifr &= ~VIA_IM_T2;
        setirq(I_VIA1T2, 0);
        return ((via1tb - clk + via1tbu) & 0xff);

      case VIA_T2CH /*TIMER_BH*/: /* timer B high */
        return (((via1tb - clk + via1tbu) >> 8) & 0xff);

      case VIA_SR: /* Serial Port Shift Register */
	return (via1[addr]);

	/* Interrupts */

      case VIA_IFR: /* Interrupt Flag Register */
	{
	    BYTE    t = via1ifr; 
	    if (via1ifr & via1ier /*[VIA_IER]*/)
		t |= 0x80;
	    return (t);
	}

      case VIA_IER: /* 6522 Interrupt Control Register */
	    return (via1ier /*[VIA_IER]*/ | 0x80);

    }  /* switch */

    return (via1[addr]);
}


/* ------------------------------------------------------------------------- */

int    int_via1t1(void)
{
    if (debugflg)
	printf("via1 timer A interrupt\n");

    if (!(via1[VIA_ACR] & 0x40)) { /* one-shot mode */
#if defined (VIA1_TIMER_DEBUG)
        printf ("Via#1 Timer A interrupt -- one-shot mode: next int won't happen\n");
#endif
	via1ta = 0;
	via1tau = clk;
	unsetalarm(I_VIA1T1);		/*int_clk[I_VIA1T1] = 0;*/
    }
    else {		/* continuous mode */
        /* load counter with latch value */
        via1[VIA_T1CL] = via1[VIA_T1LL];
        via1[VIA_T1CH] = via1[VIA_T1LH];
	update_via1ta(1);
/*	int_clk[I_VIA1T1] = via1tau + via1ta;*/
    }
    via1ifr |= VIA_IM_T1;
    if(via1ier /*[VIA_IER]*/ & VIA_IM_T1 ) setirq(I_VIA1T1, 1);

    return 0; /*(viaier & VIA_IM_T1) ? 1:0;*/
}

/*
 * Timer B is always in one-shot mode
 */

int    int_via1t2(void)
{
    if (debugflg)
	printf("via1 timer B interrupt\n");

    via1tb = 0;
    via1tbu = clk;
    unsetalarm(I_VIA1T2);	/*int_clk[I_VIA1T2] = 0;*/

    via1ifr |= VIA_IM_T2;
    if( via1ier & VIA_IM_T2 ) setirq(I_VIA1T2, 1);

    return 0; 
}


/* ------------------------------------------------------------------------- */

static void update_via1ta(int force)
{
    if(force) {
       if(debugflg)
          printf("update via timer A : latch=%d, counter =%d, clk = %ld\n",
                via1[VIA_T1CL] + (via1[VIA_T1CH] << 8),
                via1[VIA_T1LL] + (via1[VIA_T1LH] << 8),
                clk);

      via1ta = via1[VIA_T1CL] + (via1[VIA_T1CH] << 8);
      via1tau = clk;
      setalarm(I_VIA1T1, via1ta);
    }
}

static void update_via1tb(int force)
{
    if(force) {
      via1tb = via1[VIA_T2CL] + (via1[VIA_T2CH] << 8);
      via1tbu = clk;
      setalarm(I_VIA1T2, via1tb);
    }
}

int     show_keyarr(void)
{
    int     i, j;
    for (j = 0; j < KBD_ROWS; j++) {
	printf("%d:", j);
	for (i = 0x80; i; i >>= 1)
	    printf(" %d", keyarr[j] & i);
	printf("\n");
    }
    return (0);
}
