/*
 * $Id: via2.c,v 1.1 1996/04/18 17:16:32 fachat Exp ettore $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 *
 * Written by
 *   Vesa-Matti Puro  (vmp@lut.fi)
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: via2.c,v $
 * Revision 1.1  1996/04/18 17:16:32  fachat
 * Initial revision
 *
 */

#include <stdio.h>
#include <time.h>

#include "config.h"
#include "vmachine.h"
#include "extern.h"
#include "via.h"
#include "interrupt.h"

/*#define VIA2_TIMER_DEBUG */

/* global */

BYTE    via2[16];
int     via2ta_stop = 1;
int     via2tb_stop = 1;
int	via2ta_interrupt = 0;
int	via2tb_interrupt = 0;

/* local functions */

static void update_via2ta ( int );
static void update_via2tb ( int );

/*
 * Local variables
 */

static int via2ifr;	/* Interrupt Flag register for cia 2 */
static int via2ier;	/* Interrupt Flag register for cia 2 */

static int via2ta;	/* value of cia 2 timer A at last update */
static int via2tb;	/* value of cia 2 timer B at last update */

static CLOCK via2tau;	/* time when cia 2 timer A is updated */
static CLOCK via2tbu;	/* time when cia 2 timer B is updated */

/* ------------------------------------------------------------------------- */
/* VIA 2 */


void    reset_via2(void)
{
    int i;
    if(debugflg) printf("VIA2: reset\n");
    /* clear registers */
    for(i=0;i<4;i++) via2[i]=0;
    for(i=11;i<16;i++) via2[i]=0;

/*
 * The following line doesn't appear in the specs...
 */
    /*via2[VIA_ACR] = 0x40;*/  /* timer T1 run mode */

    /* disable vice interrupts */
    setirq(I_VIA2T1, 0); unsetalarm(I_VIA2T1);
    setirq(I_VIA2T2, 0); unsetalarm(I_VIA2T2);
    setirq(I_VIA2SR, 0); unsetalarm(I_VIA2SR);
    setirq(I_VIA2FL, 0); unsetalarm(I_VIA2FL);
    via2ier = 0;
    via2ifr = 0;
}


void    store_via2(ADDRESS addr, BYTE byte)
{
    if (debugflg)
	printf("store via2[%x] %x\n", (int) addr, (int) byte);

    switch (addr) {

      case VIA_PRA: /* port A */
      case VIA_PRA_NHS: /* port A, no handshake */
#if 1
      case VIA_PRB:
      case VIA_DDRA:
      case VIA_DDRB:
	via2[addr] = byte;
	break;
#else
	    via2[VIA_PRA] = (via2[VIA_PRA] & ~via2[VIA_DDRA]) |
		(byte & via2[VIA_DDRA]);
	break;

      case VIA_PRB: /* port B */
	via2[VIA_PRB] = (via2[VIA_PRB] & ~via2[VIA_DDRB]) |
	    (byte & via2[VIA_DDRB]);
	break;
#endif

	/* Timers */

      case VIA_T1CL:
      case VIA_T1LL:
        via2[VIA_T1LL] = byte;
        break;

      case VIA_T1CH /*TIMER_AH*/: /* Write timer A high */
        if(debugflg) printf("Write timer A high: %02x\n",byte);
        via2[VIA_T1LH] = byte;
        /* load counter with latch value */
        via2[VIA_T1CL] = via2[VIA_T1LL];
        via2[VIA_T1CH] = via2[VIA_T1LH];
        /* Clear T1 interrupt */
        via2ifr &= ~VIA_IM_T1;
        setirq(I_VIA2T1, 0);
        update_via2ta(1);
        break;

      case VIA_T1LH: /* Write timer A high order latch */
        via2[addr] = byte;
        /* Clear T1 interrupt */
        via2ifr &= ~VIA_IM_T1;
        setirq(I_VIA2T1, 0);
        break;

      case VIA_T2CH: /* Write timer 2 high */
        via2[VIA_T2CH] = byte;
        via2[VIA_T2CL] = via2[VIA_T2LL]; /* bogus, both are identical */
        update_via2tb(1);
        /* Clear T2 interrupt */
        via2ifr &= ~VIA_IM_T2;
        setirq(I_VIA2T2, 0);
        break;

#if 0 
      case VIA_T1CH /*TIMER_AH*/: /* Write timer A high */
	update_via2ta(1);
      case VIA_T1LH /*TIMER_L_AH*/: /* Write timer A high order latch */
	/* Clear T1 interrupt */
	via2[addr] = byte;
	break;

      case VIA_T2CH /*TIMER_BH*/: /* Write timer B high */
	via2[addr] = byte;
	/* Clear T2 interrupt */
	update_via2tb(1);
	break;

      case VIA_SR: /* Serial Port output buffer */
	via2[addr] = byte;
	break;

#endif /* 0 */

	/* Interrupts */

      case VIA_IFR: /* 6522 Interrupt Flag Register */
        via2ifr &= ~byte;
        if(!via2ifr & VIA_IM_T1) setirq(I_VIA2T1, 0);
        if(!via2ifr & VIA_IM_T2) setirq(I_VIA2T2, 0);
#if 0
	via2int &= ~byte;
#endif
	break;

      case VIA_IER: /* Interrupt Control Register */
#if defined (VIA2_TIMER_DEBUG)
        printf ("Via#2 set VIA_IER: 0x%x\n", byte);
#endif
        if (byte & VIA_IM_IRQ) {
            /* set interrupts */
            if ((byte & VIA_IM_T1) && (via2ifr & VIA_IM_T1)) {
                setirq(I_VIA2T1, 2);
            }
            if ((byte & VIA_IM_T2) && (via2ifr & VIA_IM_T2)) {
                setirq(I_VIA2T2, 2);
            }
            via2ier |= byte & 0x7f;
        }
        else {
            /* clear interrupts */
            if( byte & VIA_IM_T1 ) setirq(I_VIA2T1, 0);
            if( byte & VIA_IM_T2 ) setirq(I_VIA2T2, 0);
            via2ier &= ~byte;
        }
        break;

#if 0
	if (byte & VIA_IM_IRQ) {
	    /* set interrupts */
	    if (byte & VIA_IM_T1) {
		via2ta_interrupt = 1;
#if defined (VIA2_TIMER_DEBUG)
	        printf ("Via#2 Timer A interrupt enabling requested\n");
#endif
	    }
	    if (byte & VIA_IM_T2) {
	        via2ta_interrupt = 1;
#if defined (VIA2_TIMER_DEBUG)
	        printf ("Via#2 Timer B interrupt enabling requested\n");
#endif
	    }
	    if ((byte & VIA_IM_T1) && !via2ta_stop) { /* EP */
		update_via2ta(0);
		int_clk[I_VIA2T1] = via2tau + via2ta;
#if defined (VIA2_TIMER_DEBUG)
		printf ("Via#2 Timer A interrupt enabled\n");
#endif
	    }
	    if ((byte & VIA_IM_T2) && !via2tb_stop) { /* EP */
		update_via2tb(0);
		int_clk[I_VIA2T2] = via2tbu + via2tb;
#if defined (VIA2_TIMER_DEBUG)
		printf ("Via#2 Timer B interrupt enabled\n");
#endif
	    }
	    if (byte & VIA_IM_SR) {
		int_clk[I_VIA2SR] = 0;	/* serial data empty/full not impl. */
	    }

	    if (byte & (VIA_IM_CB1 | VIA_IM_CB2 | VIA_IM_CA1 |VIA_IM_CA2))
		int_clk[I_VIA2FL] = 0;	/* FLAG1/ RESTORE key */

	    via2[addr] |= byte & 0x7f;
	}
	else {
	    /* clear interrupts */

	    if (byte & VIA_IM_T1) {
		int_clk[I_VIA2T1] = 0;
#if defined (VIA2_TIMER_DEBUG)
		printf ("Via#2 Timer A interrupt disabled\n");
#endif
		via2ta_interrupt = 0;
		/* cia1ta_stop = 1; */
    	    }
	    if (byte & VIA_IM_T2) {
		int_clk[I_VIA2T2] = 0;
#if defined (VIA2_TIMER_DEBUG)
	        printf ("Via#2 Timer B interrupt disabled\n");
#endif
		via2tb_interrupt = 0;
	        /* via2tb_stop = 1; */
	    }
	    if (byte & VIA_IM_SR)
		int_clk[I_VIA2SR] = 0;
	    if (byte & (VIA_IM_CB1 | VIA_IM_CB2 | VIA_IM_CA1 | VIA_IM_CA2))  /* only 1 used */
		int_clk[I_VIA2FL] = 0;

	    via2[addr] &= ~byte;
	}

	find_next_int();
#ifdef NEW_TIMER_CODE
	update_intr_alarm();
#endif
	break;

#endif /* 0 */

	/* Control */

      case VIA_ACR:
	via2[addr] = byte;

	/* bit 7 timer 1 output to PB7 */
	/* bit 6 timer 1 run mode -- default seems to be continuous */

	/* bit 5 timer 2 count mode */
	if (byte & 32) {
/* TODO */
/*	    update_via2tb(0);*/	/* stop timer if mode == 1 */
	}

	/* bit 4, 3, 2 shift register control */

	/* bit 1, 0  latch enable port B and A */
	break;

      case VIA_PCR:
	via2[addr] = byte;

	/* bit 7, 6, 5  CB2 handshake/interrupt control */
	/* bit 4  CB1 interrupt control */

	/* bit 3, 2, 1  CA2 handshake/interrupt control */
	/* bit 0  CA1 interrupt control */

	break;

      default:
	via2[addr] = byte;

    }  /* switch */
}


/* ------------------------------------------------------------------------- */

BYTE    read_via2(ADDRESS addr)
{
    if (debugflg)
        printf("read via2[%d]\n", addr);

    switch (addr) {

      case VIA_PRA: /* port A */
      case VIA_PRA_NHS: /* port A, no handshake */
	return ((via2[VIA_PRA] & via2[VIA_DDRA]) | (0x7f & ~via2[VIA_DDRA]));

      case VIA_PRB: /* port B */
	return ((via2[VIA_PRB] & via2[VIA_DDRB]) | (0xff & ~via2[VIA_DDRB]));


	/* Timers */

      case VIA_T1CL /*TIMER_AL*/: /* timer A low */
        via2ifr &= ~VIA_IM_T1;
        setirq(I_VIA2T1, 0);
        return ((via2ta - clk + via2tau) & 0xff);


      case VIA_T1CH /*TIMER_AH*/: /* timer A high */
        return (((via2ta - clk + via2tau) >> 8) & 0xff);

      case VIA_T2CL /*TIMER_BL*/: /* timer B low */
        via2ifr &= ~VIA_IM_T2;
        setirq(I_VIA2T2, 0);
        return ((via2tb - clk + via2tbu) & 0xff);

      case VIA_T2CH /*TIMER_BH*/: /* timer B high */
        return (((via2tb - clk + via2tbu) >> 8) & 0xff);

      case VIA_SR: /* Serial Port Shift Register */
        return (via2[addr]);

#if 0
      case VIA_T1CL /*TIMER_AL*/: /* timer A low */
	update_via2ta(0);
	return (via2ta & 0xff);

      case VIA_T1CH /*TIMER_AH*/: /* timer A high */
	update_via2ta(0);
	return ((via2ta >> 8) & 0xff);

      case VIA_T2CL /*TIMER_BL*/: /* timer B low */
	update_via2tb(0);
	return (via2tb & 0xff);

      case VIA_T2CH /*TIMER_BH*/: /* timer B high */
	update_via2tb(0);
	return ((via2tb >> 8) & 0xff);

      case VIA_SR: /* Serial Port Shift Register */
	return (via2[addr]);
#endif /* 0 */

	/* Interrupts */

      case VIA_IFR: /* Interrupt Flag Register */
	{
	    BYTE    t = via2ifr;
	    if (via2ifr & via2ier /*[VIA_IER]*/)
		t |= 0x80;
	    return (t);
	}

      case VIA_IER: /* 6522 Interrupt Control Register */
	    return (via2ier /*[VIA_IER]*/ | 0x80);

    }  /* switch */

    return (via2[addr]);
}


/* ------------------------------------------------------------------------- */

int    int_via2t1(void)
{
    if (debugflg)
	printf("via2 timer A interrupt\n");

    if (!(via2[VIA_ACR] & 0x40)) { /* one-shot mode */
#if defined (VIA2_TIMER_DEBUG)
        printf ("Via#2 Timer A interrupt -- one-shot mode: next int won't happen\n");
#endif
	via2ta = 0;
	via2tau = clk;
	unsetalarm(I_VIA2T1);		/*int_clk[I_VIA2T1] = 0;*/
    }
    else {		/* continuous mode */
        /* load counter with latch value */
        via2[VIA_T1CL] = via2[VIA_T1LL];
        via2[VIA_T1CH] = via2[VIA_T1LH];
	update_via2ta(1);
/*	int_clk[I_VIA2T1] = via2tau + via2ta;*/
    }

    via2ifr |= VIA_IM_T1;
    if(via2ier /*[VIA_IER]*/ & VIA_IM_T1 ) setirq(I_VIA2T1, 1);

    return 0; /*(viaier & VIA_IM_T1) ? 1:0;*/

#if 0
    via2int |= 0x80 | 1;

    /* count timer A underflow pulses */
    if ((via2[15] & 0x61) == 0x41)
	if (--via2tb <= 0)
	    int_clk[I_VIA2T2] = clk;
    return 2;
#endif
}


/*
 * Timer B can run in 2 (4) modes
 * via2[f] & 0x60 == 0x00   count system 02 pulses
 * via2[f] & 0x60 == 0x40   count timer A underflows
 * via2[f] & 0x60 == 0x20 | 0x60 count CNT pulses => counter stops
 */


int    int_via2t2(void)
{
    if (debugflg)
	printf("via2 timer B interrupt\n");

    via2tb = 0;
    via2tbu = clk;
    unsetalarm(I_VIA2T2);       /*int_clk[I_VIA2T2] = 0;*/

    via2ifr |= VIA_IM_T2;
    if( via2ier & VIA_IM_T2 ) setirq(I_VIA2T2, 1);

    return 0;

#if 0
    if (!(via2[VIA_ACR] & 0x20)) { /* one-shot mode */
#if defined (VIA2_TIMER_DEBUG)
        printf ("Via#2 Timer B interrupt -- one-shot mode: next int won't happen\n");
#endif
	via2tb = 0;
	via2tbu = clk;
	int_clk[I_VIA2T2] = 0;
    }
    else {		/* continuous mode */
#if defined (VIA2_TIMER_DEBUG)
        printf ("Via#2 Timer B interrupt\n");
#endif	
	update_via2tb(1);
	if ((via2[15] & 0x61) == 0x01)
	    int_clk[I_VIA2T2] = via2tbu + via2tb;
    }
    via2int |= 2;
    return (via2int & via2[VIA_IER] & 2) ? 2:0;
#endif /* 0 */
}


/* ------------------------------------------------------------------------- */

static void update_via2ta(int force)
{
    if(force) {
       if(debugflg)
          printf("update via timer A : latch=%d, counter =%d, clk = %ld\n",
                via2[VIA_T1CL] + (via2[VIA_T1CH] << 8),
                via2[VIA_T1LL] + (via2[VIA_T1LH] << 8),
                clk);

      via2ta = via2[VIA_T1CL] + (via2[VIA_T1CH] << 8);
      via2tau = clk;
      setalarm(I_VIA2T1, via2ta);
    }

#if 0
    if (force)
	via2ta = via2[VIA_T1LL /*TIMER_AL*/] + (via2[VIA_T1LH /*TIMER_AH*/] << 8);
    else if ((via2[14] & 0x21) == 1)
	via2ta = (int)(clk - via2tau + via2ta) & 0xffff;
    via2tau = clk;
#endif
}


static void update_via2tb(int force)
{
    if(force) {
      via2tb = via2[VIA_T2CL] + (via2[VIA_T2CH] << 8);
      via2tbu = clk;
      setalarm(I_VIA2T2, via2tb);
    }
#if 0
    if (force) {
	via2tb = via2[VIA_T2LL /*TIMER_BL*/] + (via2[VIA_T2CH /*TIMER_BH*/] << 8);
    }
    else if ((via2[15] & 0x61) == 1) {
	/* count system 02 pulses */
	via2tb = (int)(clk - via2tbu + via2tb) & 0xffff;
    }
    via2tbu = clk;
#endif
}

