/*
 * $Id: vic.c,v 1.2 1996/07/29 23:24:29 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README files for copyright notice.
 *
 *
 * VIC video chip emulation.
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: vic.c,v $
 * Revision 1.2  1996/07/29 23:24:29  ettore
 * Some garbage removed.
 * Adapted for use with the new DoRaster.
 *
 * Revision 1.1  1996/04/01 05:48:56  ettore
 * Initial revision
 *
 */


#define VIC20
#define _VIC_C

#include "extern.h"
#include "config.h"
#include "vic.h"
#include "raster.c"
#include "vmachine.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

/* #define VIC_VMODE_DEBUG */
/* #define VIC_RASTER_DEBUG */

/* global */

BYTE vic[64];
BYTE colorRam[0x400];

BYTE *scrbase = colorRam;	/* pointer to start of screen */
BYTE *charbase = colorRam;	/* pointer to start of chargen */
BYTE *bitmap1_base = colorRam;	/* pointer to upper half of bitmap */
BYTE *bitmap2_base = colorRam;	/* pointer to lower half of bitmap */

int vbank = 0;
int rasterintline = 0;		/* line for raster IRQ */

extern Window borderwin;
extern unsigned long colors[];
extern GC gc;
extern int width, height;

VideoStruct *v_vic = NULL;

int yscroll = 0;
int xscroll = 0;


/*

   screen_init(): initializes the VIC memory settings

 */

void
screen_init (void)
{
  ADDRESS addr;
  BYTE *tmp1, *tmp2;
  int scraddr;
  int tmp4, tmp5;

  tmp1 = scrbase;
  scraddr = ((vic[0x02] & 0x80) << 1) | ((vic[0x05] & 0x70) << 6);
  scrbase = ram + scraddr;

  if (debugflg)
    printf ("scraddr:%x\n", scraddr);

  tmp2 = charbase;
  addr = 0x8000 + ((vic[0x05] & 0x7) << 10);
  bitmap1_base = ram + (addr & 0xe000);

  if (debugflg)
    printf ("charbase:%x\n", addr);

  if (vic[0x05] & 16)
    {				/* flag: Chargen in RAM */
      addr &= 0x1c00;
      charbase = ram + addr;
      bitmap2_base = ram + ((addr & 0xf000));
    }
  else
    {
      charbase = char_rom + (addr & 0x0800);
      bitmap2_base = char_rom;
    }

  tmp4 = yscroll;
  tmp5 = xscroll;
  yscroll = vic[0x00] & 0x7f;
  xscroll = vic[0x01];

  if (charbase != tmp2 || scrbase != tmp1)
    init_memmaps ();

  v_vic->screenmem = scrbase;
  v_vic->colormem = colorRam + (vic[0x2] & 0x80 ? 0x200 : 0x0);
  v_vic->chargenmem = charbase;
}


/*

   store_vic(): stores a value in a VIC register.

 */

void
store_vic (ADDRESS addr, BYTE value)
{
  addr &= 0xf;
  vic[addr] = value;
  if (debugflg)
    printf ("store vic[%x] %x\n", addr, (int) value);

  switch (addr)
    {
    case 0:			/* $9000  Screen X Location */
    case 1:			/* $9001  Screen Y Location */
      /* (not implemented) */
      return;

    case 2:			/* $9002  Columns Displayed */
      /* color_ram = (value & 0x80) ? 0x9600 : 0x9400; */
      if (v_vic->tcols != (value & 0x7f))
	{
	  int w = (v_vic->xpix + v_vic->xpos * 2) * PIXEL_XSIZE;
	  if ((v_vic->tcols = (value & 0x7f)) > 27)
	    v_vic->tcols = 27;
	  v_vic->xpix = v_vic->tcols * 8;
	  Vic_ArrangeWindow (w, v_vic->winheight);
	}
      screen_init ();
      break;
      
    case 3:			/* $9003  Rows Displayed, Character size  */
      /* (not implemented) */
      screen_init ();
      return;

    case 4:			/* $9004  Raster line count -- read only */
      return;

    case 5:			/* $9005  Video and char matrix base address */
      screen_init ();
      return;

    case 6:			/* $9006 */
    case 7:			/* $9007  Light Pen X,Y */
      return;

    case 8:			/* $9008 */
    case 9:			/* $9009  Paddle X,Y */
      return;

    case 10:			/* $900A  Bass Enable and Frequency */
    case 11:			/* $900B  Alto Enable and Frequency */
    case 12:			/* $900C  Soprano Enable and Frequency */
    case 13:			/* $900D  Noise Enable and Frequency */
      return;

    case 14:			/* $900E  Auxiliary Colour, Master Volume */
      return;

    case 15:			/* $900F  Screen and Border Colours, Reverse Video */
      v_vic->bordcol = value & 0x7;
      v_vic->bgcol = HI_NIBBLE (value);
      return;
    }
}


/*

   read_vic(): reads from one VIC register.

 */

BYTE
read_vic (ADDRESS addr)
{
  addr &= 0xf;
  switch (addr)
    {

#if 0
    case 3:			/* $D011 */
      return ((vic[addr] & 0x7f) | ((CURRENT_LINE (clk) & 0x100) >> 1));

    case 4:			/* $D012  Raster Line Count */
      return (CURRENT_LINE (clk) & 0xff);

    case 6:
    case 7:			/* Light Pen X,Y */
      return (0xff);
#endif

    default:
      return (vic[addr]);
    }
}


/*

   store_colorram(): stores a value in the VIC color RAM.

 */

void
store_colorram (ADDRESS addr, BYTE value)
{
  colorRam[addr & 0x3ff] = value;
}


/*

   read_colorram(): reads a value from the VIC color RAM.

 */

BYTE
read_colorram (ADDRESS addr)
{
  return colorRam[addr & 0x3ff];
}

void
int_raster (void)
{
  return;
}


/*

   Vic_Init(): Initializes the VIC emulation.

 */

Window
Vic_Init (Display * disp, int scr_num, char *win_name, char *icon_name, char *res_class,
	  char *res_name, int *w, int *h, char *icon_data[], Colormap * cmap_return,
	  int exact_speed, int asleep, int refrate, Atom * wm_delwin)
{
  v_vic = AllocVideo (SCR_COLS, SCR_LINES, BORD_WIDTH, BORD_HEIGHT, XPIX, YPIX, XSIZE, YSIZE,
		      VIC_NUMCOLORS, refrate, VIC_NUMSPRITES, exact_speed, win_name, asleep);
  if (!v_vic)
    {
      fprintf (stderr, "fatal error: can't initialize Vic emulation.\n");
      return -1;
    }
  if (OpenVideoWindow (disp, scr_num, v_vic, win_name, win_name, res_class, res_name, icon_data,
		       vic_cnames, cmap_return, wm_delwin))
    {
      fprintf (stderr, "fatal error: can't open window for Vic emulation.\n");
      return -1;
    }
  if (SetVideoModes (v_vic, 2,
  /* VIC_NORMAL_MODE */
		     NormalGetTextData, NormalGetColorData,
		     NormalGetBgDataFromRegister, HIRES_VMODE,
  /* VIC_II_MULTICOLOR_MODE */
		     NormalGetTextData, NormalGetColorData,
		     NormalGetBgDataFromRegister, HIRES_VMODE))
    {
      fprintf (stderr, "fatal error: can't initialize Vic emulation.\n");
      return -1;
    }
  v_vic->vmode = VIC_NORMAL_MODE;
  /* Vic_SetVModes(); */
  screen_init ();
  RefreshAll (v_vic);
  *w = v_vic->winwidth;
  *h = v_vic->winheight;
  return (v_vic->win);
}


/*

   Vic_ArrangeWindow(): centers the image at the center of the window.

 */

void
Vic_ArrangeWindow (int w, int h)
{
  ArrangeVideoImage (v_vic, w, h);
  RefreshAll (v_vic);
}

void
Vic_Free (void)
{
  FreeVideo (v_vic);
}

void
Vic_Pause (void)
{
  PauseVideo (v_vic);
}

void
Vic_Restart (void)
{
  RestartVideo (v_vic);
}

void
DoRaster (void)
{
  Raster (v_vic);
}
