/* 
 * $Id: vicii.h,v 1.2 1996/07/29 23:30:23 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *
 * Header file for VIC-II emulation.
 *
 * Written by
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 *
 * $Log: vicii.h,v $
 * Revision 1.2  1996/07/29 23:30:23  ettore
 * *** empty log message ***
 *
 * Revision 1.1  1996/04/01 05:49:46  ettore
 * Initial revision
 *
 */


#ifndef _VICII_H
#define _VICII_H

#define BORD_HEIGHT  50
#define BORD_WIDTH   24
#define SCR_COLS    411		/* full overscan x size */

#define VIC_II_NUM_SPRITES  8
#define NUM_SPRITES	    VIC_II_NUM_SPRITES

/* the following #defines are needed to provide inline function definition */

#define NEEDS_NormalGetTextData
#define NEEDS_NormalGetBitmapData
#define NEEDS_NormalGetColorData
#define NEEDS_NormalGetBgDataFromRegister
#define NEEDS_GetData
#define NEEDS_GetDataHiLo

#include "raster.h"


/* color definitions -- someone check these out please :) */

#define VIC_II_NUMCOLORS 16

#if defined (_VICII_C)
char *vic_ii_cnames[] =
{
  "#000000",			/* Black */
  "#ffffff",			/* White */
  "#a00000",			/* Red */
  "#00ffff",			/* Cyan */
  "#ff00ff",			/* Purple */
  "#00a000",			/* Green */
  "#0000a0",			/* Blue */
  "#ffff00",			/* Yellow */
  "#ffa000",			/* Orange */
  "#a05000",			/* Brown */
  "#ff8080",			/* Light Red */
  "#404040",			/* Dark Gray */
  "#808080",			/* Medium Gray */
  "#00ff00",			/* Light Green */
  "#00a0ff",			/* Light Blue */
  "#e0e0e0"			/* Light Gray */
};
#endif /* defined (_VICII_C) */


/* video mode definitions */

#define  VIC_II_NORMAL_TEXT_MODE         0
#define  VIC_II_MULTICOLOR_TEXT_MODE     1
#define  VIC_II_EXTENDED_TEXT_MODE       2
#define  VIC_II_HIRES_BITMAP_MODE        3
#define  VIC_II_MULTICOLOR_BITMAP_MODE   4
#define  VIC_II_BLACK_MODE		 5 /* Extended Color and Multicolor bits both set to 1 */

#define  VIC_FETCH	13
#define  VIC_FETCH_ADD  2

/* Prototypes... */

void VicII_SetVMode (void);
Window VicII_Init (Display * disp, int scr_num, char *win_name, char *icon_name, char *res_class,
		   char *res_name, int *width, int *height, char *icon_data[],
		   Colormap * cmap_return, int exact_speed, int asleep, int refrate,
		   Atom * wm_delwin);
void VicII_ArrangeWindow (int width, int height);
void VicII_Free (void);
void VicII_Restart (void);
void VicII_Pause (void);
int VicII_GetHiresBitmapColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe,int no_check);
int VicII_GetHiresBitmapBgData (VideoStruct * v, LineStruct * l, int *xs, int *xe);
int VicII_GetMCTextColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int VicII_GetMCBitmapColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int VicII_GetExtTextData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int VicII_GetExtTextColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int VicII_GetExtTextBgData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int VicII_GetBlackData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int VicII_GetBlackColorData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
int VicII_GetBlackBgData (VideoStruct * v, LineStruct * l, int *xs, int *xe, int no_check);
void VicII_DrawMCTextLine (PIXEL * p, LineStruct * l, int xs, int xe, PIXEL * colors);
void VicII_DrawExtTextLine (PIXEL * p, LineStruct * l, int xs, int xe);

void VicII_SetStatus (int awake);
void DoRaster (void);

#endif /* _VICII_H */
