/*
 * $Id: x64.c,v 1.12 1996/07/29 23:36:31 ettore Exp ettore $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 *
 * Written by
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Hannu Helminen   (dm@stekt.oulu.fi)
 *   Jouko Valta      (jopi@stekt.oulu.fi)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *   Andre Fachat     (a.fachat@physik.tu-chemnitz.de)
 *
 * Patch by
 *   Richard Hable    (Richard.Hable@jk.uni-linz.ac.at) /RH/
 *
 *
 * $Log: x64.c,v $
 * Revision 1.12  1996/07/29 23:36:31  ettore
 * Some garbage removed.
 * Handles WM_DelWin.
 * Searches a suitable visual before opening windows.
 * Handles the experimental popup menu.
 *
 * Revision 1.11  1996/04/01  09:01:41  jopi
 * C128, VIC-20 and PET (business and graphics) keyboards
 * Enhanced keyboard reading.
 *
 * Revision 1.10  1995/11/07  16:51:00  jopi
 * VIC20, PET
 * Combined different video versions
 *
 * Revision 1.9  1995/04/01  07:54:09  jopi
 * X64 0.3 PL 0
 * More options for OpenWin routine.
 *
 * Revision 1.8  1994/12/12  16:59:44  jopi
 * New Vic emulation
 *
 * Revision 1.7  1994/06/16  17:19:26  jopi
 * Code reorganized and cleaned up
 *
 * Revision 1.6  1994/02/18  16:06:03  jopi
 * *** empty log message ***
 *
 * Revision 1.5  1994/01/26  16:21:38  jopi
 * X64 version 0.2 PL 1
 * keyboard input latched
 *
 * Revision 1.4  1993/11/10  01:55:34  jopi
 * *** empty log message ***
 *
 * Revision 1.3  93/06/21  13:39:52  jopi
 *  X64 version 0.2 PL 0
 *
 */

#define _X64_C

/* Xlib include files */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <X11/cursorfont.h>

#include <X11/keysym.h>
#if defined(__hpux) && !defined(hpXK_BackTab)
#include <X11/HPkeysym.h>
#endif
/*
#include <X11/xpm.h>
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>
#include <sys/ioctl.h>

#include "cpu.h"	/* for PC */
#include "vmachine.h"	/* VIC definitions */
#include "extern.h"
#include "x64.h"
#include "popup.h"
#include "interrupt.h"  /* restore() */


#ifdef B0
#undef B0
#endif

/* #define PALETTE  */ /* tmp */

#define BORDER_WIDTH 4
#define BITMAPDEPTH 1

#define MAXDOWN 16


/* Define Keyboard Buffer control */
#define ACK_UNREAD	1
#define ACK_DOWN	2
#define ACK_MODIFIER	4

#ifdef AUTOREPEAT
#define ACK_PRESSED	(ACK_UNREAD)
#else
#define ACK_PRESSED	(ACK_UNREAD | ACK_DOWN)
#endif

/*
 * Local functions
 */

Window OpenWindow ( int, int, int, int, int, char *, char *, int,
  int, int, int );

/*
 * Display and screen are used as arguments to nearly every Xlib routine
 * so it simplifies routine calls to declare them global.
 * In the additional source files these variables are declared ``extern''.
 */

Display *display;
int      screen;
static Window  rootw;
Atom     WM_DelWin;

Window   borderwin, VDCborderwin;
GC       gc;

Visual  *visual;
int      depth = 8;		/* we can only support this for now */

char    *display_name = NULL;	/* X Server to connect to */

int      width, height;		/* window size */
#if defined( C128 ) && defined( USE_VDC )
int      vdcwidth, vdcheight;	/* VDC window size */
#endif

int     redraw;


/* ------------------------------------------------------------------------- */

/* keyboard */

#ifdef VIC20
#define ROW0	7
#define ROW7	0
#define COL3	128
#define COL7	8
#else
#define ROW0	0
#define ROW7	7
#define COL3	8
#define COL7	128
#endif


BYTE    joy[3] = { 0, 0, 0 };

typedef struct {
    KeySym  key;
    int     ack;
    int     ptime;	/* Time key first pressed */
    int     rtime;	/* Time of last KeyPress or KeyRelease */
    CLOCK   cycles;	/* Time it was passed to the CIA Matrix */
}  kbq;


static int     keys_down = 0;
static int     kbd_changed = 0;

#ifdef LATCHING
static kbq keyqueue[MAXDOWN];
#endif

/*
#define LATCHING
*/

#ifndef LATCHING
int    keypressed = -1;
#endif


/*
 * Keyboard Matrix
 */


/* VIC/C64/C128 keyboard */

#ifndef PET
static keyconv keyconvmap[] = {
    {XK_Delete,	ROW0, 1, 0},		/* del */
    {XK_Return,	ROW0, 2, 0},
    {XK_Right,	ROW0, 4, 0},
    {XK_F7,	ROW0, COL3, 0},
    {XK_F1,	ROW0, 16, 0},
    {XK_F3,	ROW0, 32, 0},
    {XK_F5,	ROW0, 64, 0},
    {XK_Down,	ROW0, COL7, 0},

    {XK_3,	1, 1, 0},
    {XK_w,	1, 2, 0},
    {XK_a,	1, 4, 0},
    {XK_4,	1, COL3, 0},
    {XK_z,	1, 16, 0},
    {XK_s,	1, 32, 0},
    {XK_e,	1, 64, 0},
    {XK_Shift_L,	1, COL7, 0},

    {XK_5,	2, 1, 0},
    {XK_r,	2, 2, 0},
    {XK_d,	2, 4, 0},
    {XK_6,	2, COL3, 0},
    {XK_c,	2, 16, 0},
    {XK_f,	2, 32, 0},
    {XK_t,	2, 64, 0},
    {XK_x,	2, COL7, 0},

    {XK_7,	3, 1, 0},
    {XK_y,	3, 2, 0},
    {XK_g,	3, 4, 0},
    {XK_8,	3, COL3, 0},
    {XK_b,	3, 16, 0},
    {XK_h,	3, 32, 0},
    {XK_u,	3, 64, 0},
    {XK_v, 	3, COL7, 0},

    {XK_9,	4, 1, 0},
    {XK_i,	4, 2, 0},
    {XK_j,	4, 4, 0},
    {XK_0,	4, COL3, 0},
    {XK_m,	4, 16, 0},
    {XK_k,	4, 32, 0},
    {XK_o,	4, 64, 0},
    {XK_n,	4, COL7, 0},

    {XK_plus,	5, 1, 528},
    {XK_p,	5, 2, 0},
    {XK_l,	5, 4, 0},
    {XK_minus,	5, COL3, 0},
    {XK_period,	5, 16, 0},
    {XK_colon,	5, 32, 528},
    {XK_at,	5, 64, 528},		/* shifted @ gives 'grave' */
    {XK_comma,	5, COL7, 0},

    {XK_sterling,	6, 1, 0},	/* Sterling Pound sign */
    {XK_bar,		6, 1, 0},	/* Sterling Pound sign */
    {XK_asterisk,	6, 2, 528},
    {XK_semicolon,	6, 4, 528},
    {XK_Home,		6, COL3, 0},

#if 0
    {XK_Shift_R,	6, 16, 16},
#else
    {XK_Shift_R,	0, 0, 16},	/* invisible shift ;) */
#endif

    {XK_equal,		6, 32, 528},
#ifdef US_KBD
    { XK_asciicircum,	6, 64,0 },	/* Up arrow (US kbd) */
#else
    {XK_asciitilde,	6, 64, 0},	/* Up arrow (scandinavian kbd) */
#endif
    {XK_slash,		6, COL7, 528},

    {XK_1,		ROW7, 1, 0},
    {XK_underscore,	ROW7, 2, 0},	/* Left arrow */
    {XK_Control_L,	ROW7, 4, 0},	/* CNTRL */
    {XK_2,		ROW7, COL3, 0},
    {XK_space,		ROW7, 16, 0},
    {XK_Alt_L,		ROW7, 32, 0},	/* CBM */
    {XK_q,		ROW7, 64, 0},
    {XK_Escape,		ROW7, COL7, 0},	/* STOP */

    {XK_BackSpace, ROW0, 1, 0},	/* Del */

    /*
     * Some keys for DECstations
     */
    {XK_KP_F1,	ROW0, 16, 0},	/* F1 */
    {XK_KP_F2,	ROW0, 32, 0},	/* F3 */
    {XK_KP_F3,	ROW0, 64, 0},	/* F5 */
    {XK_KP_F4,	ROW0, 4, 0},	/* F7 */
    {XK_Insert,	ROW0, 1, 16},	/* Inst */
    {XK_Find,	6, COL3, 0},	/* Home */

#ifdef DECMAP
    {DXK_Remove,ROW0, 1, 0},	/* del (DECkeysym.h) */
    {XK_Help,	ROW7, COL7, 0},	/* because no real ESC-key on the keyboard /RH/ */
#endif

    /*
     * C128 Keyboard Extension
     * C128 Special keys, rows K1, K2 and K3 from the VIC-IIe
     *
     * Kx d02f	dc01:
     * 08  fe	KP_1	KP_7	KP_4	KP_2	Tab	KP_5	KP_8	Help
     * 09  fd	KP_3	KP_9	KP_6	KP_Entr	LineFd KP_Minus	KP_Plus	Esc 
     * 10  fb  NoScroll	Right	Left	Down	Up	KP_Dot	KP_0	Alt
     *
     */

#ifdef C128
    {XK_KP_1,		 8, 128, 0},	/* Numpad 1 */
    {XK_KP_2,		 8,  16, 0},	/* Numpad 2 */
    {XK_KP_3,		 9, 128, 0},	/* Numpad 3 */
    {XK_KP_4,		 8,  32, 0},	/* Numpad 4 */
    {XK_KP_5,		 8,   4, 0},	/* Numpad 5 */
    {XK_KP_6,		 9,  32, 0},	/* Numpad 6 */
    {XK_KP_7,		 8,  64, 0},	/* Numpad 7 */
    {XK_KP_8,		 8,   2, 0},	/* Numpad 8 */
    {XK_KP_9,		 9,  64, 0},	/* Numpad 9 */
    {XK_KP_0,		10,   2, 0},	/* Numpad 0 */
    {XK_KP_Subtract,	 9,   4, 0},	/* Numpad - */
    {XK_KP_Add,		 0,   2, 0},	/* Numpad + */
    {XK_KP_Separator,	10,   4, 0},	/* Numpad . */

#if 0
    {XK_Help,		 8,   1, 0},	/* Help */
    {XK_Tab,		 8,   8, 0},	/* Tab */

    {XK_Escape,		 9,   1, 0},	/* Esc */
    {XK_Linefeed,	 9,   8, 0},	/* Linefeed */
    {XK_KP_Enter,	 9,  16, 0},	/* Numpad Enter */
#endif
#if 0
    {XK_Alt_L,		10,   1, 0},	/* Alt */
    {XK_Menu,		10, 128, 0},	/* NoScroll */

    {XK_Up,		10,   8, 0},	/* Separate Up */
    {XK_Down,		10,  16, 0},	/* Separate Down */
    {XK_Left,		10,  32, 0},	/* Separate Left */
    {XK_Right,		10,  64, 0},	/* Separate Right */
#endif
#endif  /* C128 */


    /*
     * Shifted C64/128 keys
     */

    {XK_Linefeed, ROW0, 2, 16},
    {XK_Left,	ROW0, 4, 16},
    {XK_F8,	ROW0, COL3, 16},
    {XK_F2,	ROW0, 16, 16},
    {XK_F4,	ROW0, 32, 16},
    {XK_F6,	ROW0, 64, 16},
    {XK_Up,	ROW0, COL7, 16},

    {XK_numbersign, 1, 1, 16},
    {XK_W,	1, 2, 16},
    {XK_A,	1, 4, 16},
    {XK_dollar,	1, COL3, 16},
    {XK_Z,	1, 16, 16},
    {XK_S,	1, 32, 16},
    {XK_E,	1, 64, 16},
    /* { XK_Shift_L,	1, COL7, 0 }, */

    {XK_percent, 2, 1, 16},
    {XK_R,	2, 2, 16},
    {XK_D,	2, 4, 16},
    {XK_ampersand, 2, COL3, 16},
    {XK_C,	2, 16, 16},
    {XK_F,	2, 32, 16},
    {XK_T,	2, 64, 16},
    {XK_X,	2, COL7, 16},

    {XK_quoteright, 3, 1, 16},		/* apostrophe */
    {XK_Y,	3, 2, 16},
    {XK_G,	3, 4, 16},
    {XK_parenleft, 3, COL3, 16},
    {XK_B,	3, 16, 16},
    {XK_H,	3, 32, 16},
    {XK_U,	3, 64, 16},
    {XK_V,	3, COL7, 16},

    {XK_parenright, 4, 1, 16},
    {XK_I,	4, 2, 16},
    {XK_J,	4, 4, 16},
    /* { XK_0,	4, COL3, 16 }, */
    {XK_M,	4, 16, 16},
    {XK_K,	4, 32, 16},
    {XK_O,	4, 64, 16},
    {XK_N,	4, COL7, 16},

    /* { XK_plus,	5,1,16 }, */
    {XK_P,		5, 2, 16},
    {XK_L,		5, 4, 16},
    /* { XK_minus,	5,COL3,16 }, */
    {XK_greater,	5, 16, 16},
    {XK_bracketleft,	5, 32, 16},
    {XK_grave,		5, 64, 16},	/* shifted @ */
    {XK_less,		5, COL7, 16},

    /* { XK_sterling,	6, 1, 16 }, */
    {XK_backslash,	6, 1, 16},
    /* { XK_asterisk,	6, 2, 16 }, */
    {XK_bracketright,	6, 4, 16},
    /* {XK_Home,	6, COL3, 16 },
       {XK_Shift_R,	6, 16, 16 },
       {XK_equal,	6, 32, 16 }, */
#ifdef US_KBD
    { XK_asciitilde,	6, 64, 0 },	/* shift up arrow (US kbd) */
#else
    {XK_asciicircum,	6, 64, 0},	/* shift up arrow (scandinavian kbd) */
#endif
    {XK_question,	6, COL7, 16},

    {XK_exclam,		ROW7, 1, 16},
    /* { XK_underscore,	ROW7, 2, 16 },*/	/* right arrow */
    /* { XK_Control_L,	ROW7, 4, 16 },*/	/* CNTRL */
    {XK_quotedbl,	ROW7, COL3, 16},
    /* { XK_space,	ROW7, 16, 16 },
       { XK_Alt_L,	ROW7, 32, 16 },*/	/* CBM */
    {XK_Q,		ROW7, 64, 16},
    /* { XK_Escape,	ROW7, COL7, 16 },*/	/* STOP */
    {0, 0, 0, 0}
};


/* End of VIC/C64/C128 keyboard */

/* ------------------------------------------------------------------------- */

#else  /* PET keyboard(s) */

/* For the PET keyboards, the shift flag has a slightly different meaning:
	0	no shift involved
	1	normal, but shifted key
	2	left shift key
	4	right shift key
	8	allow shift key to be combined with the normal key
		(currently used for TAB, BackSpace,...)

   An additional kbd_shiftlfag is supported to save the state of the shift
   keys - to let the shift state not be overridden by some key combinations,
   if the shift keys are explicitely pressed

   the variables kbd_*shiftrow and kbd_*shiftbit contain the position of 
   the left and right shift keys 
*/

static short kbd_shiftflag=0;

static short kbd_lshiftrow = 6;
static short kbd_lshiftbit = 1;
static short kbd_rshiftrow = 6;
static short kbd_rshiftbit =64;

#if 0
static keyconv *keyconvmap = pet80map;
static int CONV_KEYS = sizeof(pet80map)/sizeof(keyconv);
#endif

/*#if PET8*/

/* PET 8032 Business Keyboard */

/*
static short kbd_lshiftrow = 6;
static short kbd_lshiftbit = 1;
static short kbd_rshiftrow = 6;
static short kbd_rshiftbit =64;
*/

static keyconv pet80map[] = {

    {XK_0,	1, 8, 0},
    {XK_1,	1, 1, 0},
    {XK_2,	0, 1, 0},
    {XK_3,	9, 2, 0},
    {XK_4,	1, 2, 0},
    {XK_5,	0, 2, 0},
    {XK_6,	9, 4, 0},
    {XK_7,	1, 4, 0},
    {XK_8,	0, 4, 0},
    {XK_9,	9, 8, 0},

    {XK_Control_L, 9, 16, 8},		/* left Ctrl maps to run/stop */

    {XK_Home,	8, 16,	8},
    {XK_Right,	0, 32,	8},
    {XK_Down,	5, 16,	8},
    {XK_Delete,	4, 128,	0},		/* del */
    {XK_BackSpace, 4, 128, 8},		/* inst/del */
    {XK_Shift_L,6,  1,	2},
    {XK_Shift_R,6,  64,	4},
    {XK_Return,	3, 16, 0},
    {XK_Escape,	2,  1,	0},		/* Escape */

    {XK_Tab,	8,  1,  8},		/* Tab -> rev/off */
#ifdef hpXK_BackTab
    {hpXK_BackTab,	8,  1,  1},		/* Tab -> rev/off */
#endif
    {XK_KP_Tab, 8, 1, 8},
#ifdef hpXK_KP_BackTab
    {hpXK_KP_BackTab, 8, 1, 1},
#endif

    {XK_KP_0,	7, 16,	0},	/* Numpad 0 */
    {XK_KP_1,	8, 128,	0},	/* Numpad 1 */
    {XK_KP_2,	7, 128,	0},	/* Numpad 2 */
    {XK_KP_3,	6, 128,	0},	/* Numpad 3 */
    {XK_KP_4,	5, 128,	0},	/* Numpad 4 */
    {XK_KP_5,	2, 128,	0},	/* Numpad 5 */
    {XK_KP_6,	3, 128,	0},	/* Numpad 6 */
    {XK_KP_7,	1, 16,	0},	/* Numpad 7 */
    {XK_KP_8,	0, 16,	0},	/* Numpad 8 */
    {XK_KP_9,	1, 128,	0},	/* Numpad 9 */
    {XK_KP_Separator, 7, 8, 0},
    {XK_KP_Multiply, 9, 32, 1},
    {XK_KP_Divide, 8, 64, 0},
    {XK_KP_Add, 2, 64, 1},
    {XK_KP_Subtract, 0, 8, 0},
    {XK_KP_Enter, 3, 16, 0},
    {XK_KP_Decimal, 6, 16, 0},	/* Numpad . */ 

    {XK_period,	6, 16, 0},
    {XK_minus,	0, 8,	0},
    {XK_space,	8, 4,	0},
    {XK_colon,	9, 32,	0},
    {XK_underscore, 9, 1, 0},		/* Left arrow */
    {XK_comma,	7, 8,   0},

    {XK_slash,	8, 64,	0},
    {XK_semicolon,	2, 64,	0},

    {XK_at,	3, 64,	0},		/* shifted @ gives 'grave' */
    {XK_a,	3,  1,	0},
    {XK_b,	6,  4,	0},
    {XK_c,	6,  2,	0},
    {XK_d,	3,  2,	0},
    {XK_e,	5,  2,	0},
    {XK_f,	2,  4,	0},
    {XK_g,	3,  4,	0},
    {XK_h,	2,  8,	0},
    {XK_i,	4, 32,	0},
    {XK_j,	3,  8,	0},
    {XK_k,	2, 32,	0},
    {XK_l,	3, 32,	0},
    {XK_m,	8,  8,	0}, /* *** */
    {XK_n,	7,  4,	0},
    {XK_o,	5, 32,	0},
    {XK_p,	4, 64,	0},
    {XK_q,	5,  1,	0},
    {XK_r,	4,  4,	0},
    {XK_s,	2,  2,	0},
    {XK_t,	5,  4,	0},
    {XK_u,	5,  8,	0},
    {XK_v,	7,  2,	0},
    {XK_w,	4,  2,	0},
    {XK_x,	8,  2,	0}, /* *** */
    {XK_y,	4,  8,	0},
    {XK_z,	7,  1,	0},

    {XK_asciicircum,	1, 32, 0},	/* Up arrow (US kbd) */
    {XK_asciitilde,	1, 32, 0},	/* Up arrow (scandinavian kbd) */

    {XK_bracketright,	2, 32, 0},
    {XK_bracketleft,	5, 64, 0},

    /*
     * Shifted PET keys
     */

    {XK_Insert,	4, 128, 1},
    {XK_Left,	0, 32, 1}, 	/*4, 16},*/
    {XK_Up,	5, 16, 1}, 	/*0, 128, 16},*/

    {XK_A,	3,  1,	1},
    {XK_B,	6,  4,	1},
    {XK_C,	6,  2,	1},
    {XK_D,	3,  2,	1},
    {XK_E,	5,  2,	1},
    {XK_F,	2,  4,	1},
    {XK_G,	3,  4,	1},
    {XK_H,	2,  8,	1},
    {XK_I,	4, 32,	1},
    {XK_J,	3,  8,	1},
    {XK_K,	2, 32,	1},
    {XK_L,	3, 32,	1},
    {XK_M,	8,  8,	1},
    {XK_N,	7,  4,	1},
    {XK_O,	5, 32,	1},
    {XK_P,	4, 64,	1},
    {XK_Q,	5,  1,	1},
    {XK_R,	4,  4,	1},
    {XK_S,	2,  2,	1},
    {XK_T,	5,  4,	1},
    {XK_U,	5,  8,	1},
    {XK_V, 	7,  2,	1},
    {XK_W,	4,  2,	1},
    {XK_X,	8,  2,	1},
    {XK_Y,	4,  8,	1},
    {XK_Z,	7,  1,	1},

    {XK_exclam,		1, 1, 1},
    {XK_quotedbl,	0, 1, 1},
    {XK_numbersign,	9, 2, 1},
    {XK_dollar,		1, 2, 1},
    {XK_percent,	0, 2, 1},
    {XK_ampersand,	9, 4, 1},
    {XK_quoteright,	1, 4, 1},	/* apostrophe = shift 7 */
    {XK_parenleft,	0, 4, 1},
    {XK_parenright,	9, 8, 1},
    {XK_asterisk,	9, 32, 1},
    {XK_plus,		2, 64, 1},

    {XK_less,		7, 8, 1},
    {XK_equal,		0, 8, 1},
    {XK_greater,	6, 8, 1},
    {XK_question,	8, 64, 1},

    {0, 0, 0, 0}
};


/* ------------------------------------------------------------------------- */

/*#else*/  /* PET 3032/4032 Graphics Keyboard */

/*
static short kbd_lshiftrow = 8;
static short kbd_lshiftbit = 1;
static short kbd_rshiftrow = 8;
static short kbd_rshiftbit =32;
*/

keyconv pet40map[] = {
    {XK_Return,		6, 32, 0},
    {XK_Down,		1, 64, 8},
    {XK_Right,		0, 128, 8},
    {XK_Home,		0, 64, 8},
    {XK_BackSpace,	1, 128, 8},
    {XK_Delete,		1, 128, 0},

    {XK_Tab,		9,  1,  8},	/* Tab -> rev/off */
#ifdef hpXK_BackTab
    {hpXK_BackTab,	9,  1,  1},	/* Tab -> rev/off */
#endif
    {XK_KP_Tab, 9, 1, 8},
#ifdef hpXK_KP_BackTab
    {hpXK_KP_BackTab,	9,  1,  1},	/* Tab -> rev/off */
#endif

    {XK_Escape,		9, 16, 8},	/* Escape -> Run/Stop */
    {XK_Find,		0, 64, 0},	/* Home */

    {XK_Shift_L,	8, 1, 2},
    {XK_Shift_R,	8, 32, 4},

#ifdef DECMAP
    {DXK_Remove,	1, 128, 0},	/* del (DECkeysym.h) */
    {XK_Help,		9, 16, 8},	/* because no real ESC-key on the 
					   keyboard /RH/ */
#endif  /* DECMAP */

    {XK_a,	4,  1, 0},
    {XK_b,	6,  4, 0},
    {XK_c, 	6,  2, 0},
    {XK_d,	4,  2, 0},
    {XK_e,	2,  2, 0},
    {XK_f,	5,  2, 0},
    {XK_g,	4,  4, 0},
    {XK_h,	5,  4, 0},
    {XK_i,	3,  8, 0},
    {XK_j,	4,  8, 0},
    {XK_k,	5,  8, 0},
    {XK_l,	4, 16, 0},
    {XK_m,	6,  8, 0},
    {XK_n,	7,  4, 0},	
    {XK_o,	2, 16, 0},
    {XK_p,	3, 16, 0},
    {XK_q,	2,  1, 0},
    {XK_r,	3,  2, 0},
    {XK_s,	5,  1, 0},
    {XK_t,	2,  4, 0},
    {XK_u,	2,  8, 0},
    {XK_v,	7,  2, 0},
    {XK_w,	3,  1, 0},
    {XK_x,	7,  1, 0},
    {XK_y,	3,  4, 0},
    {XK_z,	6,  1, 0},

    {XK_KP_0,	8, 64, 0},
    {XK_KP_1,	6, 64, 0},
    {XK_KP_2,	7, 64, 0},
    {XK_KP_3,	6, 128, 0},
    {XK_KP_4,	4, 64, 0},
    {XK_KP_5,	5, 64, 0},
    {XK_KP_6,	4, 128, 0},
    {XK_KP_7,	2, 64, 0},
    {XK_KP_8,	3, 64, 0},
    {XK_KP_9,	2, 128, 0},

    {XK_KP_Separator, 7, 8, 0},
    {XK_KP_Multiply, 5, 128, 0},
    {XK_KP_Divide, 3, 128, 0},
    {XK_KP_Add, 7, 128, 0},
    {XK_KP_Subtract, 8, 128, 0},
    {XK_KP_Enter, 6, 32, 0},
    {XK_KP_Decimal, 9, 64, 0},	/* Numpad . */ 

    {XK_0,	8, 64, 0},
    {XK_1,	6, 64, 0},
    {XK_2,	7, 64, 0},
    {XK_3,	6, 128, 0},
    {XK_4,	4, 64, 0},
    {XK_5,	5, 64, 0},
    {XK_6,	4, 128, 0},
    {XK_7,	2, 64, 0},
    {XK_8,	3, 64, 0},
    {XK_9,	2, 128, 0},

    {XK_slash,		3, 128, 0},
    {XK_bracketright,	1, 16, 0},
    {XK_bracketleft,	0, 16, 0},
    {XK_parenright,	1, 16, 0},
    {XK_parenleft,	0, 16, 0},
    {XK_question,	7, 16, 0},
    {XK_dollar,		1,  2, 0},
    {XK_asterisk,	5, 128, 0},
    {XK_colon,		5, 16, 0},
    {XK_comma,		7,  8, 0},
    {XK_quotedbl,	1,  1, 0},
    {XK_sterling,	1,  8, 0},
    {XK_backslash, 	1,  8, 0},
    {XK_bar,		1,  8, 0},
    {XK_numbersign,	0,  2, 0},
    {XK_exclam,		0,  1, 0},
    {XK_percent,	0,  4, 0},
    {XK_plus,		7, 128, 0},
    {XK_underscore,	0, 32, 0},
    {XK_ampersand,	0,  8, 0},
    {XK_semicolon,	6, 16, 0},
    {XK_minus,		8, 128, 0},
    {XK_equal,		9, 128, 0},
    {XK_space,		9, 4, 0},
    {XK_greater,	8, 16, 0},
    {XK_less,		9, 8, 0},
    {XK_period,		9, 64, 0},
    {XK_at,		8, 2, 0},
    {XK_asciicircum,	2, 32, 0},	/* up arrow (US kbd) */
    {XK_asciitilde,	2, 32, 0},	/* up arrow (scandinavian kbd) */

    /* shifted keys */
    {XK_Left, 		0, 128, 1},
    {XK_Up, 		1, 64, 1},
    {XK_Insert,		1, 128, 1},
    {XK_Clear,		0, 64, 1},

    {XK_grave, 		8, 2, 1},	/* shifted @ */

    {XK_W, 		3, 1, 1},
    {XK_A, 		4, 1, 1},
    {XK_Z, 		6, 1, 1},
    {XK_S, 		5, 1, 1},
    {XK_E, 		2, 2, 1},
    {XK_R, 		3, 2, 1},
    {XK_D, 		4, 2, 1},
    {XK_C, 		6, 2, 1},
    {XK_F, 		5, 2, 1},
    {XK_T, 		2, 4, 1},
    {XK_X, 		7, 1, 1},
    {XK_Y, 		3, 4, 1},
    {XK_G, 		4, 4, 1},
    {XK_B, 		6, 4, 1},
    {XK_H, 		5, 4, 1},
    {XK_U, 		2, 8, 1},
    {XK_V, 		7, 2, 1},
    {XK_I, 		3, 8, 1},
    {XK_J, 		4, 8, 1},
    {XK_M, 		6, 8, 1},
    {XK_K, 		5, 8, 1},
    {XK_O, 		2, 16, 1},
    {XK_N, 		7, 4, 1},
    {XK_P, 		3, 16, 1},
    {XK_L, 		4, 16, 1},
    {XK_Q, 		2, 1, 1},
 
    {0, 0, 0, 0}
};

static keyconv *keyconvmap = pet80map;
static int CONV_KEYS = sizeof(pet80map)/sizeof(keyconv);

/*#endif*/  /* PET_4032 */
#endif  /* PET */

#ifndef PET
#define CONV_KEYS (sizeof(keyconvmap)/sizeof(keyconv))
#endif


/* ------------------------------------------------------------------------- */
/* switching keyboards for different PETs */

#ifdef PET	

void set80key(void) {
    kbd_lshiftrow = 6;
    kbd_lshiftbit = 1;
    kbd_rshiftrow = 6;
    kbd_rshiftbit =64;

    keyconvmap = pet80map;
    CONV_KEYS = sizeof(pet80map)/sizeof(keyconv);
}

void set40key(void) {
    kbd_lshiftrow = 8;
    kbd_lshiftbit = 1;
    kbd_rshiftrow = 8;
    kbd_rshiftbit =32;

    keyconvmap = pet40map;
    CONV_KEYS = sizeof(pet40map)/sizeof(keyconv);
}

#endif

/* ------------------------------------------------------------------------- */

/* Colors */
/* SHOULD WE KEEP THIS?? */
unsigned long bgcolors[MAX_COLORS + SPECIAL_COLORS];

#ifdef PALETTE
/* colors for border, background & sprites */

XColor  exact_defs[MAX_COLORS];
GC      cgcs[MAX_COLORS];
#endif

unsigned long colors[MAX_COLORS];

#if 0
static char   *cnames[] = {
    "Black",   "White",    "Red",      "Cyan",
    "Purple",  "Green3",   "Blue",     "Yellow",
    "Orange",  "Brown",    "Pink",     "Dim Gray",
    "Gray",  "Pale Green", "Light Blue", "Light Gray"
};

static char   *bwnames[] = {
    "Black", "White",     "Dim Gray", "Light Gray",
    "Gray",  "Gray",      "Dim Gray", "Light Gray",
    "Gray",  "Dim Gray",  "Gray",     "Dim Gray",
    "Gray",  "Light Gray","Gray",     "Light Gray"
};

#define B0 { 0,0,0,0 }
#define B1 { 0,0,0,1 }
#define B2 { 1,0,0,1 }
#define B3 { 1,1,0,1 }
#define B4 { 1,1,1,1 }
#endif

#if 0
static int dither[][4] = {
    B0, B4, B1, B3,
    B2, B2, B1, B3,
    B2, B1, B2, B1,
    B2, B3, B2, B3
};
#endif

#if 0
static int  get_colors(void)
{
#ifdef PALETTE
    XGCValues values;
#endif
    XColor    exact_def, rgb_def;
    Colormap  def_cmap = DefaultColormap(display, screen);
    int       i;


    /* Allocate colors for characters on screen */

    if (DefaultDepth(display, screen) >= 4) {

#ifdef PALETTE
	/* Allocate read/write colors for backgrounds & sprites
	 * palette is easier to modify, but not portable.
	 */
	if (!XAllocColorCells(display, def_cmap, False, NULL, 0, bgcolors,
			      MAX_COLORS + SPECIAL_COLORS)) {
	    printf("Cannot allocate r/w colors. Please free the palette.\n");
	    exit(0);
	}
#endif

	/* allocate read only colors for characters in screen */
	for (i = 0; i < MAX_COLORS; i++) {
	    if (!XAllocNamedColor
		(display, def_cmap, cnames[i], &exact_def, &rgb_def)) {
		printf("Cannot allocate color %s\n", cnames[i]);
		break;
	    }

#ifdef PALETTE
	    values.foreground = exact_def.pixel;
	    cgcs[i] = XCreateGC(display, rootw, GCForeground, &values);
	    exact_defs[i] = exact_def;
	    exact_defs[i].flags = DoRed | DoGreen | DoBlue;
#endif
	    colors[i] = exact_def.pixel;
	}
	if (i == MAX_COLORS)
	    return (2);

	/* Could not allocate all colors, so free them and try again */

	if (i > 0)
	    XFreeColors(display, def_cmap, colors, i-1, 0);
    }


    /*
     * Next, take a new try with grayscale colors
     */

    for (i = 0; i < MAX_COLORS; i++) {
	if (!XAllocNamedColor
	    (display, def_cmap, bwnames[i], &exact_def, &rgb_def)) {
	    break;
	}
	colors[i] = exact_def.pixel;
    }
    if (i == MAX_COLORS)
	return (1);

    /* Could not allocate grayscale colors, so free them */

    if (i > 0)
	XFreeColors(display, def_cmap, colors, i-1, 0);


    /* All we have left is dithering... */

    printf ("No colors available.\n");
    return (0);
}
#endif

#if 0
/*
 * General purpose window opening routine.
 */

Window OpenWindow(place, win_x, win_y, w, h, win_name, icon_name, 
  maxwidth, maxheight, minwidth, minheight)
     int place;
     int win_x, win_y, w, h, minwidth, minheight;
     char *win_name, *icon_name;

{
    Window        win;
    XTextProperty windowName, iconName;
    XWMHints      wm_hints;
    XClassHint    class_hints;
    XSizeHints    size_hints;		/* Preferred sizes for window man */
    XSetWindowAttributes xswattr;
    Pixmap icon_pixmap;

    unsigned int  display_width, display_height;
    int  flags   = PSize | PMinSize | PMaxSize;
    int  scr;

    /* Display size is a member of display structure */
    scr = DefaultScreen(display);
    display_width = DisplayWidth(display, scr);
    display_height = DisplayHeight(display, scr);

    if (h < 0 || w < 0) {
	w =  XPIX + 60;
	h = YPIX + 60;
    }

    if (place == 0) {		/* center */
	flags |= PPosition;
	win_x = (display_width  - w) / 2;
	win_y = (display_height - h) / 2;
    }
    else if (place > 0)	{	/* locate as requested */
	flags |= PPosition;

	if (win_x < 0)
	    win_x = (display_width  - w);
	if (win_y < 0)
	    win_y = (display_height - h);
    }
    /* random placement if place < 0 */

/*
    xswattr.background_pixel = BlackPixel(display, scr);
    xswattr.border_pixel = BlackPixel(display, scr);
    xswattr.backing_store = WhenMapped;
*/

/*    xswattr.background_pixel = bgcolors[C_BORDER]; */
    xswattr.cursor        = XCreateFontCursor(display, XC_gumby);
    xswattr.backing_store = Always;
    xswattr.event_mask    = ExposureMask | KeyReleaseMask | KeyPressMask |
	ButtonPressMask | PointerMotionMask | StructureNotifyMask |
	FocusChangeMask | EnterWindowMask | LeaveWindowMask;

    win = XCreateWindow(display, RootWindow(display, scr), win_x, win_y,
	      w, h, 0,
	      CopyFromParent, CopyFromParent, CopyFromParent,
	      /* CWBackingStore | CWBackPixel | */ CWEventMask | CWCursor,
	      &xswattr);

    if (!win) {
	fprintf(stderr, "could not open window called %s", win_name);
	x_exit();
	exit(0);
    }

    size_hints.flags = flags;
    size_hints.width = w;
    size_hints.height = h;
    size_hints.min_width =  minwidth;
    size_hints.min_height = minheight;
    size_hints.max_width = maxwidth;
    size_hints.max_height = maxheight;
    size_hints.x = win_x;
    size_hints.y = win_y;

    XSetStandardProperties(display, win, win_name, win_name,
	None, NULL, 0, &size_hints);

    /* */
    if (XStringListToTextProperty(&win_name, 1, &windowName) == 0) {
	(void) fprintf
	    (stderr, "structure allocation for windowName failed.\n");
	exit(-1);
    }
    if (XStringListToTextProperty(&icon_name, 1, &iconName) == 0) {
	(void) fprintf
	    (stderr, "structure allocation for iconName failed.\n");
	exit(-1);
    }
    /* */

    class_hints.res_name = RES_NAME;
    class_hints.res_class = icon_name;
    XSetClassHint(display, win, &class_hints);

    /*XSelectInput(display, win, ExposureMask | KeyPressMask |
		 ButtonPressMask | StructureNotifyMask);*/

    /* */
    wm_hints.initial_state = NormalState;
    wm_hints.input = True;
    wm_hints.flags = StateHint | InputHint;

#if 0
    /* let's create a nice icon... */
    icon_pixmap = XCreatePixmapFromBitmapData (display, 
                 RootWindow (display, scr), 
                 dummy_bits, dummy_width, dummy_height, 
                 BlackPixel (display, scr), 
                 WhitePixel (display, scr),
                 DefaultDepth (display, scr));
    wm_hints.icon_pixmap = icon_pixmap;
#endif

#if 0
    icon_pixmap = XCreatePixmapFromBitmapData (display,
                  RootWindow (display, scr),
                  (char *)dummy_bits, dummy_width, dummy_height,
                  BlackPixel (display, scr),
                  WhitePixel (display, scr),
                  DefaultDepth (display, scr));
#endif
    (void) XCreatePixmapFromData (display, 
                  RootWindow (display, scr),
                  icon, &icon_pixmap, NULL, NULL);
    wm_hints.icon_pixmap = icon_pixmap;
    wm_hints.flags |= IconPixmapHint;

    XSetWMProperties(display, win, &windowName, &iconName, NULL, 0,
	&size_hints, &wm_hints, &class_hints);
    /* */


    /* Display window */
    XMapWindow(display, win);

    return (win);
}
#endif /* garbage */


/* ------------------------------------------------------------------------- */

char  *get_x_depth(void)
{
    static char buf[4];

    if (display != NULL)
	sprintf (buf, "%d", DefaultDepth(display, screen));

    return (buf);
}

  
/* ------------------------------------------------------------------------- */

Window win;

int    x_start(void)
{
    XGCValues values;
    Colormap cmap;
    XVisualInfo visinfo;
    
#ifdef XDEBUGGER
    debug_main ();

#else
    /* Connect to X server */
    if ((display = XOpenDisplay(display_name)) == NULL) {
	(void) fprintf (stderr, "cannot connect to X server %s\n",
	     XDisplayName(display_name));
	return(-1);
    }
#endif

    /* Get screen size from display structure macro */
    screen = DefaultScreen(display);
    rootw = RootWindow(display, screen);
#ifdef XCONTROL
    debug_main ();	/* needs colors */
#endif

    values.foreground = BlackPixel(display,screen);
    values.background = WhitePixel(display,screen);
    gc = XCreateGC(display,rootw,GCForeground | GCBackground, &values);
    
    WM_DelWin = XInternAtom(display, "WM_DELETE_WINDOW", False);

    /* Let's find a visual for our windows: the current implementation only accepts  */
    /* PseudoColor/8 bit depth. It should not be so difficult to find one, though... */
    if (!XMatchVisualInfo (display, screen, 8, PseudoColor, &visinfo)) {
      fprintf (stderr, "Can't find a suitable X Visual.\n");
      end64();
    }
    visual = visinfo.visual;

    INIT_VIDEO;
#if !defined(C128) || !defined(USE_VDC)
    if (borderwin <= 0)
#else
    if (borderwin <= 0 || VDCborderwin <= 0)
#endif
      end64 ();

    return (0);
}	       /* End x_start */


/* ------------------------------------------------------------------------- */

void    x_exit(void)
{
#ifdef PALETTE
    int     i;

    for (i = 0; i < MAX_COLORS; i++)
	XFreeGC(display, cgcs[i]);
#endif
    XAutoRepeatOn(display);
    FREE_VIDEO;
    XCloseDisplay(display);
}


/* ------------------------------------------------------------------------- */

/*
 * Check X events. If either Control or XDebugger Window is
 * available, this routine is replaced by them.
 */

#if (!defined(XDEBUGGER) && !defined(XCONTROL))

void    x_loop(void)
{
    XEvent  report;	/* Structure for event information */
    int     nEvents = XEventsQueued(display, QueuedAfterFlush);


/*    if (redraw) {
	vic_redraw();
	redraw = 0;
    } */
/*
#ifndef SETOWN
    flush_queue();
#endif
*/

    while (--nEvents >= 0) {
	XNextEvent(display, &report);

	
	/* Main Window Event */
	if (report.xany.window == borderwin ||
	    report.xany.window == VDCborderwin )
	{
	    handle_mainwin_event(&report);
	    continue;
	}
    }	/* End while */
}

#endif


/* ------------------------------------------------------------------------- */

#ifndef PET
static int  handle_joy_emu(XEvent *report)
{
    static char buffer[20];
    KeySym  key;
    XComposeStatus compose;
    int     count;
    int     show_keyarr();

    /* printf("debugwin event\n"); */
    switch (report -> type) {
      case KeyPress:
	count = XLookupString(&report -> xkey, buffer, 20, &key, &compose);
	switch (key) {
	  case XK_KP_8:
	    joy[joyport] |= 1;
	    break;
	  case XK_KP_2:
	    joy[joyport] |= 2;
	    break;
	  case XK_KP_4:
	    joy[joyport] |= 4;
	    break;
	  case XK_KP_6:
	    joy[joyport] |= 8;
	    break;
	  case XK_KP_0:
	    joy[joyport] |= 16;
	    break;
	  default:
	    return 0;
	}
	if (debugflg)
	    printf("press joy[%d] %x\n", joyport, joy[joyport]);
	return 1;

      case KeyRelease:
	count = XLookupString(&report -> xkey, buffer, 20, &key, &compose);
	switch (key) {
	  case XK_KP_8:
	    joy[joyport] &= ~1;
	    break;
	  case XK_KP_2:
	    joy[joyport] &= ~2;
	    break;
	  case XK_KP_4:
	    joy[joyport] &= ~4;
	    break;
	  case XK_KP_6:
	    joy[joyport] &= ~8;
	    break;
	  case XK_KP_0:
	    joy[joyport] &= ~16;
	    break;
	  default:
	    return 0;
	}
	if (debugflg)
	    printf("release joy[%d] %x\n", joyport, joy[joyport]);
	return 1;
    }
    return 0;
}
#endif

void    handle_mainwin_event(XEvent *report)
{
    switch (report -> type) {

      case Expose:
	if (debugflg)
	    printf("Expose %d\n", report -> xexpose.count);

	/* Unless this is the last contiquous expose, don't draw the window */
	if (report -> xexpose.count != 0)
	    break;
	ARRANGE_WINDOW;
	break;

      case ConfigureNotify:
	if (debugflg)
	    printf("Config notify\n");
	/* window has been resized; change width and height */
#if defined( C128 ) && defined ( USE_VDC )
	if ( report -> xconfigure.window == VDCborderwin ) {
	    vdcheight = report -> xconfigure.height;
	    vdcwidth = report -> xconfigure.width;
	} else 
#endif
	{
	    height = report -> xconfigure.height;
	    width = report -> xconfigure.width;
	}
	ARRANGE_WINDOW;
	break;

#ifdef LIGHTPEN
      case MotionNotify:
	lp_x = report -> xmotion.x;
	lp_y = report -> xmotion.y;
	break;
#endif

      case ButtonPress:
	switch (report -> xbutton.button) {
	  case Button1:
#if defined (POPUP_MENU)
	    if( report->xbutton.state & ShiftMask) {
#endif
	      XAutoRepeatOn(display);
	      /* -- drop to monitor; continue with x or C-d */
	      PAUSE_VIDEO;
	      XSync(display, 1);
	      mon(PC);
	      RESTART_VIDEO;
#ifndef AUTOREPEAT
	      XAutoRepeatOff(display);
#endif
#if defined (POPUP_MENU)
	    } else {
	      int x,y;
	      Window w;
	      PAUSE_VIDEO;
	      XTranslateCoordinates(display, 
				  report -> xbutton.window, RootWindow(display,screen), 
				    report -> xbutton.x, report -> xbutton.y,
				    &x, &y, &w);
	      open_action_popup(report->xbutton.window, report, x, y);
	      RESTART_VIDEO;
	    }
#endif
	    break;
	  case Button2:
#if defined (DEBUG)
	    debugflg = !debugflg;
#endif
	    break;
	  case Button3:
#if defined (POPUP_MENU)
	    if( report->xbutton.state & ShiftMask) {
#endif
	    /* open debugging window */
#if (defined(XDEBUGGER) || defined(XCONTROL))
	    toggle_cntl_win();
#endif
#if defined (POPUP_MENU)
	    } else {
	      int x,y;
	      Window w;
	      XTranslateCoordinates(display, 
				    report -> xbutton.window, RootWindow(display,screen), 
				    report -> xbutton.x, report -> xbutton.y,
				    &x, &y, &w);
	      PAUSE_VIDEO;
	      open_toggle_popup(report->xbutton.window, x, y);
	      RESTART_VIDEO;
	    }
#endif
	    break;
	}
	break;

      case KeyPress:
	{
	    static char buffer[20];
	    KeySym  key;
	    XComposeStatus compose;
#ifdef LATCHING
	    kbq     *kptr;
#endif
	    int     count, i;

#ifndef PET
	    if( handle_joy_emu( report ) )
		break;
#endif
	    count = XLookupString(&report -> xkey, buffer, 20, &key, &compose);

#ifdef KBDBUG
	    printf("\ncount %d Key %d %x Compose %x\n",
		   count, (int) *buffer, key, compose);
#endif
#ifndef PET
	    if (key == XK_Tab)	/* Restore key (Esc-Tab = break) */
		restore();
#endif
#ifndef LATCHING
	    for (i = 0; keyconvmap[i].sym; ++i) {
		if (key == keyconvmap[i].sym) {

		    keyarr[keyconvmap[i].row] |= keyconvmap[i].keybit;
#ifndef PET
		    if (keyconvmap[i].shift < 0x100)
			keyarr[6] |= (BYTE) keyconvmap[i].shift;
		    else {
			keyarr[6] &= ~0x10;
			keyarr[1] &= ~0x80;
		    }
#else
		    if (keyconvmap[i].shift) {
		      kbd_shiftflag |= keyconvmap[i].shift;
		    } else { 
		      kbd_shiftflag &= ~keyconvmap[i].shift;
		    }
		    /* the following code keeps up with shift keys */
		    /* if no other key is pressed, or a shifted key is
		       emulated and a shift key is pressed, or a combinable
		       key is pressed, then the shift keys are passed through.
		       if a non-shifted char is emulated, shift keys are 
		       turned off.
		       */
/*printf("pressed key %d, shift=%d, flag now %d\n", 
                               i, keyconvmap[i].shift, kbd_shiftflag);*/
		    if(!(kbd_shiftflag & 9)) {  /* non-shifted */
/*printf("remove shift keys\n"); */
		      keyarr[kbd_lshiftrow] &= ~kbd_lshiftbit;
		      keyarr[kbd_rshiftrow] &= ~kbd_rshiftbit;
		    } else
		      if(kbd_shiftflag & 6) {     /* shift keys are pressed */
			if( (keys_down==1) || (kbd_shiftflag & 9)) {
/*printf("map real shift keys\n"); */
			  if(kbd_shiftflag & 2) 
			    keyarr[kbd_lshiftrow]|=kbd_lshiftbit;
			  if(kbd_shiftflag & 4) 
			    keyarr[kbd_rshiftrow]|=kbd_rshiftbit;
			}
		      } else
			if(kbd_shiftflag & 1) {
/*printf("emulate shift key\n");*/
			  keyarr[kbd_lshiftrow] |= kbd_lshiftbit;
			}

#endif  /* PET */
		    keypressed = i;

#ifdef AUTOREPEAT

#ifdef KBDBUG
		    printf("%d\n", clk - keydelay[i]);
#endif

		    if (!keydelay[i])	/* set pressed keys */
			keys_down++;
		    keydelay[i] = clk + KEYDELAY;	/* c64 system clock */
#else
		    keys_down++;
#endif

		    if (debugflg)
			printf("KEY FOUND %ld\n", report -> xkey.time);
		    break;
		}
	    }

	    if (debugflg) {
		show_keyarr();
		printf("count %d Key %d %lx\n", count, (int) *buffer, key);
	    }

#else  /* LATCHING */
	    /*
	     * On a multitasking system input has to be buffered since
	     * we don't have direct contact to the keyboard.
	     * First search for last occurrence of current key to work out
	     * possible autorepeat.
	     */

	    for (i = keys_down, kptr = &keyqueue[i];
		 i-- && key != (--kptr)->key;);

	    if (key == kptr->key &&
		(report -> xkey.time - kptr->rtime < REPEAT_DELAY)) {
		kptr->rtime = report -> xkey.time;
		kptr->ack  = ACK_PRESSED;
	    }
	    else if (keys_down < MAXDOWN) {
		    if (kptr->ack)
			i = keys_down;		/* Add the key in the end */

		    kptr->key  = key;
		    kptr->ptime= kptr->rtime = report -> xkey.time;
		    kptr->ack  = ACK_PRESSED;
#ifdef AUTOREPEAT
		    /* mark modifiers (non-repeating) */ 
		    if (key == XK_Shift_L || key == XK_Shift_R ||
			key == XK_Control_L || key == XK_Return)
			kptr->ack |= (ACK_MODIFIER | ACK_DOWN);
#endif  /* repeat */

		    kbd_changed++;
		    keys_down++;
	    }

#ifdef KBDBUG
	    printf("KEY FOUND [%d/%d] count %d  Key %02x  - %ld\n",
		   i, keys_down, count, key & 0xffff, report -> xkey.time);

	    for (i = 0; i < MAXDOWN; i++)
		printf("%02x ", kptr->key & 0xffff);
	    printf("\n");
#endif
#endif  /* LATCHING */


	    /* This hack puts keystrings directly into char buffer.
	       Do not overfill the queue ... */
#ifdef KBD_HACK
#ifdef KBD_HACKALL
	      if( ( key == XK_Escape ) /*RH*/
		|| ( key == XK_Help )
		|| ( key == XK_Home )
		|| ( key == XK_Control_L )
		|| ( key == XK_Alt_L )
		|| ( key == XK_Delete )
		|| ( key == XK_Right )
	 	|| ( key == XK_Down )
		|| ( key == XK_Find )
		|| ( key == XK_Insert )
		|| ( (key >= XK_F1) && (key <= XK_F8) )  /* F1..F8 */
		|| ( key == XK_Left )
		|| ( key == XK_Up ) ) {;} else {
#endif
	     if (count > 0) {
		 if ( ((*buffer & 0xc0) == 0x40) && ((*buffer & 0x1f) < 0x1e) )
		     *buffer ^= 0x20;
#ifdef PET
		 ram[0x026f + ram[0x9e]++] = *buffer;
#else
		 ram[0x0277 + ram[0xc6]++] = *buffer;
#endif  /* PET */
	     }

#ifdef KBD_HACKALL
	 } /*RH*/
#endif
#endif  /* KBD_HACK */


#ifdef KBDBUG2
	printf("queue %d \n", keys_down);
	for (i = 0; i < MAXDOWN; i++)
	    printf("%02x%c  ", kptr->key, kptr->ack + '@');
	printf("\n");

	printf("%2d keys: %02X %02X %02X %02X  %02X %02X %02X %02X\n",
	       keys_down, keyarr[0], keyarr[1], keyarr[2], keyarr[3],
	       keyarr[4], keyarr[5], keyarr[6], keyarr[7]);
#endif

	    break;
	}

#ifndef LATCHING
#ifndef AUTOREPEAT
      case KeyRelease:
	{
	    static char buffer[20];
	    KeySym  key;
	    XComposeStatus compose;
	    int     count, j;
#ifndef PET	    
	    if( handle_joy_emu( report ) )
		break;
#endif	    
	    count = XLookupString((XKeyEvent *) report,
				  buffer, 20, &key, &compose);
	    for (j = 0; j < CONV_KEYS; j++)
		if (key == keyconvmap[j].sym) {
		    keypressed = j;
		    break;
		}
	    if (keypressed >= 0) {
		if (!(--keys_down)) {

#ifdef KBDBUG
		    printf("release: no keys left\n");
#endif

		    for (j = 0; j < KBD_ROWS; j++)
			keyarr[j] = 0;
		    keypressed = -1;
#ifdef PET
		    kbd_shiftflag=0;
#endif
		} else {
		    /* do not use "XOR" here... */
		    keyarr[keyconvmap[keypressed].row] &=
			~keyconvmap[keypressed].keybit;
		/* keyarr[6] &= ~keyconvmap[keypressed].shift; */
#ifdef PET
		    kbd_shiftflag &= ~keyconvmap[keypressed].shift;
		    /* test if keyrelease removed all (left) shift keys */
		    if((kbd_shiftflag & 3) == 0) 
		      keyarr[kbd_lshiftrow] &= ~kbd_lshiftbit;
#endif
		}
		if (debugflg)
		    printf("KEY RELEASED %d\n", keypressed);
	    }
	    if (debugflg)
		printf("count %d Key %d %lx\n", count, (int) *buffer, key);
	    break;
	}
#endif  /* AUTOREPEAT */

#else  /* LATCHING */
      case KeyRelease:
	{
	    static char buffer[20];
	    KeySym  key;
	    XComposeStatus compose;
	    kbq     *kptr;
	    int     count, i;
#ifndef PET	    
	    if( handle_joy_emu( report ) )
		break;
#endif	    
	    count = XLookupString((XKeyEvent *) report, buffer,
				  20, &key, &compose);

#ifdef AUTOREPEAT
	    /* With autorepeat, process only modifiers released. */ 
	    if (key != XK_Shift_L && key != XK_Shift_R &&
		key != XK_Control_L && key != XK_Return)
		break;
#endif /* repeat */

#ifdef KBDBUG
	    printf("release %d Key %d %lx\n", count, (int) *buffer, key);
#endif
	    /*
	     * search for (last occurrence of) current key
	     * and set a flag it's released. The entry will be
	     * cleared by x_get_keyarr once it's read.
	     */

	    for (i = keys_down, kptr = &keyqueue[i]; i >= 0; --i, --kptr)
		if (key == kptr->key) {
		    kptr->rtime = report -> xkey.time;
		    kptr->ack &= ~ACK_DOWN;
		    ++kbd_changed;
#ifndef KBDBUG
		    if (debugflg)
#endif
			printf("KEY UP    [%d/%d] %ld\n",
			       i, keys_down, report -> xkey.time);
		    /*break;*/
		}

	    break;
	}
#endif  /* LATCHING */


#ifndef AUTOREPEAT
      case FocusIn:
      case EnterNotify:

	XAutoRepeatOff(display);
	break;
#endif /* repeat */


      case FocusOut:
      case LeaveNotify:
	{
	    register int i;

	    /*
	     * Entering to the screen subwindow triggers this too, thus,
	     * it must be filtered out.
	     */

	    if (!report -> xcrossing.subwindow) {
		XAutoRepeatOn(display);
#ifdef KBDBUG
		printf("no focus\n");
#endif

	    /* this is just to recover from server's panic
	     * (may be obsolete already)
	     */
#ifdef LATCHING
		for (i = 0; i < MAXDOWN; i++) {
		    keyqueue[i].key = 0;
		    keyqueue[i].ack = 0;
		}
#else
#ifdef AUTOREPEAT
		for (i = 0; i < CONV_KEYS; i++)
		    keydelay[i] = 0;	/* clear kbd */
#endif
#endif  /* LATCHING */

		for (i = 0; i < KBD_ROWS; i++)
		    keyarr[i] = 0;
		keys_down = 0;
	    }
	}
	break;

      case ClientMessage:
	/* flush_queue(); */
	if (*(report->xclient.data.l) == WM_DelWin && report->xclient.format == 32)
	  end64();
	break;

      default:
	/*
	 * All events selected by StructureNotifyMask except ConfigureNotify
	 * are thrown away here, since nothing is done with them
	 */
	break;
    }	       /* End switch */
}


/* ------------------------------------------------------------------------- */

int     x_get_keyarr(void)
{
    /* This routine is called once for each IRQ. */

#ifdef LATCHING

    KeySym  key;
    int     i,j;

/*    if (kbd_changed || (update >0 && !--update)) {
	kbd_changed = 0;
*/

	/*
	 * Clean up entries that have become obsolete. Since the emulator
	 * can get lagged occasionally, virtual timing is used here.
	 * All entries will expire after some amout of CPU cycles disregarding
	 * whether they have actually been read or not.
	 */

	for (i = j = 0; i < keys_down; i++) {
	    if (keyqueue[j].ack & (ACK_DOWN | ACK_UNREAD))
		j++;
	    if (i > j) {
		if (keyqueue[j].ack & (ACK_DOWN | ACK_UNREAD))
		{
		    /* memmov (keyqueue[j], keyqueue[i], sizeof(kbq));*/

		    keyqueue[j].key = keyqueue[i].key;
		    keyqueue[j].ack = keyqueue[i].ack;
		    keyqueue[j].ptime= keyqueue[i].ptime;
		    keyqueue[j].rtime= keyqueue[i].rtime;
		    keyqueue[j].cycles= keyqueue[i].cycles;
#ifdef KBDBUG
		    printf("Que %d -> %d\n", i, j);
#endif
		}
		else {
		    keyqueue[i].key = 0;
		    keyqueue[i].ack = 0;
#ifdef KBDBUG
		    printf("Remove [%d/%d] %02x\n", i, keys_down, keyqueue[i].key);
#endif
		}
	    }
	} /* for */

#if 0
    if (j)
	update = 10;		/* re-check after delay */
#endif

    keys_down = j;


#ifdef KBDBUG2
	printf("queue (-%d) %d \n", i-j, keys_down);
	for (i = 0; i < MAXDOWN; i++)
	    printf("%02x%c  ", keyqueue[i].key, keyqueue[i].ack + '@');
	printf("\n");
#endif

	/*
	 * re-build keyboard matrix
	 */

	for (i = 0; i < KBD_ROWS; i++)
	    keyarr[i] = 0;


	/* read all simultaneously pressed keys from keypress event buffer */

	for (j = 0; j < keys_down && keyqueue[j].ack &&
	     keyqueue[j].ptime <= keyqueue[0].rtime; j++) {

	    key = keyqueue[j].key;
	    keyqueue[j].ack &=  ~ACK_UNREAD;

#ifdef KBDBUG
	    if (j > 0)
		printf("Que %d\n", j);
#endif
	    for (i = 0; i < CONV_KEYS; i++) {
		if (key == keyconvmap[i].sym) {
		    keyarr[keyconvmap[i].row] |= keyconvmap[i].keybit;

#ifndef PET
		    if (keyconvmap[i].shift == 16)
			keyarr[6] |= 16;  /* add right shift */
		    else
			keyarr[6] &= ~16;
#else
		    if (keyconvmap[i].shift)
		      keyarr[8] &= ~keyconvmap[keypressed].shift;
		    else
		      keyarr[8] |= keyconvmap[keypresses].shift;
#endif  /* PET */

#ifdef KBDBUG
		    printf("Keyarr: %02X\n", key);
#endif
		    break;  /* match found, exit loop */
		}
	    }  /* for */
	}  /* keys */

#ifdef KBDBUG
	/*if (debugflg)
	  show_keyarr();*/

	printf("%2d keys: %02X %02X %02X %02X  %02X %02X %02X %02X\n",
	       keys_down, keyarr[0], keyarr[1], keyarr[2], keyarr[3],
	       keyarr[4], keyarr[5], keyarr[6], keyarr[7]);
#endif


/*    }*/ /* changed */

  return (kbd_changed);
}


#else   /* LATCHING -------------------------------------------------------- */


#ifdef AUTOREPEAT
    register int i, j;

    for (i = 0; keyconvmap[i].row; i++)
	if (keydelay[i] > 0 && keydelay[i] < (clk)) {
	    keypressed = i;
	    /* do not use "XOR" here... */
	    keyarr[keyconvmap[keypressed].row] &= ~keyconvmap[keypressed].keybit;
#ifndef PET
	    keyarr[6] &= ~keyconvmap[keypressed].shift;
#else
#ifdef PET8
#else		/* PET 3032/4032 */
	    keyarr[8] &= ~keyconvmap[keypressed].shift;
#endif
#endif  /* PET */

	    keys_down--;
	    keydelay[i] = 0;

	    if (debugflg)
		printf("KEY RELEASED %d (%d)\n", keypressed, clk);

#ifdef KBDBUG
	    dbg_up++;
	    printf("left: %d\n", dbg_dn - dbg_up);
#endif
	}

#ifdef KBDBUG
    printf("%2d keys: %3d %2d %2d %2d  %2d %2d %2d %2d\n", keys_down,
	   keyarr[0], keyarr[1], keyarr[2], keyarr[3],
	   keyarr[4], keyarr[5], keyarr[6], keyarr[7]);
#endif
#endif  /* AUTOREPEAT */


    return (kbd_changed);
}
#endif  /* LATCHING */
