/*
 * $Id: x64.h,v 1.9 1995/11/07 16:51:00 jopi Exp jopi $
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice
 *
 *
 * Written by
 *   Jarkko Sonninen  (sonninen@lut.fi)
 *   Ettore Perazzoli (ettore@comm2000.it)
 *
 */


#ifndef X64_H
#define X64_H

#include <X11/Xlib.h>
#include "types.h"

extern int redraw;

extern Display *display;
extern int     screen;
extern Visual *visual;
extern int     depth;

typedef struct {
    KeySym  sym;
    BYTE    row;
    BYTE    keybit;
    short   shift;
}       keyconv;


/*
 * Functions
 */

extern Window open_screen_win ( Window borderwin, int xpix, int ypix, int bg, int border );
extern void open_sprite_win (  Window borderwin, Window win );
extern void handle_mainwin_event ( XEvent *report );


/* control window */
#if (defined(XDEBUGGER) || defined(XCONTROL))

extern void  debug_main ( void );
extern void  toggle_cntl_win ( void );
#endif

#define	VIC20_WIN_TITLE		"VIC20 Emulator"
#define VIC20_RES_CLASS	 	"VIC20"
#define VIC20_RES_NAME		"xvic"

#define PET_WIN_TITLE		"CBM PET Emulator"
#define PET_RES_CLASS   	"PET"
#define PET_RES_NAME		"xpet"

#define C128_VDC_WIN_TITLE   	"C128 Emulator (VDC)"
#define C128_VIC_WIN_TITLE	"C128 Emulator (VIC)"
#define C128_VDC_RES_CLASS	"C128 80c"
#define C128_VIC_RES_CLASS	"C128 40c"
#define	C128_RES_NAME		"x128"

#define	C64_WIN_TITLE		"C64 Emulator"
#define C64_RES_CLASS		"C64"
#define C64_RES_NAME		"x64"

#define VDC_WIN_TITLE		C128_VDC_WIN_TITLE

#ifdef C128
#define VIC_II_WIN_TITLE	C128_VIC_WIN_TITLE
#else
#define VIC_II_WIN_TITLE	C64_WIN_TITLE
#endif

#define VIC_I_WIN_TITLE		VIC20_WIN_TITLE
#define CRTC_WIN_TITLE		PET_WIN_TITLE

#if defined( C128 )
#include "vicii.h"
#include "vdc.h"
#elif defined( VIC20 )
#include "vic.h"
#elif defined( PET )
#include "crtc.h"
#else
#include "vicii.h"
#endif

#if defined( VIC20)

#if defined( _X64_C)
#include "vic_icon.xpm"
#endif

#define RES_NAME     	VIC20_RES_NAME
#define RES_CLASS    	VIC20_RES_CLASS
#define WIN_TITLE    	VIC20_WIN_TITLE
#define INIT_VIDEO      { borderwin = Vic_Init(display, screen, WIN_TITLE, WIN_TITLE, RES_CLASS,\
					       RES_NAME, &width, &height, icon_data, &cmap,\
					       speed, 0, refrate, &WM_DelWin); }
#define PAUSE_VIDEO	Vic_Pause ()
#define RESTART_VIDEO	Vic_Restart ()
#define ARRANGE_WINDOW  Vic_ArrangeWindow( width, height )
#define FREE_VIDEO      Vic_Free()

#elif defined( PET)

#if defined( _X64_C)
#include "pet_icon.xpm"
#endif

#define RES_NAME     	PET_RES_NAME
#define RES_CLASS    	PET_RES_CLASS
#define WIN_TITLE   	PET_WIN_TITLE
#define INIT_VIDEO      { borderwin = Crtc_Init(display, screen, WIN_TITLE, WIN_TITLE, RES_CLASS,\
						RES_NAME, &width, &height, icon_data, &cmap,\
						speed, 0, refrate, &WM_DelWin); }
#define PAUSE_VIDEO	Crtc_Pause ()
#define RESTART_VIDEO	Crtc_Restart ()
#define ARRANGE_WINDOW  Crtc_ArrangeWindow( width, height )
#define FREE_VIDEO      Crtc_Free()

#elif defined( C128)

#define RES_NAME     	C128_RES_NAME 
#define VDC_RES_CLASS	C128_VDC_RES_CLASS
#define VIC_RES_CLASS   C128_VIC_RES_CLASS
#define VDC_WIN_TITLE	C128_VDC_WIN_TITLE
#define VIC_WIN_TITLE	C128_VIC_WIN_TITLE

#if defined( _X64_C)
#include "c128_icon.xpm"
#endif

#if defined( USE_VDC)
#define INIT_VIDEO      { VDCborderwin = \
			    Vdc_Init(display, screen, VDC_WIN_TITLE, VDC_WIN_TITLE,\
				    VDC_RES_CLASS, RES_NAME, &vdcwidth, &vdcheight, icon_data,\
				    &cmap, 0, !video80, vdcrefrate, &WM_DelWin); \
			  borderwin =\
			    VicII_Init(display, screen, VIC_WIN_TITLE, VIC_WIN_TITLE, \
			              VIC_RES_CLASS, RES_NAME, &width, &height, icon_data, &cmap,\
				      speed, !video40, vicrefrate, &WM_DelWin);\
			  VicII_SetStatus (video40); Vdc_SetStatus (video80); }
#define PAUSE_VIDEO	{ VicII_Pause (); Vdc_Pause (); }
#define RESTART_VIDEO   { VicII_Restart (); Vdc_Restart (); }
#define ARRANGE_WINDOW  { VicII_ArrangeWindow( width, height );\
			  Vdc_ArrangeWindow( vdcwidth, vdcheight ); }
#define FREE_VIDEO      { VicII_Free(); Vdc_Free(); }
#else
#define INIT_VIDEO      borderwin = VicII_Init(display, screen, VIC_WIN_TITLE, VIC_WIN_TITLE, \
					       VIC_RES_CLASS, RES_NAME, &width, &height,\
					       icon_data, &cmap, speed, !video40, vicrefrate,\
					       &WM_DelWin);
#define PAUSE_VIDEO     VicII_Pause ()
#define RESTART_VIDEO   VicII_Restart ()
#define ARRANGE_WINDOW  VicII_ArrangeWindow( width, height )
#define FREE_VIDEO      VicII_Free()

#endif /* USE_VDC */

#else  /* defined( CBM64) */

#if defined( _X64_C) 
#include "c64_icon.xpm"
#endif

#define RES_NAME   	C64_RES_NAME  
#define RES_CLASS   	C64_RES_CLASS
#define WIN_TITLE  	C64_WIN_TITLE
#define INIT_VIDEO      { borderwin = VicII_Init(display, screen, WIN_TITLE, WIN_TITLE, RES_CLASS,\
						 RES_NAME, &width, &height, icon_data, &cmap,\
						 speed, 0, refrate, &WM_DelWin); }
#define PAUSE_VIDEO     VicII_Pause ()
#define RESTART_VIDEO   VicII_Restart ()
#define ARRANGE_WINDOW  VicII_ArrangeWindow( width, height )
#define FREE_VIDEO      VicII_Free()

#endif

#endif  /* X64_H */
