/** fMSX: portable MSX emulator ******************************/
/**                                                         **/
/**                          Lame.h                         **/
/**                                                         **/
/** This file contains simplified version of Z80 emulation  **/
/** which can be used if your compiler can't handle the     **/
/** original version. Simplified code is going to be        **/
/** slightly slower, although optimization may improve its  **/
/** perfomance. In this version, M_WRMEM() is implemented   **/
/** as a function [instead of a macro] and each subset of   **/
/** Z80 commands is also made into a separate function      **/
/** called when appropriate prefix appears.                 **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin 1994,1995                 **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/

/* 250196 - Modified to suit the Commodore 128.  --jopi@stekt */
/* 250196 - Changed memory reading to a macro. */


void CodesCB(void)
{
  register byte I;

  switch(M_I_RDMEM(R.PC.W))
  {
#include "CodesCB.h"
    default:
      if(TrapBadOps)
        printf("Unrecognized CB instruction at PC=%hX\n",R.PC.W-2);
  }
}

void CodesDDCB(register pair J)
{
  register byte I;

#define XX IX    
  switch(M_I_RDMEM(R.PC.W))
  {
#include "CodesXCB.h"
    default:
      if(TrapBadOps)
        printf("Unrecognized DD CB instruction at PC=%hX\n",R.PC.W-3);
  }
#undef XX
}

void CodesFDCB(register pair J)
{
  register byte I;

#define XX IY    
  switch(M_I_RDMEM(R.PC.W))
  {
#include "CodesXCB.h"
    default:
      if(TrapBadOps)
        printf("Unrecognized FD CB instruction at PC=%hX\n",R.PC.W-3);
  }
#undef XX
}

void CodesED(void)
{
  register byte I;
  register pair J;

  switch(M_I_RDMEM(R.PC.W))
  {
#include "CodesED.h"
    default:
      if(TrapBadOps)
        printf
        (
          "Unrecognized instruction: ED %X at PC=%hX\n",
         M_RDMEM(R.PC.W-1), R.PC.W-2 );
  }
}

void CodesDD(void)
{
  register byte I;
  register pair J;

#define XX IX
  switch(M_I_RDMEM(R.PC.W))
  {
#include "CodesXX.h"
    case PFX_FD:
    case PFX_DD:
      R.PC.W--;break;
    case PFX_CB:
      J.W=R.XX.W + M_I_RDMEM(R.PC.W);
      CodesDDCB(J);break;
    case HALT:
#ifdef INTERRUPTS
      if(R.IFF&0x01) { if(Cnt) R.PC.W--; }
      else
#endif   
      { 
        printf("CPU HALTed and stuck at PC=%hX\n",R.PC.W-=2);
        CPURunning=0;
      }
      break;
    default:
      if(TrapBadOps)
        printf("Unrecognized DD instruction at PC=%hX\n",R.PC.W-2);
  }
#undef XX
}

void CodesFD(void)
{
  register byte I;
  register pair J;

#define XX IY
  switch(M_I_RDMEM(R.PC.W))
  {
#include "CodesXX.h"
    case PFX_FD:
    case PFX_DD:
      R.PC.W--;break;
    case PFX_CB:
      J.W=R.XX.W+M_I_RDMEM(R.PC.W);
      CodesFDCB(J);break;
    case HALT:
#ifdef INTERRUPTS
      if(R.IFF&0x01) { if(Cnt) R.PC.W--; }
      else
#endif
      {           
        printf("CPU HALTed and stuck at PC=%hX\n",R.PC.W-=2);
        CPURunning=0;
      }
      break;
    default:
      if(TrapBadOps)
        printf("Unrecognized FD instruction at PC=%hX\n",R.PC.W-2);
  }
#undef XX
}

word Z80(register byte *A,reg Regs)
{
  register byte I;
  register pair J;

  R=Regs;Addr=A;

#ifdef INTERRUPTS
  Cnt=IPeriod;
#endif
  
  for(CPURunning=1;CPURunning;)
  {
    switch(M_I_RDMEM(R.PC.W))
    {
#include "Codes.h"
      case PFX_CB: CodesCB();break;
      case PFX_ED: CodesED();break;
      case PFX_FD: CodesFD();break;
      case PFX_DD: CodesDD();break;
      case HALT: 
#ifdef INTERRUPTS
        if(R.IFF&0x01) { if(Cnt) R.PC.W--; }
        else
#endif   
        {
          printf("CPU HALTed and stuck at PC=%hX\n",--R.PC.W);
          CPURunning=0;
        }
        break;
      default:
        if(TrapBadOps)
          printf("Unrecognized instruction at PC=%hX\n",R.PC.W-1);
    }
 
#ifdef INTERRUPTS
    if(!Cnt--) 
    {
      Cnt=IPeriod;
      if(IntSync) IFlag=1;
    }
    if(IFlag)
    {
      IFlag=0;J.W=Interrupt();
      if((J.W!=0xFFFF)&&(R.IFF&0x01))
      { M_PUSH(PC);R.PC.W=R.IFF&0x04? ((word)R.I<<8)+0xFF:J.W; }
    }  
#endif

#ifdef DEBUG_Z80
    if(R.PC.W==Trap) Trace=1;  /*** Turn tracing on if trapped ***/
    if(Trace) Debug(Addr,R);   /*** Call single-step debugger  ***/
#endif

  }
  return(R.PC.W);
}
